@@ -1,6 +1,6 @@
 ALTER TABLE `sccpdevice` ADD `softkeyset` VARCHAR( 50 ) NULL DEFAULT NULL AFTER `addon`;
 ALTER TABLE `sccpdevice` ADD `pickupcontext` VARCHAR( 100 ) NULL DEFAULT "" AFTER `pickupexten`;
-ALTER TABLE `sccpdevice` ADD `nat` VARCHAR( 5 ) NULL DEFAULT "no" BEFORE `name`;
+ALTER TABLE `sccpdevice` CHANGE `nat` VARCHAR( 4 ) DEFAULT "auto";
 ALTER TABLE `sccpdevice` CHANGE COLUMN `earlyrtp` VARCHAR(10);
 ALTER TABLE `sccpdevice` ADD `audio_tos` VARCHAR( 11 ) NULL DEFAULT "0xB8" AFTER `softkeyset`;
 ALTER TABLE `sccpdevice` ADD `audio_cos` VARCHAR( 1 ) NULL DEFAULT "6" AFTER `audio_tos`;
@@ -15,7 +15,7 @@ ALTER TABLE `sccpdevice` DROP COLUMN `trustphoneip`;
 ALTER TABLE `sccpdevice` DROP COLUMN `dtmfmode`;
 
 ALTER TABLE `sccpline` CHANGE COLUMN `dnd` `dndFeature`;
-ALTER TABLE `sccpline` ADD `dnd` VARCHAR( 5 ) DEFAULT "on" AFTER `amaflags`;
+ALTER TABLE `sccpline` ADD `dnd` VARCHAR( 7 ) DEFAULT "reject" AFTER `amaflags`;
 ALTER TABLE `sccpline` DROP COLUMN `rtptos`;
 ALTER TABLE `sccpline` DROP COLUMN `audio_tos`;
 ALTER TABLE `sccpline` DROP COLUMN `audio_cos`;
