@@ -13,8 +13,8 @@
  * When to use: Methods communicating to asterisk about module initialization, status, destruction
  * Relations:   Main hub for all other sourcefiles.
  *
- * $Date: 2014-11-09 11:27:58 +0100 (Sun, 09 Nov 2014) $
- * $Revision: 5797 $
+ * $Date: 2015-04-19 20:49:40 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6026 $
  */
 
 #include <config.h>
@@ -37,7 +37,6 @@
 #include "sccp_conference.h"
 #include "sccp_labels.h"
 #include "sccp_softkeys.h"
-#include "sccp_adv_features.h"
 #include "sccp_cli.h"
 #include "sccp_appfunctions.h"
 #include "sccp_management.h"
@@ -47,15 +46,11 @@
 #include "revision.h"
 #include <signal.h>
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5797 $")
-#define ENUMMACRO_FILE "chan_sccp_enums.hh"
-#define ENUMMACRO_GENERATE ENUMMACRO_GENFUNCTION
-#include "sccp_enum_macro.h"
-#undef  ENUMMACRO_GENERATE
-#undef  ENUMMACRO_FILE
-    /*!
-     * \brief       Buffer for Jitterbuffer use
-     */
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6026 $");
+
+/*!
+ * \brief       Buffer for Jitterbuffer use
+ */
 #if defined(__cplusplus) || defined(c_plusplus)
 static ast_jb_conf default_jbconf = {
 flags:	0,
@@ -231,13 +226,12 @@ EXIT:
  *
  * Used to map SKinny Message Id's to their Handling Implementations
  */
-struct sccp_messageMap_cb {
+struct messageMap_cb {
 	void (*const messageHandler_cb) (sccp_session_t * s, sccp_device_t * d, sccp_msg_t * msg);
 	boolean_t deviceIsNecessary;
 };
-typedef struct sccp_messageMap_cb sccp_messageMap_cb_t;
 
-static const struct sccp_messageMap_cb messagesCbMap[] = {
+static const struct messageMap_cb sccpMessagesCbMap[] = {
 	[KeepAliveMessage] = {sccp_handle_KeepAliveMessage, FALSE},						// on 7985,6911 phones and tokenmsg, a KeepAliveMessage is send before register/token 
 	[OffHookMessage] = {sccp_handle_offhook, TRUE},
 	[OnHookMessage] = {sccp_handle_onhook, TRUE},
@@ -271,7 +265,6 @@ static const struct sccp_messageMap_cb messagesCbMap[] = {
 	[MediaPathCapabilityMessage] = {sccp_handle_unknown_message, TRUE},
 	[DisplayDynamicNotifyMessage] = {sccp_handle_unknown_message, TRUE},
 	[DisplayDynamicPriNotifyMessage] = {sccp_handle_unknown_message, TRUE},
-	[SpeedDialStatDynamicMessage] = {sccp_handle_speed_dial_stat_req, TRUE},
 	[ExtensionDeviceCaps] = {sccp_handle_unknown_message, TRUE},
 	[DeviceToUserDataVersion1Message] = {sccp_handle_device_to_user, TRUE},
 	[DeviceToUserDataResponseVersion1Message] = {sccp_handle_device_to_user_response, TRUE},
@@ -280,12 +273,14 @@ static const struct sccp_messageMap_cb messagesCbMap[] = {
 	[RegisterMessage] = {sccp_handle_register, FALSE},
 	[AlarmMessage] = {sccp_handle_alarm, FALSE},
 	[XMLAlarmMessage] = {sccp_handle_XMLAlarmMessage, FALSE},
-	[SPCPRegisterTokenRequest] = {sccp_handle_SPCPTokenReq, FALSE},
 	[StartMultiMediaTransmissionAck] = {sccp_handle_startmultimediatransmission_ack, TRUE},
 	[MediaTransmissionFailure] = {sccp_handle_mediatransmissionfailure, TRUE},
 	[MiscellaneousCommandMessage] = {sccp_handle_miscellaneousCommandMessage, TRUE},
 };
 
+static const struct messageMap_cb spcpMessagesCbMap[] = {
+	[SPCPRegisterTokenRequest - SPCP_MESSAGE_OFFSET] = {sccp_handle_SPCPTokenReq, FALSE},
+};
 /*!
  * \brief       Controller function to handle Received Messages
  * \param       msg Message as sccp_msg_t
@@ -293,7 +288,7 @@ static const struct sccp_messageMap_cb messagesCbMap[] = {
  */
 int sccp_handle_message(sccp_msg_t * msg, sccp_session_t * s)
 {
-	const sccp_messageMap_cb_t *messageMap_cb = NULL;
+	const struct messageMap_cb *messageMap_cb = NULL;
 	uint32_t mid = 0;
 	sccp_device_t *device = NULL;
 
@@ -313,7 +308,11 @@ int sccp_handle_message(sccp_msg_t * msg, sccp_session_t * s)
 	mid = letohl(msg->header.lel_messageId);
 
 	/* search for message handler */
-	messageMap_cb = &messagesCbMap[mid];
+	if (mid >= SPCP_MESSAGE_OFFSET) {
+		messageMap_cb = &spcpMessagesCbMap[mid - SPCP_MESSAGE_OFFSET]; 
+	} else {
+		messageMap_cb = &sccpMessagesCbMap[mid];
+	}
 	sccp_log((DEBUGCAT_MESSAGE)) (VERBOSE_PREFIX_3 "%s: >> Got message %s (%x)\n", DEV_ID_LOG(s->device), msgtype2str(mid), mid);
 
 	/* we dont know how to handle event */
@@ -457,6 +456,8 @@ int load_config(void)
 			}
 			ast_verbose(VERBOSE_PREFIX_3 "SCCP channel driver up and running on %s:%d\n", addrStr, sccp_socket_getPort(&GLOB(bindaddr)));
 
+			sccp_socket_setoptions(GLOB(descriptor));
+			
 			if (listen(GLOB(descriptor), DEFAULT_SCCP_BACKLOG)) {
 				pbx_log(LOG_WARNING, "Failed to start listening to %s:%d: %s\n", addrStr, sccp_socket_getPort(&GLOB(bindaddr)), strerror(errno));
 				close(GLOB(descriptor));
@@ -534,9 +535,9 @@ boolean_t sccp_prePBXLoad(void)
 
 	GLOB(externrefresh) = 60;
 	GLOB(keepalive) = SCCP_KEEPALIVE;
-	sccp_copy_string(GLOB(dateformat), "D/M/YA", sizeof(GLOB(dateformat)));
-	sccp_copy_string(GLOB(context), "default", sizeof(GLOB(context)));
-	sccp_copy_string(GLOB(servername), "Asterisk", sizeof(GLOB(servername)));
+	//sccp_copy_string(GLOB(dateformat), "M/D/YA", sizeof(GLOB(dateformat)));
+	//sccp_copy_string(GLOB(context), "default", sizeof(GLOB(context)));
+	//sccp_copy_string(GLOB(servername), "Asterisk", sizeof(GLOB(servername)));
 
 	/* Wait up to 16 seconds for first digit */
 	GLOB(firstdigittimeout) = 16;
@@ -583,6 +584,9 @@ boolean_t sccp_postPBX_load(void)
 #endif
 	sprintf(SCCP_VERSIONSTR, "Skinny Client Control Protocol (SCCP). Release: %s %s - %s (built by '%s' on '%s')\n", SCCP_VERSION, SCCP_BRANCH, SCCP_REVISIONSTR, BUILD_USER, BUILD_DATE);
 
+#if CS_TEST_FRAMEWORK
+	sccp_utils_register_tests();
+#endif
 	GLOB(module_running) = TRUE;
 	sccp_refcount_schedule_cleanup((const void *) 0);
 	pbx_mutex_unlock(&GLOB(lock));
@@ -620,6 +624,10 @@ int sccp_preUnload(void)
 
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "SCCP: Unloading Module\n");
 
+#if CS_TEST_FRAMEWORK
+	sccp_utils_unregister_tests();
+#endif
+
 	sccp_event_unsubscribe(SCCP_EVENT_FEATURE_CHANGED, sccp_device_featureChangedDisplay);
 	sccp_event_unsubscribe(SCCP_EVENT_FEATURE_CHANGED, sccp_util_featureStorageBackend);
 
@@ -655,7 +663,6 @@ int sccp_preUnload(void)
 		sccp_line_clean(l, TRUE);
 	}
 	SCCP_RWLIST_TRAVERSE_SAFE_END;
-	SCCP_RWLIST_UNLOCK(&GLOB(lines));
 	if (SCCP_RWLIST_EMPTY(&GLOB(lines))) {
 		SCCP_RWLIST_HEAD_DESTROY(&GLOB(lines));
 	}
@@ -732,8 +739,8 @@ int sccp_reload(void)
 	pbx_mutex_lock(&GLOB(lock));
 	if (GLOB(reload_in_progress) == TRUE) {
 		pbx_log(LOG_ERROR, "SCCP reloading already in progress.\n");
-		pbx_mutex_unlock(&GLOB(lock));
-		return 1;
+		returnval = 1;
+		goto EXIT;
 	}
 
 	sccp_config_file_status_t cfg = sccp_config_getConfig(FALSE);
@@ -747,16 +754,12 @@ int sccp_reload(void)
 			pbx_log(LOG_NOTICE, "SCCP reloading configuration.\n");
 			readingtype = SCCP_CONFIG_READRELOAD;
 			GLOB(reload_in_progress) = TRUE;
-			pbx_mutex_unlock(&GLOB(lock));
 			if (!sccp_config_general(readingtype)) {
 				pbx_log(LOG_ERROR, "Unable to reload configuration.\n");
-				GLOB(reload_in_progress) = FALSE;
-				pbx_mutex_unlock(&GLOB(lock));
-				return 2;
+				returnval = 2;
+				break;
 			}
 			sccp_config_readDevicesLines(readingtype);
-			pbx_mutex_lock(&GLOB(lock));
-			GLOB(reload_in_progress) = FALSE;
 			returnval = 3;
 			break;
 		case CONFIG_STATUS_FILE_OLD:
@@ -780,6 +783,8 @@ int sccp_reload(void)
 			returnval = 4;
 			break;
 	}
+EXIT:
+	GLOB(reload_in_progress) = FALSE;
 	pbx_mutex_unlock(&GLOB(lock));
 	return returnval;
 }
