@@ -13,8 +13,8 @@
  *              See the LICENSE file at the top of the source tree.
  * \warning     File has been Lined up using 8 Space TABS
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-19 13:12:47 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6022 $
  */
 
 #ifndef __CHAN_SCCP_H
@@ -34,12 +34,6 @@ extern "C" {
 #include <config.h>
 //#include "common.h"
 
-#define ENUMMACRO_FILE "chan_sccp_enums.hh"
-#define ENUMMACRO_GENERATE ENUMMACRO_GENHEADER
-#include "sccp_enum_macro.h"
-#undef  ENUMMACRO_GENERATE
-#undef  ENUMMACRO_FILE
-
 #define sccp_mutex_t ast_mutex_t
 
 /* Add bswap function if necessary */
@@ -114,7 +108,6 @@ char SCCP_REVISIONSTR[30];
 #define SCCP_FIRST_SERVICEINSTANCE 1										/* whats the instance of the first service button */
 #define SCCP_FIRST_SPEEDDIALINSTANCE 1										/* whats the instance of the first speeddial button */
 
-#define SCCP_SOCKET_ACCEPT_TIMEOUT 2000
 #define SCCP_DISPLAYSTATUS_TIMEOUT 5
 
 /* Simulated Enbloc Dialing */
@@ -187,16 +180,6 @@ extern struct sccp_pbx_cb sccp_pbx;
 
 #define CS_AST_CHANNEL_PVT_IS_SCCP(x) CS_AST_CHANNEL_PVT_CMP_TYPE(x,"SCCP")
 
-#ifdef CS_AST_HAS_CHANNEL_BRIDGE_PEER
-#define CS_AST_BRIDGED_CHANNEL(x) ast_channel_bridge_peer(x)
-#else
-#ifdef CS_AST_HAS_BRIDGED_CHANNEL
-#define CS_AST_BRIDGED_CHANNEL(x) ast_bridged_channel(x)
-#else
-#define CS_AST_BRIDGED_CHANNEL(x) x->bridge
-#endif
-#endif
-
 #ifdef AST_MAX_EXTENSION
 #define SCCP_MAX_EXTENSION AST_MAX_EXTENSION
 #else
@@ -266,100 +249,11 @@ typedef enum { FALSE = 0, TRUE = 1 } boolean_t;									/*!< Asterisk Reverses T
 #define TRUE B_TRUE
 #endif
 typedef void sk_func(sccp_device_t * d, sccp_line_t * l, sccp_channel_t * c);
-typedef enum { PARK_RESULT_FAIL = 0, PARK_RESULT_SUCCESS = 1 } sccp_parkresult_t;				/*!< Enum Park Result */
-typedef enum {
-	/* *INDENT-OFF* */
-	CALLERID_PRESENCE_FORBIDDEN 	= 0,
-	CALLERID_PRESENCE_ALLOWED 	= 1
-	/* *INDENT-ON* */
-} sccp_calleridpresence_t;											/*!< Enum Caller Id Presence */
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_RTP_STATUS_INACTIVE 	= 0,
-	SCCP_RTP_STATUS_PROGRESS 	= 1 << 0,
-	SCCP_RTP_STATUS_ACTIVE		= 1 << 1,
-	/* *INDENT-ON* */
-} sccp_rtp_status_t;												/*!< RTP status information */
 
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_EXTENSION_NOTEXISTS 	= 0,
-	SCCP_EXTENSION_MATCHMORE 	= 1,
-	SCCP_EXTENSION_EXACTMATCH 	= 2,
-	/* *INDENT-ON* */
-} sccp_extension_status_t;											/*!< extension status information */
-
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_REQUEST_STATUS_ERROR 	= 0,
-	SCCP_REQUEST_STATUS_LINEUNKNOWN = 1,
-	SCCP_REQUEST_STATUS_LINEUNAVAIL = 2,
-	SCCP_REQUEST_STATUS_SUCCESS 	= 3,
-	/* *INDENT-ON* */
-} sccp_channel_request_status_t;										/*!< channel request status */
-
-typedef enum {
-	SCCP_MESSAGE_PRIORITY_IDLE = 0,
-	SCCP_MESSAGE_PRIORITY_VOICEMAIL,
-	SCCP_MESSAGE_PRIORITY_MONITOR,
-	SCCP_MESSAGE_PRIORITY_PRIVACY,
-	SCCP_MESSAGE_PRIORITY_DND,
-	SCCP_MESSAGE_PRIORITY_CFWD,
-} sccp_message_priority_t;
-
-typedef enum {
-	SCCP_PUSH_RESULT_FAIL,
-	SCCP_PUSH_RESULT_NOT_SUPPORTED,
-	SCCP_PUSH_RESULT_SUCCESS,
-} sccp_push_result_t;
-
-typedef enum {
-	SCCP_TOKEN_STATE_NOTOKEN,
-	SCCP_TOKEN_STATE_ACK,
-	SCCP_TOKEN_STATE_REJ,
-} sccp_tokenstate_t;
-
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_SS_DIAL 			= 0,
-	SCCP_SS_GETFORWARDEXTEN 	= 1,
-	SCCP_SS_GETPICKUPEXTEN 		= 2,
-	SCCP_SS_GETMEETMEROOM 		= 3,
-	SCCP_SS_GETBARGEEXTEN 		= 4,
-	SCCP_SS_GETCBARGEROOM 		= 5,
-#ifdef CS_SCCP_CONFERENCE
-	SCCP_SS_GETCONFERENCEROOM 	= 6,
-#endif
-} sccp_softswitch_action_t;
-
-typedef enum { 
-	SCCP_PHONEBOOK_NONE		= 0, 
-	SCCP_PHONEBOOK_MISSED		= 1, 
-	SCCP_PHONEBOOK_RECEIVED		= 2, 
-	//SCCP_PHONEBOOK_PLACED = 3
-} sccp_phonebook_t;
-
-#include "sccp_protocol.h"
 #ifdef HAVE_ASTERISK
 #include "pbx_impl/ast/ast.h"
 #endif
 
-/*!
- * \brief SCCP ButtonType Structure
- */
-static const struct sccp_buttontype {
-	sccp_config_buttontype_t buttontype;
-	const char *const text;
-} sccp_buttontypes[] = {
-        /* *INDENT-OFF* */
-	{LINE, 				"LINE"},
-	{SPEEDDIAL, 			"SPEEDDIAL"},
-	{SERVICE, 			"SERVICE"},
-	{FEATURE, 			"FEATURE"},
-	{EMPTY, 			"EMPTY"}
-	/* *INDENT-ON* */
-};
-
 /*!
  * \brief SCCP Conference Structure
  */
@@ -530,14 +424,6 @@ static const struct sccp_feature_type {
 	/* *INDENT-ON* */
 };
 
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_FEATURE_MONITOR_STATE_DISABLED 	= 0,
-	SCCP_FEATURE_MONITOR_STATE_ACTIVE	= 1 << 1,
-	SCCP_FEATURE_MONITOR_STATE_REQUESTED 	= 1 << 2,
-	/* *INDENT-ON* */
-} sccp_feature_monitor_state_t;											/*!< monitor feature state */
-
 /*!
  * \brief SCCP Feature Configuration Structure
  */
@@ -546,7 +432,7 @@ typedef struct {
 	boolean_t initialized;											/*!< Feature Enabled */
 	uint32_t previousStatus;										/*!< Feature Previous State */
 	uint32_t status;											/*!< Feature State */
-	char configOptions[SCCP_MAX_EXTENSION];									/*!< space for config values */
+//	char configOptions[SCCP_MAX_EXTENSION];									/*!< space for config values */
 } sccp_featureConfiguration_t;											/*!< SCCP Feature Configuration */
 
 /*!
@@ -564,25 +450,6 @@ struct sccp_ha {
 	int sense;
 };
 
-/*!
- * \brief Reading Type Enum
- */
-typedef enum {
-	SCCP_CONFIG_READINITIAL,
-	SCCP_CONFIG_READRELOAD
-} sccp_readingtype_t;												/*!< Reading Type */
-
-/*!
- * \brief Status of configuration change
- */
-typedef enum {
-	/* *INDENT-OFF* */
-	SCCP_CONFIG_NOUPDATENEEDED 	= 0,
-	SCCP_CONFIG_NEEDDEVICERESET 	= 1 << 1,
-	SCCP_CONFIG_WARNING 		= 1 << 2,
-	SCCP_CONFIG_ERROR 		= 1 << 3
-	/* *INDENT-ON* */
-} sccp_configurationchange_t;											/*!< configuration state change */
 
 /*!
  * \brief SCCP Mailbox Type Definition
@@ -650,11 +517,6 @@ struct sccp_callinfo {
 	unsigned int lastRedirectingParty_valid:1;								/*!< TRUE if the name information is valid/present */
 };														/*!< SCCP CallInfo Structure */
 
-typedef enum {
-	SCCP_CALLSTATISTIC_LAST = 0,
-	SCCP_CALLSTATISTIC_AVG = 1
-} sccp_call_statistics_type_t;
-
 typedef struct sccp_call_statistic {
 	uint32_t num;
 	uint32_t packets_sent;
@@ -915,7 +777,7 @@ struct sccp_device {
 	uint8_t protocolversion;										/*!< Skinny Supported Protocol Version */
 	uint8_t inuseprotocolversion;										/*!< Skinny Used Protocol Version */
 	uint16_t registrationState;										/*!< If the device has been fully registered yet */
-	boolean_t nat;												/*!< Network Address Translation Support (Boolean, default=on) */
+	sccp_nat_t nat;												/*!< Network Address Translation Support (Boolean, default=on) */
 	boolean_t directrtp;											/*!< Direct RTP Support (Boolean, default=on) */
 
 	sccp_channel_t *active_channel;										/*!< Active SCCP Channel */
@@ -1025,6 +887,7 @@ struct sccp_device {
 	sccp_featureConfiguration_t overlapFeature;								/*!< Overlap Dial Feature */
 	sccp_featureConfiguration_t monitorFeature;								/*!< Monitor (automon) Feature */
 	sccp_featureConfiguration_t dndFeature;									/*!< dnd Feature */
+	uint16_t dndmode;											/*!< dnd mode: see SCCP_DNDMODE_* */
 	sccp_featureConfiguration_t priFeature;									/*!< priority Feature */
 	sccp_featureConfiguration_t mobFeature;									/*!< priority Feature */
 
@@ -1068,7 +931,8 @@ struct sccp_device {
 	void (*setBackgroundImage) (const sccp_device_t * device, const char *url);				/*!< set device background image */
 	void (*displayBackgroundImagePreview) (const sccp_device_t * device, const char *url);			/*!< display background image as preview */
 	void (*setRingTone) (const sccp_device_t * device, const char *url);					/*!< set the default Ringtone */
-	 sccp_dtmfmode_t(*getDtmfMode) (const sccp_device_t * device);
+	
+	sccp_dtmfmode_t(*getDtmfMode) (const sccp_device_t * device);
 
 	char *(messageStack[SCCP_MAX_MESSAGESTACK]);								/*!< Message Stack Array */
 #ifndef SCCP_ATOMIC
@@ -1088,9 +952,9 @@ struct sccp_device {
 #endif
 	uint16_t defaultLineInstance;										/*!< Default Line Instance */
 	char softkeyDefinition[50];										/*!< requested softKey configuration */
-#ifdef CS_EXPERIMENTAL
-	sccp_softKeySetConfiguration_t *softkeyset;
-#endif
+	sccp_softKeySetConfiguration_t *softkeyset;								/*!< Allow for a copy of the softkeyset, if any of the softkeys needs to be redefined, for example for urihook/uriaction */
+	
+	void (*copyStr2Locale) (const sccp_device_t *d, char *dst, const char *src, size_t dst_size);		/*!< copy string to device converted to locale if necessary */
 
 	boolean_t pendingDelete;										/*!< this bit will tell the scheduler to delete this line when unused */
 	boolean_t pendingUpdate;										/*!< this will contain the updated line struct once reloaded from config to update the line when unused */
@@ -1223,7 +1087,7 @@ struct sccp_channel {
 	sccp_autoanswer_t autoanswer_type;									/*!< Auto Answer Type */
 
 	/* don't allow sccp phones to monitor (hint) this call */
-	sccp_softswitch_action_t ss_action;									/*!< Simple Switch Action. This is used in dial thread to collect numbers for callforward, pickup and so on -FS */
+	sccp_softswitch_t softswitch_action;									/*!< Simple Switch Action. This is used in dial thread to collect numbers for callforward, pickup and so on -FS */
 	uint16_t ss_data;											/*!< Simple Switch Integer param */
 	uint16_t subscribers;											/*!< Used to determine if a sharedline should be hungup immediately, if everybody declined the call */
 
@@ -1243,6 +1107,7 @@ struct sccp_channel {
 	char musicclass[SCCP_MAX_MUSICCLASS];									/*!< Music Class */
 	sccp_callinfo_t callInfo;
 	sccp_dtmfmode_t dtmfmode;										/*!< DTMF Mode (0 inband - 1 outofband) */
+	sccp_video_mode_t videomode;										/*!< Video Mode (0 off - 1 user - 2 auto) */
 };														/*!< SCCP Channel Structure */
 
 /*!
@@ -1320,7 +1185,7 @@ struct sccp_global_vars {
 	boolean_t cfwdall;											/*!< Call Forward All Support (Boolean, default=on) */
 	boolean_t cfwdbusy;											/*!< Call Forward on Busy Support (Boolean, default=on) */
 	boolean_t cfwdnoanswer;											/*!< Call Forward on No-Answer Support (Boolean, default=on) */
-	boolean_t nat;												/*!< Network Address Translation */
+	sccp_nat_t nat;												/*!< Network Address Translation */
 	boolean_t directrtp;											/*!< Direct RTP */
 	boolean_t useoverlap;											/*!< Overlap Dial Support */
 	sccp_group_t callgroup;											/*!< Call Group */
@@ -1403,9 +1268,7 @@ struct softKeySetConfiguration {
 	char name[50];												/*!< Name for this configuration */
 	softkey_modes modes[16];										/*!< SoftKeySet modes, see KEYMODE_ */
 	uint8_t numberOfSoftKeySets;										/*!< Number of SoftKeySets we define */
-#ifdef CS_EXPERIMENTAL
 	sccp_softkeyMap_cb_t *softkeyCbMap;									/*!< Softkey Callback Map, ie handlers */
-#endif
 	unsigned int pendingDelete:1;
 	unsigned int pendingUpdate:1;
 	SCCP_LIST_ENTRY (sccp_softKeySetConfiguration_t) list;							/*!< Next list entry */
