@@ -1,15 +1,15 @@
 /*!
- * \file	ast.h
+ * \file        ast.h
  * \brief       SCCP PBX Asterisk Header
- * \author	Diederik de Groot <ddegroot [at] users.sourceforge.net>
- * \note	Reworked, but based on chan_sccp code.
- *		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *		Modified by Jan Czmok and Julien Goodwin
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
+ * \note        Reworked, but based on chan_sccp code.
+ *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *              Modified by Jan Czmok and Julien Goodwin
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5789 $  
+ * $Date: 2015-04-19 20:49:40 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6026 $  
  */
 #ifndef __SCCP_PBX_WRAPPER_H
 #define __SCCP_PBX_WRAPPER_H
@@ -42,12 +42,14 @@
 #include <asterisk/musiconhold.h>
 #include <asterisk/astdb.h>
 #include <asterisk/features.h>
-#ifdef HAVE_PBX_EVENT
+#ifdef HAVE_PBX_EVENT_H
 #include <asterisk/event.h>
 #endif
+#ifdef HAVE_PBX_CHANNEL_pvt_H
 #ifndef CS_AST_HAS_TECH_PVT
 #include <asterisk/channel_pvt.h>
 #endif
+#endif
 #ifdef HAVE_PBX_DEVICESTATE_H
 #include <asterisk/devicestate.h>
 #endif
@@ -75,15 +77,12 @@
 #endif
 #include <asterisk/indications.h>
 
+#if CS_TEST_FRAMEWORK
+#include "asterisk/test.h"
+#endif
+
 #include "define.h"
-#ifdef ASTERISK_CONF_1_2
-#include "ast102.h"
-#define PBXVER 10200
-#endif
-#ifdef ASTERISK_CONF_1_4
-#include "ast104.h"
-#define PBXVER 10400
-#endif
+#include "../../sccp_protocol.h"
 #ifdef ASTERISK_CONF_1_6
 #include "ast106.h"
 #endif
@@ -104,6 +103,7 @@
 #endif
 #include <asterisk/file.h>
 
+
 /* only trunk version has AST_CAUSE_ANSWERED_ELSEWHERE */
 #ifndef AST_CAUSE_ANSWERED_ELSEWHERE
 #define AST_CAUSE_ANSWERED_ELSEWHERE 200
@@ -143,7 +143,7 @@ static const struct skinny2pbx_codec_map {
 	{SKINNY_CODEC_G729,		AST_FORMAT_G729A},
 	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729A},
 	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263_PLUS},
-#endif
+#    endif
 #    if ASTERISK_VERSION_NUMBER >= 10400
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726_AAL2},
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726},
@@ -330,22 +330,6 @@ ast_format_type skinny_codec2pbx_codec(skinny_codec_t codec);
 
 int skinny_codecs2pbx_codecs(skinny_codec_t * skinny_codecs);
 
-#define sccp_strndup(str, len) \
-	_sccp_strndup((str), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-extern inline char *_sccp_strndup(const char *str, size_t len, const char *file, int lineno, const char *func);
-
-extern inline char *_sccp_strndup(const char *str, size_t len, const char *file, int lineno, const char *func)
-{
-	char *newstr = NULL;
-
-	if (str) {
-		if (!(newstr = strndup(str, len)))
-			pbx_log(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file);
-	}
-	return newstr;
-}
-
 /* 
  * sccp_free_ptr should be used when a function pointer for free() needs to be 
  * passed as the argument to a function. Otherwise, astmm will cause seg faults.
@@ -363,8 +347,8 @@ sccp_channel_t *__get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx
 sccp_channel_t *get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx_channel);
 #endif
 int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcname, char *args, const char *value);
-boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t *channel);
-boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t *channel);
+boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t * channel);
+boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t * channel);
 
 /***** database *****/
 boolean_t sccp_asterisk_addToDatabase(const char *family, const char *key, const char *value);
@@ -380,10 +364,11 @@ void sccp_asterisk_redirectedUpdate(sccp_channel_t * channel, const void *data,
 void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const char *fromNumber, const char *fromName, const char *toNumber, const char *toName, uint8_t reason);
 int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *funcname, char *args, char *buf, size_t buflen);
 boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel);
+
 #if ASTERISK_VERSION_GROUP > 106
 int sccp_wrapper_sendDigits(const sccp_channel_t * channel, const char *digits);
 int sccp_wrapper_sendDigit(const sccp_channel_t * channel, const char digit);
 #endif
-enum ast_pbx_result pbx_pbx_start (struct ast_channel *ast);
+enum ast_pbx_result pbx_pbx_start(struct ast_channel *ast);
 #endif
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
