@@ -1,13 +1,13 @@
 /*!
- * \file	ast111.h
+ * \file        ast111.h
  * \brief       SCCP PBX Asterisk Header
- * \author	Marcello Ceshia
- * \author	Diederik de Groot <ddegroot [at] users.sourceforge.net>
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \author      Marcello Ceshia
+ * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5789 $  
+ * $Date: 2015-04-08 04:37:28 +0200 (Wed, 08 Apr 2015) $
+ * $Revision: 6003 $  
  */
 
 #ifndef SCCP_AST_MAJOR_H_
@@ -122,31 +122,53 @@ int pbx_manager_register(const char *action, int authority, int (*func) (struct
 #define CLI_AMI_OUTPUT(fd, s, ...) 										\
 	if (NULL != s) {											\
 		astman_append(s, __VA_ARGS__);									\
-		local_total++;											\
+		local_line_total++;										\
 	} else {												\
 		ast_cli(fd, __VA_ARGS__);									\
 	}
 
-#define CLI_AMI_OUTPUT_PARAM(param, width, fmt, ...) 							\
+#define CLI_AMI_CAMEL_PARAM(param, camelParam)									\
+	char *current = param;											\
+	char *ptr = camelParam;											\
+	int CapsNext = 0;											\
+	while (*current) {											\
+		if ((*current >= 48 && *current <= 57 /*num*/) || (*current >= 65 && *current <= 90 /*A-Z*/) || (*current >= 97 && *current <= 122 /*a-z*/)) {	\
+			if (CapsNext) 	*ptr++ = toupper(*current++);						\
+			else 		*ptr++ = *current++;							\
+			CapsNext = 0;										\
+		} else {											\
+			CapsNext = 1;										\
+			current++;										\
+		}												\
+	}													\
+	*ptr='\0';												
+
+#define CLI_AMI_OUTPUT_PARAM(param, width, fmt, ...) 								\
 	if (NULL != s) {											\
-		astman_append(s, "%s: " fmt "\r\n", param, __VA_ARGS__);					\
-		local_total++;											\
+		char *camelParam = ast_strdupa(param);								\
+		CLI_AMI_CAMEL_PARAM(param, camelParam);								\
+		astman_append(s, "%s: " fmt "\r\n", camelParam, __VA_ARGS__);					\
+		local_line_total++;										\
 	} else {												\
 		ast_cli(fd, "%-*.*s %s " fmt "\n", width, width, param, ":", __VA_ARGS__);			\
 	}
 
 #define CLI_AMI_OUTPUT_BOOL(param, width, value) 								\
 	if (NULL != s) {											\
-		astman_append(s, "%s: %s\r\n", param, ((value) ? "on" : "off"));				\
-		local_total++;											\
+		char *camelParam = ast_strdupa(param);								\
+		CLI_AMI_CAMEL_PARAM(param, camelParam);								\
+		astman_append(s, "%s: %s\r\n", camelParam, ((value) ? "on" : "off"));				\
+		local_line_total++;										\
 	} else {												\
 		ast_cli(fd, "%-*.*s %s %s\n", width, width, param, ":", ((value) ? "on" : "off")); 		\
 	}
 
 #define CLI_AMI_OUTPUT_YES_NO(param, width, value) 								\
 	if (NULL != s) {											\
-		astman_append(s, "%s: %s\r\n", param, ((value) ? "yes" : "no"));				\
-		local_total++;											\
+		char *camelParam = ast_strdupa(param);								\
+		CLI_AMI_CAMEL_PARAM(param, camelParam);								\
+		astman_append(s, "%s: %s\r\n", camelParam, ((value) ? "yes" : "no"));				\
+		local_line_total++;										\
 	} else {												\
 		ast_cli(fd, "%-*.*s %s %s\n", width, width, param, ":", ((value) ? "yes" : "no")); 		\
 	}
@@ -157,7 +179,7 @@ int pbx_manager_register(const char *action, int authority, int (*func) (struct
                 char tmp_ ## line[100];										\
 	        snprintf(tmp_ ## line,sizeof(tmp_ ## line),fmt,__VA_ARGS__);					\
 	        astman_send_error(s, m, tmp_ ## line);								\
-		local_total++;											\
+		local_line_total++;										\
 	} else {												\
 		ast_cli(fd, "SCCP CLI ERROR: " fmt, __VA_ARGS__);						\
 	}													\
@@ -170,7 +192,7 @@ int pbx_manager_register(const char *action, int authority, int (*func) (struct
 //   param3=cli string to be types as array of strings
 //   param4=registration description
 //   param5=usage string
-#define CLI_AMI_ENTRY(_FUNCTION_NAME,_CALLED_FUNCTION,_DESCR,_USAGE, _COMPLETER_REPEAT)				\
+#define CLI_AMI_ENTRY(_FUNCTION_NAME,_CALLED_FUNCTION,_DESCR,_USAGE, _COMPLETER_REPEAT, _EVENTLIST)		\
 	static int manager_ ## _FUNCTION_NAME(struct mansession *s, const struct message *m)			\
 	{													\
 		const char *id = astman_get_header(m, "ActionID");						\
@@ -183,21 +205,29 @@ int pbx_manager_register(const char *action, int authority, int (*func) (struct
 			}											\
 		}												\
 		char idtext[256] = "";										\
-		int total = 0;											\
+		/*int total = 0; */										\
+		sccp_cli_totals_t totals = {0};									\
 		if (!pbx_strlen_zero(id)) {									\
-			snprintf(idtext, sizeof(idtext), "ActionID: %s", id);				\
+			snprintf(idtext, sizeof(idtext), "ActionID: %s\r\n", id);				\
 		}												\
-		if (RESULT_SUCCESS==_CALLED_FUNCTION(-1, &total, s, m, ARRAY_LEN(arguments), arguments)) {	\
-		        astman_send_ack(s, m, AMI_COMMAND);							\
+		if (_EVENTLIST == TRUE) {									\
+			astman_send_listack(s, m, AMI_COMMAND " list will follow", "start");			\
+		}												\
+		if (RESULT_SUCCESS==_CALLED_FUNCTION(-1, &totals, s, m, ARRAY_LEN(arguments), arguments)) {	\
+			if (_EVENTLIST == TRUE) {								\
+				astman_append(s,								\
+				"Event: " AMI_COMMAND "Complete\r\n"						\
+				"EventList: Complete\r\n"							\
+				"ListItems: %d\r\n"								\
+				"ListTableItems: %d\r\n"							\
+				"%s"										\
+				"\r\n", totals.lines, totals.tables, idtext);  				\
+			} else {										\
+				astman_append(s, "\r\n");							\
+			}											\
                 } else {											\
                         astman_send_error(s, m, "Execution Failed\n");						\
                 }												\
-		astman_append(s,										\
-		"Event: " AMI_COMMAND "Complete\r\n"								\
-		"EventList: Complete\r\n"									\
-		"ListItems: %d\r\n"										\
-		"%s"												\
-		"\r\n\r\n", total, idtext);  									\
 		return 0;											\
 	}													\
 														\
