@@ -1,12 +1,12 @@
 /*!
- * \file	define.h
+ * \file        define.h
  * \brief       SCCP PBX Redefinition Asterisk Header
- * \author	Diederik de Groot <ddegroot [at] users.sourceforge.net>
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5789 $  
+ * $Date: 2015-04-03 03:51:36 +0200 (Fri, 03 Apr 2015) $
+ * $Revision: 5995 $  
  */
 #ifndef SCCP_AST_DEFINE_H_
 #define SCCP_AST_DEFINE_H_
@@ -24,6 +24,7 @@
 #define sccp_asprintf ast_asprintf
 #define sccp_vasprintf ast_vasprintf
 #define sccp_calloc_cache ast_calloc_cache
+#define sccp_alloca(size) __builtin_alloca(size)
 #define CS_BRIDGEPEERNAME "BRIDGEPEER"
 
 // codec / format redefinitions
@@ -75,7 +76,7 @@
 #ifndef CS_BRIDGE_BASE_NEW
 #define pbx_bridge_new(_a,_b,_c,_d,_e) ast_bridge_new(_a, _b)
 #else
-#define pbx_bridge_new(_a,_b,_c,_d,_e) ast_bridge_base_new(_a, _b)
+#define pbx_bridge_new(_a,_b,_c,_d,_e) ast_bridge_base_new(_a, _b, _c, _d, _e)
 #endif
 #define pbx_bridge_remove ast_bridge_remove
 #define pbx_bridge_result ast_bridge_result
@@ -195,19 +196,30 @@ typedef struct ast_event pbx_event_t;
 #define pbx_mutex_destroy ast_mutex_destroy
 #define pbx_mutex_init ast_mutex_init
 #define pbx_mutex_init_notracking ast_mutex_init_notracking
-#define pbx_mutex_lock ast_mutex_lock
-#define pbx_mutex_trylock ast_mutex_trylock
-#define pbx_mutex_unlock ast_mutex_unlock
-
 #define pbx_rwlock_t ast_rwlock_t
 #define pbx_rwlock_destroy ast_rwlock_destroy
 #define pbx_rwlock_init ast_rwlock_init
 #define pbx_rwlock_init_notracking ast_rwlock_init_notracking
+
+#if CS_LOCK_DEBUG
+#define pbx_mutex_lock(x) {ast_debug(4, "%d (%s) MUTEX_LOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_mutex_lock(x);}
+#define pbx_mutex_trylock(x) {ast_debug(4, "%d (%s) MUTEX_TRYLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_mutex_trylock(x);}
+#define pbx_mutex_unlock(x) {ast_debug(4, "%d (%s) MUTEX_UNLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_mutex_unlock(x);}
+#define pbx_rwlock_rdlock(x) {ast_debug(5, "%d (%s) RWLOCK_RDLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_rwlock_rdlock(x);}
+#define pbx_rwlock_wrlock(x) {ast_debug(5, "%d (%s) RWLOCK_WRLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_rwlock_wrlock(x);}
+#define pbx_rwlock_tryrdlock(x) {ast_debug(5, "%d (%s) RWLOCK_TRYRDLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_rwlock_tryrdlock(x);}
+#define pbx_rwlock_trywrlock(x) {ast_debug(5, "%d (%s) RWLOCK_TRYWRLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_rwlock_trywrlock(x);}
+#define pbx_rwlock_unlock(x) {ast_debug(5, "%d (%s) RWLOCK_UNLOCK: " #x ": %p\n", __LINE__, __PRETTY_FUNCTION__, x); ast_rwlock_unlock(x);}
+#else
+#define pbx_mutex_lock ast_mutex_lock
+#define pbx_mutex_trylock ast_mutex_trylock
+#define pbx_mutex_unlock ast_mutex_unlock
 #define pbx_rwlock_rdlock ast_rwlock_rdlock
 #define pbx_rwlock_wrlock ast_rwlock_wrlock
 #define pbx_rwlock_tryrdlock ast_rwlock_tryrdlock
 #define pbx_rwlock_trywrlock ast_rwlock_trywrlock
 #define pbx_rwlock_unlock ast_rwlock_unlock
+#endif
 
 #define pbx_park_call ast_park_call
 #define pbx_party_name_free ast_party_name_free
@@ -269,6 +281,7 @@ typedef struct ast_event pbx_event_t;
 #define pbx_state2str ast_state2str
 #define pbx_str_alloca ast_str_alloca
 #define pbx_str_append ast_str_append
+#define pbx_str_append_escapecommas ast_str_append_escapecommas
 #define pbx_str_buffer ast_str_buffer
 #define pbx_strdup ast_strdup
 #define pbx_strdupa ast_strdupa
