@@ -9,8 +9,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $  
+ * $Date: 2015-01-26 17:11:20 +0100 (Mon, 26 Jan 2015) $
+ * $Revision: 5889 $  
  */
 
 #ifndef __SCCP_CHANNEL_H
@@ -22,7 +22,7 @@
 
 /* live cycle */
 sccp_channel_t *sccp_channel_allocate(sccp_line_t * l, sccp_device_t * device);					// device is optional
-sccp_channel_t *sccp_channel_newcall(sccp_line_t * l, sccp_device_t * device, const char *dial, uint8_t calltype, PBX_CHANNEL_TYPE * parentChannel);
+sccp_channel_t *sccp_channel_newcall(sccp_line_t * l, sccp_device_t * device, const char *dial, uint8_t calltype, PBX_CHANNEL_TYPE * parentChannel, const void *ids);
 
 void sccp_channel_updateChannelDesignator(sccp_channel_t * c);
 void sccp_channel_updateChannelCapability(sccp_channel_t * channel);
@@ -87,6 +87,7 @@ void sccp_channel_park(sccp_channel_t * c);
 #endif
 
 boolean_t sccp_channel_setPreferredCodec(sccp_channel_t * c, const void *data);
+boolean_t sccp_channel_setVideoMode(sccp_channel_t * c, const void *data);
 int sccp_channel_callwaiting_tone_interval(sccp_device_t * d, sccp_channel_t * c);
 const char *sccp_channel_getLinkedId(const sccp_channel_t * channel);
 
