@@ -8,13 +8,17 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-07 18:44:04 +0200 (Tue, 07 Apr 2015) $
+ * $Revision: 6002 $
  */
 #ifndef __SCCP_CLI_H
 #define __SCCP_CLI_H
 
 #define CLI_AMI_LIST_WIDTH 46
+typedef struct sccp_cli_totals {
+	int lines;
+	int tables;
+} sccp_cli_totals_t;
 void sccp_register_cli(void);
 void sccp_unregister_cli(void);
 
