@@ -4,8 +4,8 @@
  * \author      Marcello Ceschia <marcelloceschia [at] users.sorceforge.net>
  * \note        Reworked, but based on chan_sccp code.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-09 18:05:16 +0200 (Thu, 09 Apr 2015) $
+ * $Revision: 6006 $
  */
 
 #include <config.h>
@@ -38,7 +38,7 @@
 #define sccp_participant_retain(_x) 	({ast_assert(_x != NULL);(sccp_conference_participant_t *)sccp_refcount_retain(_x, __FILE__, __LINE__, __PRETTY_FUNCTION__);})
 #define sccp_participant_release(_x) 	({ast_assert(_x != NULL);(sccp_conference_participant_t *)sccp_refcount_release(_x, __FILE__, __LINE__, __PRETTY_FUNCTION__);})
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6006 $");
 static uint32_t lastConferenceID = 99;
 static const uint32_t appID = APPID_CONFERENCE;
 
@@ -206,7 +206,12 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 #endif
 #endif
 	/* using the SMART flag results in issues when removing forgeign participant, because it try to create a new conference and merge into it. Which seems to be more complex then necessary */
+#if ASTERISK_VERSION_GROUP >= 113
+	conference->bridge = pbx_bridge_new(bridgeCapabilities, AST_BRIDGE_FLAG_DISSOLVE_HANGUP, channel->designator, conferenceIdentifier, NULL);
+#else
 	conference->bridge = pbx_bridge_new(bridgeCapabilities, 0, channel->designator, conferenceIdentifier, NULL);
+#endif
+
 #if defined(CS_SCCP_VIDEO) && ASTERISK_VERSION_GROUP >= 112
 	ast_bridge_set_talker_src_video_mode(conference->bridge);
 #endif
@@ -256,8 +261,8 @@ sccp_conference_t *sccp_conference_create(sccp_device_t * device, sccp_channel_t
 		sccp_conference_update_callInfo(channel, participant->conferenceBridgePeer, conference->id);
 		participant->isModerator = TRUE;
 		device->conferencelist_active = device->conf_show_conflist;					// Activate conflist
-		//  sccp_softkey_setSoftkeyState(device, KEYMODE_CONNCONF, SKINNY_LBL_JOIN, TRUE);
-		//  sccp_softkey_setSoftkeyState(device, KEYMODE_CONNTRANS, SKINNY_LBL_JOIN, TRUE);
+		// sccp_softkey_setSoftkeyState(device, KEYMODE_CONNCONF, SKINNY_LBL_JOIN, TRUE);
+		// sccp_softkey_setSoftkeyState(device, KEYMODE_CONNTRANS, SKINNY_LBL_JOIN, TRUE);
 		sccp_dev_set_keyset(device, sccp_device_find_index_for_line(device, channel->line->name), channel->callid, KEYMODE_CONNCONF);
 		pbx_builtin_setvar_int_helper(channel->owner, "__SCCP_CONFERENCE_ID", conference->id);
 		pbx_builtin_setvar_int_helper(channel->owner, "__SCCP_CONFERENCE_PARTICIPANT_ID", participant->id);
@@ -402,7 +407,7 @@ void sccp_conference_update_callInfo(sccp_channel_t * channel, PBX_CHANNEL_TYPE
 			sccp_copy_string(channel->callInfo.calledPartyName, confstr, sizeof(channel->callInfo.calledPartyName));
 			channel->callInfo.calledParty_valid = 1;
 			break;
-		case skinny_calltype_LOOKUPERROR:
+		case SKINNY_CALLTYPE_SENTINEL:
 			break;
 	}
 
@@ -1092,7 +1097,7 @@ void sccp_conference_show_list(sccp_conference_t * conference, sccp_channel_t *
 					case SKINNY_CALLTYPE_FORWARD:
 						sprintf(xmlTmp, "%d:%s (%s)", part->id, part->channel->callInfo.originalCallingPartyName, part->channel->callInfo.originalCallingPartyName);
 						break;
-					case skinny_calltype_LOOKUPERROR:
+					case SKINNY_CALLTYPE_SENTINEL:
 						break;
 				}
 				strcat(xmlStr, xmlTmp);
@@ -1602,9 +1607,9 @@ char *sccp_complete_conference(OLDCONST char *line, OLDCONST char *word, int pos
  * 
  * \called_from_asterisk
  */
-int sccp_cli_show_conferences(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_cli_show_conferences(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
-	int local_total = 0;
+	int local_line_total = 0;
 	sccp_conference_t *conference = NULL;
 
 	// table definition
@@ -1617,16 +1622,17 @@ int sccp_cli_show_conferences(int fd, int *total, struct mansession *s, const st
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
 
-#define CLI_AMI_TABLE_FIELDS 																		\
-		CLI_AMI_TABLE_FIELD(Id,			"3.3d",		3,	conference->id)										\
-		CLI_AMI_TABLE_FIELD(Participants,	"-12.12d",	12,	SCCP_LIST_GETSIZE(&conference->participants))						\
-		CLI_AMI_TABLE_FIELD(Moderators,		"-12.12d",	12,	conference->num_moderators)								\
-		CLI_AMI_TABLE_FIELD(Announce,		"-12.12s",	12,	conference->playback_announcements ? "Yes" : "No")					\
-		CLI_AMI_TABLE_FIELD(MuteOnEntry,	"-12.12s",	12,	conference->mute_on_entry ? "Yes" : "No")						\
+#define CLI_AMI_TABLE_FIELDS 																			\
+		CLI_AMI_TABLE_FIELD(Id,			"3.3",		d,		3,	conference->id)									\
+		CLI_AMI_TABLE_FIELD(Participants,	"-12.12",	d,	12,	SCCP_LIST_GETSIZE(&conference->participants))						\
+		CLI_AMI_TABLE_FIELD(Moderators,		"-12.12",	d,	12,	conference->num_moderators)								\
+		CLI_AMI_TABLE_FIELD(Announce,		"-12.12",	s,	12,	conference->playback_announcements ? "Yes" : "No")					\
+		CLI_AMI_TABLE_FIELD(MuteOnEntry,	"-12.12",	s,	12,	conference->mute_on_entry ? "Yes" : "No")						\
 
 #include "sccp_cli_table.h"
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
+		totals->tables = 1;
 	}
 	return RESULT_SUCCESS;
 }
@@ -1643,9 +1649,9 @@ int sccp_cli_show_conferences(int fd, int *total, struct mansession *s, const st
  * 
  * \called_from_asterisk
  */
-int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_cli_show_conference(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
-	int local_total = 0;
+	int local_line_total = 0;
 	int confid = 0;
 
 	if (argc < 4 || argc > 5 || sccp_strlen_zero(argv[3])) {
@@ -1676,13 +1682,13 @@ int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const str
 #define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
-#define CLI_AMI_TABLE_FIELDS 																					\
-			CLI_AMI_TABLE_FIELD(Id,			"3.3d",		3,	participant->id)											\
-			CLI_AMI_TABLE_FIELD(ChannelName,	"-20.20s",	20,	participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL")	\
-			CLI_AMI_TABLE_FIELD(Moderator,		"-11.11s",	11,	participant->isModerator ? "Yes" : "No")								\
-			CLI_AMI_TABLE_FIELD(Muted,		"-5.5s",	5,	participant->features.mute ? "Yes" : "No")								\
-			CLI_AMI_TABLE_FIELD(Announce,		"-8.8s",	8,	participant->playback_announcements ? "Yes" : "No")							\
-			CLI_AMI_TABLE_FIELD(ConfList,		"-8.8s",	8,	(participant->device && participant->device->conferencelist_active) ? "YES" : "NO")
+#define CLI_AMI_TABLE_FIELDS 																						\
+			CLI_AMI_TABLE_FIELD(Id,			"3.3",		d,		3,	participant->id)										\
+			CLI_AMI_TABLE_FIELD(ChannelName,	"-20.20",	s,	20,	participant->conferenceBridgePeer ? pbx_channel_name(participant->conferenceBridgePeer) : "NULL")	\
+			CLI_AMI_TABLE_FIELD(Moderator,		"-11.11",	s,	11,	participant->isModerator ? "Yes" : "No")								\
+			CLI_AMI_TABLE_FIELD(Muted,		"-5.5",		s,	5,	participant->features.mute ? "Yes" : "No")								\
+			CLI_AMI_TABLE_FIELD(Announce,		"-8.8",		s,	8,	participant->playback_announcements ? "Yes" : "No")							\
+			CLI_AMI_TABLE_FIELD(ConfList,		"-8.8",		s,	8,	(participant->device && participant->device->conferencelist_active) ? "YES" : "NO")
 
 #include "sccp_cli_table.h"
 	} else {
@@ -1690,7 +1696,8 @@ int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const str
 		CLI_AMI_RETURN_ERROR(fd, s, m, "At least valid ConferenceId needs to be supplied\n %s", "");
 	}
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
+		totals->tables = 1;
 	}
 	return RESULT_SUCCESS;
 }
@@ -1707,10 +1714,10 @@ int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const str
  * 
  * \called_from_asterisk
  */
-int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_cli_conference_command(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
 	int confid = 0, partid = 0;
-	int local_total = 0;
+	int local_line_total = 0;
 	int res = RESULT_SUCCESS;
 	char error[100];
 
@@ -1781,7 +1788,7 @@ int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const
 		CLI_AMI_RETURN_ERROR(fd, s, m, "%s\n", error);
 	}
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
 	}
 	return res;
 }
