@@ -1,12 +1,12 @@
 /*!
- * \file	sccp_conference.h
+ * \file        sccp_conference.h
  * \brief       SCCP Conference Header
  * \author
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2014-05-13 12:41:46 +0200 (Tue, 13 May 2014) $
- * $Revision: 5409 $  
+ * $Date: 2015-04-07 18:44:04 +0200 (Tue, 07 Apr 2015) $
+ * $Revision: 6002 $  
  */
 
 #ifndef SCCP_CONFERENCE_H_
@@ -14,6 +14,8 @@
 
 #ifdef CS_SCCP_CONFERENCE
 
+#include "sccp_cli.h"
+
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 /* *INDENT-OFF* */
@@ -85,9 +87,9 @@ void sccp_conference_demode_participant(sccp_conference_t * conference, sccp_cha
 
 /* cli functions */
 char *sccp_complete_conference(OLDCONST char *line, OLDCONST char *word, int pos, int state);
-int sccp_cli_show_conferences(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[]);
-int sccp_cli_show_conference(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[]);
-int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[]);
+int sccp_cli_show_conferences(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[]);
+int sccp_cli_show_conference(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[]);
+int sccp_cli_conference_command(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[]);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 /* *INDENT-ON* */
@@ -95,3 +97,4 @@ int sccp_cli_conference_command(int fd, int *total, struct mansession *s, const
 #endif
 #endif														/* CS_SCCP_CONFERENCE */
 #endif														/* SCCP_CONFERENCE_H_ */
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
