@@ -14,8 +14,8 @@
  * Relations:   SCCP Device -> SCCP DeviceLine -> SCCP Line
  *              SCCP Line -> SCCP ButtonConfig -> SCCP Device
  *
- * \date        $Date: 2014-11-20 15:57:48 +0100 (Thu, 20 Nov 2014) $
- * \version     $Revision: 5803 $
+ * \date        $Date: 2015-04-20 19:22:07 +0200 (Mon, 20 Apr 2015) $
+ * \version     $Revision: 6028 $
  */
 
 #include <config.h>
@@ -31,7 +31,7 @@
 #include "sccp_indicate.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5803 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6028 $");
 int __sccp_device_destroy(const void *ptr);
 void sccp_device_removeFromGlobals(sccp_device_t * device);
 int sccp_device_destroy(const void *ptr);
@@ -192,6 +192,24 @@ static void sccp_device_setRingtone(const sccp_device_t * device, const char *ur
 	device->protocol->sendUserToDeviceDataVersionMessage(device, 0, 0, 0, transactionID, xmlStr, 0);
 }
 
+static void sccp_device_copyStr2Locale_UTF8(const sccp_device_t *d, char *dst, const char *src, size_t dst_size)
+{
+	sccp_copy_string(dst, src, dst_size);
+}
+
+#if HAVE_ICONV_H
+static void sccp_device_copyStr2Locale_Convert(const sccp_device_t *d, char *dst, const char *src, size_t dst_size)
+{
+	char *buf = sccp_alloca(dst_size);
+	size_t buf_len = dst_size;
+	memset(buf, 0, dst_size);
+	if (sccp_utils_convUtf8toLatin1(src, buf, buf_len)) {
+		sccp_copy_string(dst, buf, dst_size);
+		return;
+	}
+}
+#endif
+
 static void sccp_device_setRingtoneNotSupported(const sccp_device_t * device, const char *url)
 {
 	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: does not support setting ringtone\n", device->id);
@@ -324,7 +342,8 @@ boolean_t sccp_device_check_update(sccp_device_t * device)
 	boolean_t res = FALSE;
 
 	if (d) {
-		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "device: %s check_update, pendingUpdate: %s, pendingDelete: %s\n", d->id, d->pendingUpdate ? "TRUE" : "FALSE", d->pendingDelete ? "TRUE" : "FALSE");
+		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_2 "%s (check_update) pendingUpdate: %s, pendingDelete: %s\n", d->id, d->pendingUpdate ? "TRUE" : "FALSE", d->pendingDelete ? "TRUE" : "FALSE");
+
 		if ((d->pendingUpdate || d->pendingDelete)) {
 			do {
 				if (sccp_device_numberOfChannels(d) > 0) {
@@ -340,23 +359,6 @@ boolean_t sccp_device_check_update(sccp_device_t * device)
 					sccp_dev_clean(d, TRUE, 0);
 				} else {
 					sccp_dev_clean(d, FALSE, 0);
-					sccp_buttonconfig_t *buttonconfig;
-
-					SCCP_LIST_LOCK(&d->buttonconfig);
-					SCCP_LIST_TRAVERSE_SAFE_BEGIN(&d->buttonconfig, buttonconfig, list) {
-						if (!buttonconfig->pendingDelete && !buttonconfig->pendingUpdate) {
-							continue;
-						}
-						if (buttonconfig->pendingDelete) {
-							sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "Remove Buttonconfig for %s from List\n", d->id);
-							SCCP_LIST_REMOVE_CURRENT(list);
-							sccp_free(buttonconfig);
-						} else {
-							buttonconfig->pendingUpdate = 0;
-						}
-					}
-					SCCP_LIST_TRAVERSE_SAFE_END;
-					SCCP_LIST_UNLOCK(&d->buttonconfig);
 				}
 				res = TRUE;
 			} while (0);
@@ -376,6 +378,7 @@ boolean_t sccp_device_check_update(sccp_device_t * device)
 void sccp_device_post_reload(void)
 {
 	sccp_device_t *d;
+	sccp_log((DEBUGCAT_CONFIG)) (VERBOSE_PREFIX_1 "SCCP: (post_reload)\n");
 
 	SCCP_RWLIST_TRAVERSE_SAFE_BEGIN(&GLOB(devices), d, list) {
 		if (!d->pendingDelete && !d->pendingUpdate) {
@@ -387,6 +390,7 @@ void sccp_device_post_reload(void)
 		if (!sccp_device_check_update(d)) {
 			sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "Device %s will receive reset after current call is completed\n", d->id);
 		}
+		/* should we re-check the device after hangup ? */
 	}
 	SCCP_LIST_TRAVERSE_SAFE_END;
 }
@@ -416,9 +420,11 @@ sccp_device_t *sccp_device_create(const char *id)
 #ifdef CS_DEVSTATE_FEATURE
 	SCCP_LIST_HEAD_INIT(&d->devstateSpecifiers);
 #endif
-	//if (PBX(endpoint_create)) {
-	//  d->endpoint = PBX(endpoint_create)("sccp", id);
-	//}
+	/*
+	if (PBX(endpoint_create)) {
+		d->endpoint = PBX(endpoint_create)("sccp", id);
+	}
+	*/
 	memset(d->softKeyConfiguration.activeMask, 0xFFFF, sizeof(d->softKeyConfiguration.activeMask));
 	memset(d->call_statistics, 0, (sizeof(sccp_call_statistics_t) * 2));
 
@@ -449,10 +455,12 @@ sccp_device_t *sccp_device_create(const char *id)
 #endif
 
 	// /* disable videomode and join softkey for all softkeysets */
-	// for (i = 0; i < KEYMODE_ONHOOKSTEALABLE; i++) {
-	//	sccp_softkey_setSoftkeyState(d, i, SKINNY_LBL_VIDEO_MODE, FALSE);
-	//	sccp_softkey_setSoftkeyState(d, i, SKINNY_LBL_JOIN, FALSE);
-	// }
+	/*
+	for (i = 0; i < KEYMODE_ONHOOKSTEALABLE; i++) {
+		sccp_softkey_setSoftkeyState(d, i, SKINNY_LBL_VIDEO_MODE, FALSE);
+		sccp_softkey_setSoftkeyState(d, i, SKINNY_LBL_JOIN, FALSE);
+	}
+	*/
 
 	d->pushURL = sccp_device_pushURLNotSupported;
 	d->pushTextMessage = sccp_device_pushTextMessageNotSupported;
@@ -463,6 +471,8 @@ sccp_device_t *sccp_device_create(const char *id)
 	d->retrieveDeviceCapabilities = sccp_device_retrieveDeviceCapabilities;
 	d->setRingTone = sccp_device_setRingtoneNotSupported;
 	d->getDtmfMode = sccp_device_getDtfmMode;
+	d->copyStr2Locale = sccp_device_copyStr2Locale_UTF8;
+
 	d->pendingUpdate = 0;
 	d->pendingDelete = 0;
 	return d;
@@ -524,8 +534,9 @@ void sccp_device_setLastNumberDialed(sccp_device_t * device, const char *lastNum
 /*!
  * \brief set type of Indicate protocol by device type
  */
-void sccp_device_setIndicationProtocol(sccp_device_t * device)
+void sccp_device_preregistration(sccp_device_t * device)
 {
+	/*! \todo use device->device_features to detect devices capabilities, instead of hardcoded list of devices */
 	switch (device->skinny_type) {
 			// case SKINNY_DEVICETYPE_30SPPLUS:
 			// case SKINNY_DEVICETYPE_30VIP:
@@ -566,7 +577,11 @@ void sccp_device_setIndicationProtocol(sccp_device_t * device)
 			device->indicate = &sccp_device_indication_olderDevices;
 			break;
 	}
-	return;
+#if HAVE_ICONV_H
+	if (device && !(device->device_features & SKINNY_PHONE_FEATURES_UTF8)) {
+		device->copyStr2Locale = sccp_device_copyStr2Locale_Convert;
+	}
+#endif
 }
 
 /*!
@@ -819,9 +834,11 @@ void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn)
 			for (i = 0; i < 10; i++) {								// 4 visible, 6 in dropdown
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
-			// for (i = 5; i <= 10; i++) {
-			//   (btn++)->type = SCCP_BUTTONTYPE_SPEEDDIAL;
-			// }
+			/*
+			for (i = 5; i <= 10; i++) {
+				(btn++)->type = SCCP_BUTTONTYPE_SPEEDDIAL;
+			}
+			*/
 			(btn++)->type = SKINNY_BUTTONTYPE_CONFERENCE;
 			(btn++)->type = SKINNY_BUTTONTYPE_HOLD;
 			(btn++)->type = SKINNY_BUTTONTYPE_TRANSFER;
@@ -833,16 +850,25 @@ void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn)
 			for (i = 0; i < 1; i++) {
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
+			for (i = 2 + sccp_addons_taps(d); i > 0; i--) {
+				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
+			}
 			break;
 		case SKINNY_DEVICETYPE_SPA_303G:
 			for (i = 0; i < 3; i++) {
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
+			for (i = 2 + sccp_addons_taps(d); i > 0; i--) {
+				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
+			}
 			break;
 		case SKINNY_DEVICETYPE_SPA_504G:
 			for (i = 0; i < 4; i++) {
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
+			for (i = 2 + sccp_addons_taps(d); i > 0; i--) {
+				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
+			}
 			break;
 		case SKINNY_DEVICETYPE_SPA_509G:
 			for (i = 0; i < 12; i++) {
@@ -850,11 +876,17 @@ void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn)
 			}
 			(btn++)->type = SKINNY_BUTTONTYPE_VOICEMAIL;
 			(btn++)->type = SKINNY_BUTTONTYPE_HOLD;
+			for (i = 2 + sccp_addons_taps(d); i > 0; i--) {
+				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
+			}
 			break;
 		case SKINNY_DEVICETYPE_SPA_525G2:
 			for (i = 0; i < 8; i++) {
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
+			for (i = 2 + sccp_addons_taps(d); i > 0; i--) {
+				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
+			}
 			break;
 		case SKINNY_DEVICETYPE_CISCO6901:
 		case SKINNY_DEVICETYPE_CISCO6911:
@@ -866,13 +898,15 @@ void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn)
 			for (i = 0; i < 2; i++) {
 				(btn++)->type = SCCP_BUTTONTYPE_MULTI;
 			}
-			//for (i = 0; i < 6; i++) {
-			//  (btn++)->type = SCCP_BUTTONTYPE_SPEEDDIAL;
-			//}
-			//(btn++)->type = SKINNY_BUTTONTYPE_NONE;
-			//(btn++)->type = SKINNY_BUTTONTYPE_PRIVACY;
-			//(btn++)->type = SKINNY_BUTTONTYPE_DO_NOT_DISTURB;
-			//(btn++)->type = SKINNY_BUTTONTYPE_HLOG;                       // hunt group logout
+			/*
+			for (i = 0; i < 6; i++) {
+				(btn++)->type = SCCP_BUTTONTYPE_SPEEDDIAL;
+			}
+			(btn++)->type = SKINNY_BUTTONTYPE_NONE;
+			(btn++)->type = SKINNY_BUTTONTYPE_PRIVACY;
+			(btn++)->type = SKINNY_BUTTONTYPE_DO_NOT_DISTURB;
+			(btn++)->type = SKINNY_BUTTONTYPE_HLOG;                       // hunt group logout
+			*/
 			break;
 		case SKINNY_DEVICETYPE_CISCO6941:
 		case SKINNY_DEVICETYPE_CISCO6945:
@@ -987,7 +1021,6 @@ void sccp_dev_set_registered(sccp_device_t * d, uint8_t opt)
 			// d->mwilight &= ~(1 << 0);
 			sccp_dev_send(d, msg);
 		}
-
 		if (!d->linesRegistered) {
 			sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Device does not support RegisterAvailableLinesMessage, force this\n", DEV_ID_LOG(d));
 			sccp_handle_AvailableLines(d->session, d, NULL);
@@ -1461,13 +1494,17 @@ void sccp_dev_displaynotify_debug(const sccp_device_t * d, const char *msg, uint
  * \callgraph
  * \callergraph
  */
-void sccp_dev_cleardisplayprinotify(const sccp_device_t * d)
+void sccp_dev_cleardisplayprinotify(const sccp_device_t * d, const uint8_t priority)
 {
+	sccp_msg_t *msg;
 	if (!d || !d->session || !d->protocol || !d->hasDisplayPrompt()) {
 		return;
 	}
-	sccp_dev_sendmsg(d, ClearPriNotifyMessage);
-	sccp_log((DEBUGCAT_DEVICE + DEBUGCAT_MESSAGE)) (VERBOSE_PREFIX_3 "%s: Clear the display priority notify message\n", d->id);
+	REQ(msg, ClearPriNotifyMessage);
+	msg->data.ClearPriNotifyMessage.lel_priority = htolel(priority);
+
+	sccp_dev_send(d, msg);
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Clear the display priority notify message\n", d->id);
 }
 
 /*!
@@ -1486,12 +1523,11 @@ void sccp_dev_cleardisplayprinotify(const sccp_device_t * d)
 //void sccp_dev_displayprinotify(sccp_device_t * d, char *msg, uint32_t priority, uint32_t timeout)
 void sccp_dev_displayprinotify_debug(const sccp_device_t * d, const char *msg, const uint8_t priority, const uint8_t timeout, const char *file, const int lineno, const char *pretty_function)
 {
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: ( %s:%d:%s ) sccp_dev_displayprinotify '%s' (%d/%d)\n", DEV_ID_LOG(d), file, lineno, pretty_function, msg, timeout, priority);
-
 	if (!d || !d->session || !d->protocol || !d->hasDisplayPrompt()) {
 		return;
 	}
 	if (!msg || sccp_strlen_zero(msg)) {
+		sccp_dev_cleardisplayprinotify(d, priority);
 		return;
 	}
 	d->protocol->displayPriNotify(d, priority, timeout, msg);
@@ -1535,6 +1571,7 @@ void sccp_dev_speed_find_byindex(sccp_device_t * d, uint16_t instance, boolean_t
 			if (!sccp_strlen_zero(config->button.speeddial.hint)) {
 				sccp_copy_string(k->hint, config->button.speeddial.hint, sizeof(k->hint));
 			}
+			break;
 		}
 	}
 	SCCP_LIST_UNLOCK(&d->buttonconfig);
@@ -1906,26 +1943,9 @@ void sccp_dev_clean(sccp_device_t * device, boolean_t remove_from_global, uint8_
 			PBX(feature_addToDatabase) (family, "lastDialedNumber", d->lastNumber);
 		}
 
-		/* cleanup dynamic allocated strings */
-
-		/** normaly we should only remove this when removing the device from globals,
-		 *  in this case we can do this also when device unregistered, so we do not set this multiple times -MC
-		 */
-		//  if (d->backgroundImage) {
-		//    sccp_free(d->backgroundImage);
-		//    d->backgroundImage = NULL;
-		//  }
-
-		//  if (d->ringtone) {
-		//    sccp_free(d->ringtone);
-		//    d->ringtone = NULL;
-		//  }
-		sccp_config_cleanup_dynamically_allocated_memory(d, SCCP_CONFIG_DEVICE_SEGMENT);
-
 		/* hang up open channels and remove device from line */
-
 		SCCP_LIST_LOCK(&d->buttonconfig);
-		SCCP_LIST_TRAVERSE(&d->buttonconfig, config, list) {
+		SCCP_LIST_TRAVERSE_SAFE_BEGIN(&d->buttonconfig, config, list) {
 			if (config->type == LINE) {
 				AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(config->button.line.name, FALSE);
 
@@ -1953,7 +1973,14 @@ void sccp_dev_clean(sccp_device_t * device, boolean_t remove_from_global, uint8_
 				sccp_line_removeDevice(line, d);
 			}
 			config->instance = 0;									/* reset button configuration to rebuild template on register */
+			if (config->pendingDelete) {
+				SCCP_LIST_REMOVE_CURRENT(list);
+				sccp_free(config);
+				config = NULL;
+				
+			}
 		}
+		SCCP_LIST_TRAVERSE_SAFE_END;
 		SCCP_LIST_UNLOCK(&d->buttonconfig);
 		d->linesRegistered = FALSE;
 
@@ -2048,6 +2075,9 @@ int __sccp_device_destroy(const void *ptr)
 
 	sccp_log((DEBUGCAT_DEVICE + DEBUGCAT_CONFIG)) (VERBOSE_PREFIX_1 "%s: Destroying Device\n", d->id);
 
+	/* cleanup dynamic allocated during sccp_config (i.e. STRINGPTR) */
+	sccp_config_cleanup_dynamically_allocated_memory(d, SCCP_CONFIG_DEVICE_SEGMENT);
+
 	/* remove button config */
 	/* only generated on read config, so do not remove on reset/restart */
 	SCCP_LIST_LOCK(&d->buttonconfig);
@@ -2095,7 +2125,7 @@ int __sccp_device_destroy(const void *ptr)
 	sccp_mutex_lock(&d->messageStackLock);
 #endif
 	for (i = 0; i < SCCP_MAX_MESSAGESTACK; i++) {
-		if (d->messageStack && d->messageStack[i] != NULL) {
+		if (d->messageStack[i] != NULL) {
 			sccp_free(d->messageStack[i]);
 		}
 	}
@@ -2108,9 +2138,11 @@ int __sccp_device_destroy(const void *ptr)
 		pbx_variables_destroy(d->variables);
 		d->variables = NULL;
 	}
-	//if (PBX(endpoint_shutdown)) {
-	//  PBX(endpoint_shutdown)(d->endpoint);
-	//}
+	/*
+	if (PBX(endpoint_shutdown)) {
+		PBX(endpoint_shutdown)(d->endpoint);
+	}
+	*/
 
 	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Device Destroyed\n", d->id);
 	return 0;
@@ -2457,12 +2489,7 @@ static void sccp_device_indicate_connected(const sccp_device_t * device, sccp_li
 static void __sccp_device_indicate_immediate_dialing(const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel)
 {
 	sccp_device_setLamp(device, SKINNY_STIMULUS_LINE, lineInstance, SKINNY_LAMP_BLINK);
-	if (device->protocol) {
-		if (device->protocol->sendDialedNumber) {
-			device->protocol->sendDialedNumber(device, channel);
-		}
-	}
-	sccp_dev_set_keyset(device, lineInstance, channel->callid, KEYMODE_DIGITSFOLL);
+	sccp_dev_set_keyset(device, lineInstance, channel->callid, KEYMODE_OFFHOOK);
 }
 
 static void __sccp_device_indicate_normal_dialing(const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel)
