@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-02-18 14:01:08 +0100 (Wed, 18 Feb 2015) $
+ * $Revision: 5936 $
  */
 
 #ifndef __SCCP_DEVICE_H
@@ -57,7 +57,7 @@ sccp_msg_t *sccp_build_packet(sccp_mid_t t, size_t pkt_len);
 
 void sccp_dev_check_displayprompt(const sccp_device_t * d);
 void sccp_device_setLastNumberDialed(sccp_device_t * device, const char *lastNumberDialed);
-void sccp_device_setIndicationProtocol(sccp_device_t * device);
+void sccp_device_preregistration(sccp_device_t * device);
 void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn);
 void sccp_dev_sendmsg(const sccp_device_t * d, sccp_mid_t t);
 void sccp_dev_set_keyset(const sccp_device_t * d, uint8_t lineInstance, uint32_t callid, uint8_t softKeySetIndex);
@@ -76,7 +76,7 @@ void sccp_dev_displayprompt_debug(const sccp_device_t * d, const uint8_t lineIns
 void sccp_dev_displaynotify_debug(const sccp_device_t * d, const char *msg, const uint8_t timeout, const char *file, const int lineno, const char *pretty_function);
 void sccp_dev_displayprinotify_debug(const sccp_device_t * d, const char *msg, const uint8_t priority, const uint8_t timeout, const char *file, const int lineno, const char *pretty_function);
 void sccp_dev_cleardisplaynotify(const sccp_device_t * d);
-void sccp_dev_cleardisplayprinotify(const sccp_device_t * d);
+void sccp_dev_cleardisplayprinotify(const sccp_device_t * d, const uint8_t priority);
 void sccp_dev_speed_find_byindex(sccp_device_t * d, uint16_t instance, boolean_t withHint, sccp_speed_t * k);
 void sccp_dev_set_activeline(sccp_device_t * device, const sccp_line_t * l);
 void sccp_dev_forward_status(sccp_line_t * l, uint8_t lineInstance, sccp_device_t * device);
