@@ -6,8 +6,8 @@
  *              See the LICENSE file at the top of the source tree.
  * \since       2009-09-02
  * 
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $  
+ * $Date: 2015-02-12 16:14:18 +0100 (Thu, 12 Feb 2015) $
+ * $Revision: 5919 $  
  */
 
 /*!
@@ -23,12 +23,8 @@
 #include "sccp_device.h"
 #include "sccp_line.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
-#define ENUMMACRO_FILE "sccp_event_enums.hh"
-#define ENUMMACRO_GENERATE ENUMMACRO_GENFUNCTION
-#include "sccp_enum_macro.h"
-#undef  ENUMMACRO_GENERATE
-#undef  ENUMMACRO_FILE
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5919 $");
+
 void sccp_event_destroy(sccp_event_t * event);
 
 /*!
@@ -83,7 +79,7 @@ void sccp_event_destroy(sccp_event_t * event)
 		case SCCP_EVENT_LINE_CHANGED:
 		case SCCP_EVENT_LINE_DELETED:
 			break;
-		case sccp_event_type_LOOKUPERROR:
+		case SCCP_EVENT_TYPE_SENTINEL:
 			break;
 	}
 	//pbx_log(LOG_NOTICE, "Event destroyed- %p type: %d\n", event, event->type);
@@ -266,7 +262,7 @@ void sccp_event_fire(const sccp_event_t * event)
 		case SCCP_EVENT_LINE_CHANGED:
 		case SCCP_EVENT_LINE_DELETED:
 			break;
-		case sccp_event_type_LOOKUPERROR:
+		case SCCP_EVENT_TYPE_SENTINEL:
 			break;
 	}
 
