@@ -6,8 +6,8 @@
  *              See the LICENSE file at the top of the source tree.
  * \since       2009-01-16
  *
- * $Date: 2014-11-20 16:14:19 +0100 (Thu, 20 Nov 2014) $
- * $Revision: 5804 $
+ * $Date: 2015-04-19 13:12:47 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6022 $
  */
 
 /*!
@@ -35,19 +35,18 @@
 #include "sccp_indicate.h"
 #include "sccp_rtp.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5804 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6022 $");
 
-    /*!
-     * \brief Handle Call Forwarding
-     * \param l SCCP Line
-     * \param device SCCP Device
-     * \param type CallForward Type (NONE, ALL, BUSY, NOANSWER) as SCCP_CFWD_*
-     * \return SCCP Channel
-     *
-     * \callgraph
-     * \callergraph
-     *
-     */
+/*!
+ * \brief Handle Call Forwarding
+ * \param l SCCP Line
+ * \param device SCCP Device
+ * \param type CallForward Type (NONE, ALL, BUSY, NOANSWER) as SCCP_CFWD_*
+ * \return SCCP Channel
+ *
+ * \callgraph
+ * \callergraph
+ */
 void sccp_feat_handle_callforward(sccp_line_t * l, sccp_device_t * device, sccp_callforward_t type)
 {
 	if (!l) {
@@ -85,13 +84,13 @@ void sccp_feat_handle_callforward(sccp_line_t * l, sccp_device_t * device, sccp_
 	AUTO_RELEASE sccp_channel_t *c = sccp_device_getActiveChannel(device);
 
 	if (c) {
-		if (c->ss_action == SCCP_SS_GETFORWARDEXTEN) {
+		if (c->softswitch_action == SCCP_SOFTSWITCH_GETFORWARDEXTEN) {
 			// we have a channel, checking if
 			if (c->state == SCCP_CHANNELSTATE_RINGOUT || c->state == SCCP_CHANNELSTATE_CONNECTED || c->state == SCCP_CHANNELSTATE_PROCEED || c->state == SCCP_CHANNELSTATE_BUSY || c->state == SCCP_CHANNELSTATE_CONGESTION) {
 				if (c->calltype == SKINNY_CALLTYPE_OUTBOUND) {
 					pbx_log(LOG_ERROR, "%s: 1\n", DEV_ID_LOG(device));
 					// if we have an outbound call, we can set callforward to dialed number -FS
-					if (c->dialedNumber && !sccp_strlen_zero(c->dialedNumber)) {		// checking if we have a number !
+					if (!sccp_strlen_zero(c->dialedNumber)) {		// checking if we have a number !
 						pbx_log(LOG_ERROR, "%s: 2\n", DEV_ID_LOG(device));
 						sccp_line_cfwd(l, device, type, c->dialedNumber);
 						// we are on call, so no tone has been played until now :)
@@ -128,12 +127,13 @@ void sccp_feat_handle_callforward(sccp_line_t * l, sccp_device_t * device, sccp_
 				pbx_log(LOG_ERROR, "%s: 5\n", DEV_ID_LOG(device));
 				// we are dialing but without entering a number :D -FS
 				sccp_dev_stoptone(device, linedevice->lineInstance, (c && c->callid) ? c->callid : 0);
-				// changing SS_DIALING mode to SS_GETFORWARDEXTEN
-				c->ss_action = SCCP_SS_GETFORWARDEXTEN;						/* Simpleswitch will catch a number to be dialed */
+				// changing SOFTSWITCH_DIALING mode to SOFTSWITCH_GETFORWARDEXTEN
+				c->softswitch_action = SCCP_SOFTSWITCH_GETFORWARDEXTEN;				/* SoftSwitch will catch a number to be dialed */
 				c->ss_data = type;								/* this should be found in thread */
 				// changing channelstate to GETDIGITS
 				//sccp_indicate(device, c, SCCP_CHANNELSTATE_OFFHOOK);                          /* Removal requested by Antonio */
 				sccp_indicate(device, c, SCCP_CHANNELSTATE_GETDIGITS);
+				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				return;
 			} else {
 				// we cannot allocate a channel, or ask an extension to pickup.
@@ -161,7 +161,7 @@ void sccp_feat_handle_callforward(sccp_line_t * l, sccp_device_t * device, sccp_
 	} else {
 		if (c->state == SCCP_CHANNELSTATE_OFFHOOK) {
 
-			/** we just opened a channel for cfwd, switch ss_action = SCCP_SS_GETFORWARDEXTEN */
+			/** we just opened a channel for cfwd, switch softswitch_action = SCCP_SOFTSWITCH_GETFORWARDEXTEN */
 			sccp_channel_stop_schedule_digittimout(c);
 			// we are dialing but without entering a number :D -FS
 			sccp_dev_stoptone(device, linedevice->lineInstance, c->callid);
@@ -177,11 +177,12 @@ void sccp_feat_handle_callforward(sccp_line_t * l, sccp_device_t * device, sccp_
 		}
 	}
 
-	c->ss_action = SCCP_SS_GETFORWARDEXTEN;									/* Simpleswitch will catch a number to be dialed */
+	c->softswitch_action = SCCP_SOFTSWITCH_GETFORWARDEXTEN;							/* SoftSwitch will catch a number to be dialed */
 	c->ss_data = type;											/* this should be found in thread */
 
 	c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 	sccp_indicate(device, c, SCCP_CHANNELSTATE_GETDIGITS);
+	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 	sccp_dev_displayprompt(device, linedevice->lineInstance, c->callid, SKINNY_DISP_ENTER_NUMBER_TO_FORWARD_TO, SCCP_DISPLAYSTATUS_TIMEOUT);
 
 	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
@@ -215,14 +216,15 @@ void sccp_feat_handle_directed_pickup(sccp_line_t * l, uint8_t lineInstance, scc
 
 		if (c) {
 			// we have a channel, checking if
-			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && (!c->dialedNumber || (c->dialedNumber && sccp_strlen_zero(c->dialedNumber)))) {
+			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && sccp_strlen_zero(c->dialedNumber)) {
 				// we are dialing but without entering a number :D -FS
 				sccp_dev_stoptone(d, lineInstance, (c && c->callid) ? c->callid : 0);
-				// changing SS_DIALING mode to SS_GETFORWARDEXTEN
-				c->ss_action = SCCP_SS_GETPICKUPEXTEN;						/* Simpleswitch will catch a number to be dialed */
+				// changing SOFTSWITCH_DIALING mode to SOFTSWITCH_GETFORWARDEXTEN
+				c->softswitch_action = SCCP_SOFTSWITCH_GETPICKUPEXTEN;				/* SoftSwitch will catch a number to be dialed */
 				c->ss_data = 0;									/* this should be found in thread */
 				// changing channelstate to GETDIGITS
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				return;
 			} else {
 				/* there is an active call, let's put it on hold first */
@@ -240,11 +242,12 @@ void sccp_feat_handle_directed_pickup(sccp_line_t * l, uint8_t lineInstance, scc
 		return;
 	}
 
-	c->ss_action = SCCP_SS_GETPICKUPEXTEN;									/* Simpleswitch will catch a number to be dialed */
+	c->softswitch_action = SCCP_SOFTSWITCH_GETPICKUPEXTEN;							/* SoftSwitch will catch a number to be dialed */
 	c->ss_data = 0;												/* not needed here */
 
 	c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 	sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 
 	/* ok the number exist. allocate the asterisk channel */
 	if (!sccp_pbx_channel_allocate(c, NULL, NULL)) {
@@ -340,14 +343,12 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 
 	char *name = NULL;
 	char *number = NULL;
-	sccp_channel_t *tmpChannel;
 
+	pbx_log(LOG_NOTICE, "SCCP: (directed_pickup)\n");
 	target = PBX(findPickupChannelByExtenLocked) (original, exten, context);
 	if (target) {
-		tmp = (CS_AST_BRIDGED_CHANNEL(target) ? CS_AST_BRIDGED_CHANNEL(target) : target);
-
 		/* fixup callinfo */
-		tmpChannel = CS_AST_CHANNEL_PVT(target);
+		AUTO_RELEASE sccp_channel_t *tmpChannel = get_sccp_channel_from_pbx_channel(target);
 		if (tmpChannel) {
 			if (PBX(get_callerid_name)) {
 				PBX(get_callerid_name) (tmpChannel, &name);
@@ -356,7 +357,13 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 				PBX(get_callerid_number) (tmpChannel, &number);
 			}
 		}
-		pbx_log(LOG_NOTICE, "SCCP: (directed_pickup) %s callerid is ('%s'-'%s')\n", pbx_channel_name(tmp), name ? name : "", number ? number : "");
+
+		if ((tmp = PBX(get_bridged_channel)(target))) {
+			pbx_log(LOG_NOTICE, "SCCP: (directed_pickup) %s callerid is ('%s'-'%s')\n", pbx_channel_name(tmp), name ? name : "", number ? number : "");
+			pbx_channel_unref(tmp);
+		} else {	
+			pbx_log(LOG_NOTICE, "SCCP: (directed_pickup) %s callerid is ('%s'-'%s')\n", pbx_channel_name(target), name ? name : "", number ? number : "");
+		}
 		tmp = NULL;
 		res = 0;
 		if (d->directed_pickup_modeanswer) {
@@ -375,12 +382,22 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 			c->state = SCCP_CHANNELSTATE_PROCEED;
 			c->answered_elsewhere = TRUE;
 
+			AUTO_RELEASE sccp_device_t *orig_device = NULL;
+			AUTO_RELEASE sccp_channel_t *orig_channel = get_sccp_channel_from_pbx_channel(original);
+			if (orig_channel) {
+				orig_device = sccp_channel_getDevice_retained(orig_channel);
+			}
+
 			res = ast_do_pickup(original, target);
 			if (!res) {
 				/* directed pickup succeeded */
-				sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (directed_pickup) pickup succeeded on callid: %d\n", DEV_ID_LOG(d), c->callid);
+				sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (directed_pickup) pickup succeeded on call: %s\n", DEV_ID_LOG(d), c->designator);
 				sccp_rtp_stop(c);								/* stop previous audio */
 				pbx_channel_set_hangupcause(original, AST_CAUSE_ANSWERED_ELSEWHERE);
+				if (orig_device && orig_channel) {
+					//sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (directed_pickup) hangup: %s\n", DEV_ID_LOG(orig_device), orig_channel->designator);
+					sccp_indicate(orig_device, orig_channel, SCCP_CHANNELSTATE_ONHOOK);
+				}
 				pbx_hangup(original);								/* hangup masqueraded zombie channel */
 
 				pbx_channel_set_hangupcause(c->owner, AST_CAUSE_NORMAL_CLEARING);		/* reset picked up channel */
@@ -388,6 +405,10 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 				sccp_channel_updateChannelCapability(c);
 				if (d->directed_pickup_modeanswer) {
 					sccp_indicate(d, c, SCCP_CHANNELSTATE_CONNECTED);
+					
+					/* force 7940/7960 to display the callplane (something is suppressing it along the way, have not been able to find what, yet) */
+					uint8_t instance = sccp_device_find_index_for_line(d, c->line->name);
+					sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_NORMAL, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 				} else {
 					uint8_t instance;
 
@@ -409,6 +430,7 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 						}
 					}
 					sccp_indicate(d, c, SCCP_CHANNELSTATE_RINGING);
+					PBX(set_callstate) (c, AST_STATE_RINGING);
 				}
 			}
 		} else {
@@ -417,7 +439,7 @@ int sccp_feat_directed_pickup(sccp_channel_t * c, char *exten)
 		pbx_channel_unlock(target);
 		target = pbx_channel_unref(target);
 	} else {
-		sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (directed_pickup) findPickupChannelByExtenLocked failed on callid: %d\n", DEV_ID_LOG(d), c->callid);
+		sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (directed_pickup) findPickupChannelByExtenLocked failed on callid: %s\n", DEV_ID_LOG(d), c->designator);
 	}
 	if (!res) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (directed_pickup) Exten '%s@%s' Picked up Succesfully\n", exten, context);
@@ -451,7 +473,7 @@ int sccp_feat_grouppickup(sccp_line_t * l, sccp_device_t * d)
 {
 	int res = 0;
 
-	if (!l || !d || !d->id || sccp_strlen_zero(d->id)) {
+	if (!l || !d || sccp_strlen_zero(d->id)) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (grouppickup) no line or device\n");
 		return -1;
 	}
@@ -473,7 +495,10 @@ int sccp_feat_grouppickup(sccp_line_t * l, sccp_device_t * d)
 		sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (grouppickup) Already offhook, reusing callid %d\n", d->id, c->callid);
 	} else {
 		sccp_log((DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (grouppickup) Starting new channel\n", d->id);
-		c = sccp_channel_newcall(l, d, NULL, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL);
+		if (!(c = sccp_channel_newcall(l, d, NULL, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL))) {
+			pbx_log(LOG_ERROR, "%s: Could not create a new channel to use for grouppickup\n", d->id);
+			return -1;
+		}
 	}
 
 	sccp_channel_stop_schedule_digittimout(c);
@@ -502,8 +527,13 @@ int sccp_feat_grouppickup(sccp_line_t * l, sccp_device_t * d)
 	} else {
 		/* emulate a new call so we end up in the same state as when a new call has been started */
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: (grouppickup) Starting new channel\n", d->id);
-		c = sccp_channel_newcall(l, d, NULL, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL);
-		dest = c->owner;
+		if ((c = sccp_channel_newcall(l, d, NULL, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL))) {
+			dest = c->owner;
+		}
+	}
+	if (!dest) {
+		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: (grouppickup) Failed to get offhook channel or create new channel\n", d->id);
+		return -2;
 	}
 	/* prepare for call pickup */
 	sccp_channel_stop_schedule_digittimout(c);
@@ -700,7 +730,7 @@ void sccp_feat_idivert(sccp_device_t * d, sccp_line_t * l, sccp_channel_t * c)
 void sccp_feat_handle_conference(sccp_device_t * d, sccp_line_t * l, uint8_t lineInstance, sccp_channel_t * channel)
 {
 #ifdef CS_SCCP_CONFERENCE
-	if (!l || !d || !d->id || sccp_strlen_zero(d->id)) {
+	if (!l || !d || sccp_strlen_zero(d->id)) {
 		pbx_log(LOG_ERROR, "SCCP: Can't allocate SCCP channel if line or device are not defined!\n");
 		return;
 	}
@@ -727,10 +757,11 @@ void sccp_feat_handle_conference(sccp_device_t * d, sccp_line_t * l, uint8_t lin
 	AUTO_RELEASE sccp_channel_t *c = sccp_channel_allocate(l, d);
 
 	if (c) {
-		c->ss_action = SCCP_SS_GETCONFERENCEROOM;							/* Simpleswitch will catch a number to be dialed */
+		c->softswitch_action = SCCP_SOFTSWITCH_GETCONFERENCEROOM;					/* SoftSwitch will catch a number to be dialed */
 		c->ss_data = 0;											/* not needed here */
 		c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 		sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+		PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 
 		/* ok the number exist. allocate the asterisk channel */
 		if (!sccp_pbx_channel_allocate(c, NULL, NULL)) {
@@ -789,9 +820,10 @@ void sccp_feat_conference_start(sccp_device_t * device, sccp_line_t * l, const u
 			channel = selectedChannel->channel;
 			if (channel && channel != c) {
 				if (channel != d->active_channel && channel->state == SCCP_CHANNELSTATE_HOLD) {
-					if ((bridged_channel = CS_AST_BRIDGED_CHANNEL(channel->owner))) {
+					if ((bridged_channel = PBX(get_bridged_channel)(channel->owner))) {
 						sccp_log((DEBUGCAT_CONFERENCE + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: sccp conference: channel %s, state: %s.\n", DEV_ID_LOG(d), pbx_channel_name(bridged_channel), sccp_channelstate2str(channel->state));
 						sccp_conference_addParticipatingChannel(d->conference, c, channel, bridged_channel);
+						pbx_channel_unref(bridged_channel);
 					} else {
 						pbx_log(LOG_ERROR, "%s: sccp conference: bridgedchannel for channel %s could not be found\n", DEV_ID_LOG(d), pbx_channel_name(channel->owner));
 					}
@@ -816,9 +848,10 @@ void sccp_feat_conference_start(sccp_device_t * device, sccp_line_t * l, const u
 						SCCP_LIST_LOCK(&line->channels);
 						SCCP_LIST_TRAVERSE(&line->channels, channel, list) {
 							if (channel != d->active_channel && channel->state == SCCP_CHANNELSTATE_HOLD) {
-								if ((bridged_channel = CS_AST_BRIDGED_CHANNEL(channel->owner))) {
+								if ((bridged_channel = PBX(get_bridged_channel)(channel->owner))) {
 									sccp_log((DEBUGCAT_CONFERENCE + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: sccp conference: channel %s, state: %s.\n", DEV_ID_LOG(d), pbx_channel_name(bridged_channel), sccp_channelstate2str(channel->state));
 									sccp_conference_addParticipatingChannel(d->conference, c, channel, bridged_channel);
+									pbx_channel_unref(bridged_channel);
 								} else {
 									pbx_log(LOG_ERROR, "%s: sccp conference: bridgedchannel for channel %s could not be found\n", DEV_ID_LOG(d), pbx_channel_name(channel->owner));
 								}
@@ -894,9 +927,10 @@ void sccp_feat_join(sccp_device_t * device, sccp_line_t * l, uint8_t lineInstanc
 				sccp_channel_resume(d, moderator_channel, TRUE);				// swap active channel
 
 				pbx_log(LOG_NOTICE, "%s: Joining new participant to conference %d.\n", DEV_ID_LOG(d), d->conference->id);
-				if ((bridged_channel = CS_AST_BRIDGED_CHANNEL(newparticipant_channel->owner))) {
+				if ((bridged_channel = PBX(get_bridged_channel)(newparticipant_channel->owner))) {
 					sccp_log((DEBUGCAT_CONFERENCE + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: sccp conference: channel %s, state: %s.\n", DEV_ID_LOG(d), pbx_channel_name(bridged_channel), sccp_channelstate2str(newparticipant_channel->state));
 					sccp_conference_addParticipatingChannel(d->conference, moderator_channel, newparticipant_channel, bridged_channel);
+					pbx_channel_unref(bridged_channel);
 				} else {
 					pbx_log(LOG_ERROR, "%s: sccp conference: bridgedchannel for channel %s could not be found\n", DEV_ID_LOG(d), pbx_channel_name(newparticipant_channel->owner));
 				}
@@ -956,7 +990,7 @@ void sccp_feat_conflist(sccp_device_t * d, sccp_line_t * l, uint8_t lineInstance
  */
 void sccp_feat_handle_meetme(sccp_line_t * l, uint8_t lineInstance, sccp_device_t * d)
 {
-	if (!l || !d || !d->id || sccp_strlen_zero(d->id)) {
+	if (!l || !d || sccp_strlen_zero(d->id)) {
 		pbx_log(LOG_ERROR, "SCCP: Can't allocate SCCP channel if line or device are not defined!\n");
 		return;
 	}
@@ -967,14 +1001,15 @@ void sccp_feat_handle_meetme(sccp_line_t * l, uint8_t lineInstance, sccp_device_
 
 		if (c) {
 			// we have a channel, checking if
-			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && (!c->dialedNumber || (c->dialedNumber && sccp_strlen_zero(c->dialedNumber)))) {
+			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && sccp_strlen_zero(c->dialedNumber)) {
 				// we are dialing but without entering a number :D -FS
 				sccp_dev_stoptone(d, lineInstance, (c && c->callid) ? c->callid : 0);
-				// changing SS_DIALING mode to SS_GETFORWARDEXTEN
-				c->ss_action = SCCP_SS_GETMEETMEROOM;						/* Simpleswitch will catch a number to be dialed */
+				// changing SOFTSWITCH_DIALING mode to SOFTSWITCH_GETFORWARDEXTEN
+				c->softswitch_action = SCCP_SOFTSWITCH_GETMEETMEROOM;				/* SoftSwitch will catch a number to be dialed */
 				c->ss_data = 0;									/* this should be found in thread */
 				// changing channelstate to GETDIGITS
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				return;
 				/* there is an active call, let's put it on hold first */
 			} else if (!sccp_channel_hold(c)) {
@@ -991,13 +1026,14 @@ void sccp_feat_handle_meetme(sccp_line_t * l, uint8_t lineInstance, sccp_device_
 		return;
 	}
 
-	c->ss_action = SCCP_SS_GETMEETMEROOM;									/* Simpleswitch will catch a number to be dialed */
+	c->softswitch_action = SCCP_SOFTSWITCH_GETMEETMEROOM;							/* SoftSwitch will catch a number to be dialed */
 	c->ss_data = 0;												/* not needed here */
 
 	c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 
 	//sccp_device_setActiveChannel(d, c);
 	sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 
 	/* ok the number exist. allocate the asterisk channel */
 	if (!sccp_pbx_channel_allocate(c, NULL, NULL)) {
@@ -1161,7 +1197,7 @@ void sccp_feat_meetme_start(sccp_channel_t * c)
 void sccp_feat_handle_barge(sccp_line_t * l, uint8_t lineInstance, sccp_device_t * d)
 {
 
-	if (!l || !d || !d->id || sccp_strlen_zero(d->id)) {
+	if (!l || !d || sccp_strlen_zero(d->id)) {
 		pbx_log(LOG_ERROR, "SCCP: Can't allocate SCCP channel if line or device are not defined!\n");
 		return;
 	}
@@ -1172,14 +1208,15 @@ void sccp_feat_handle_barge(sccp_line_t * l, uint8_t lineInstance, sccp_device_t
 
 		if (c) {
 			// we have a channel, checking if
-			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && (!c->dialedNumber || (c->dialedNumber && sccp_strlen_zero(c->dialedNumber)))) {
+			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && sccp_strlen_zero(c->dialedNumber)) {
 				// we are dialing but without entering a number :D -FS
 				sccp_dev_stoptone(d, lineInstance, (c && c->callid) ? c->callid : 0);
-				// changing SS_DIALING mode to SS_GETFORWARDEXTEN
-				c->ss_action = SCCP_SS_GETBARGEEXTEN;						/* Simpleswitch will catch a number to be dialed */
+				// changing SOFTSWITCH_DIALING mode to SOFTSWITCH_GETFORWARDEXTEN
+				c->softswitch_action = SCCP_SOFTSWITCH_GETBARGEEXTEN;				/* SoftSwitch will catch a number to be dialed */
 				c->ss_data = 0;									/* this should be found in thread */
 				// changing channelstate to GETDIGITS
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				return;
 			} else if (!sccp_channel_hold(c)) {
 				/* there is an active call, let's put it on hold first */
@@ -1196,11 +1233,12 @@ void sccp_feat_handle_barge(sccp_line_t * l, uint8_t lineInstance, sccp_device_t
 		return;
 	}
 
-	c->ss_action = SCCP_SS_GETBARGEEXTEN;									/* Simpleswitch will catch a number to be dialed */
+	c->softswitch_action = SCCP_SOFTSWITCH_GETBARGEEXTEN;							/* SoftSwitch will catch a number to be dialed */
 	c->ss_data = 0;												/* not needed here */
 
 	c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 	sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 
 	/* ok the number exist. allocate the asterisk channel */
 	if (!sccp_pbx_channel_allocate(c, NULL, NULL)) {
@@ -1263,14 +1301,15 @@ void sccp_feat_handle_cbarge(sccp_line_t * l, uint8_t lineInstance, sccp_device_
 
 		if (c) {
 			// we have a channel, checking if
-			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && (!c->dialedNumber || (c->dialedNumber && sccp_strlen_zero(c->dialedNumber)))) {
+			if (c->state == SCCP_CHANNELSTATE_OFFHOOK && sccp_strlen_zero(c->dialedNumber)) {
 				// we are dialing but without entering a number :D -FS
 				sccp_dev_stoptone(d, lineInstance, (c && c->callid) ? c->callid : 0);
-				// changing SS_DIALING mode to SS_GETFORWARDEXTEN
-				c->ss_action = SCCP_SS_GETBARGEEXTEN;						/* Simpleswitch will catch a number to be dialed */
+				// changing SOFTSWITCH_DIALING mode to SOFTSWITCH_GETFORWARDEXTEN
+				c->softswitch_action = SCCP_SOFTSWITCH_GETBARGEEXTEN;				/* SoftSwitch will catch a number to be dialed */
 				c->ss_data = 0;									/* this should be found in thread */
 				// changing channelstate to GETDIGITS
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				return;
 			} else if (!sccp_channel_hold(c)) {
 				/* there is an active call, let's put it on hold first */
@@ -1287,13 +1326,14 @@ void sccp_feat_handle_cbarge(sccp_line_t * l, uint8_t lineInstance, sccp_device_
 		return;
 	}
 
-	c->ss_action = SCCP_SS_GETCBARGEROOM;									/* Simpleswitch will catch a number to be dialed */
+	c->softswitch_action = SCCP_SOFTSWITCH_GETCBARGEROOM;							/* SoftSwitch will catch a number to be dialed */
 	c->ss_data = 0;												/* not needed here */
 
 	c->calltype = SKINNY_CALLTYPE_OUTBOUND;
 
 	//sccp_device_setActiveChannel(d, c);
 	sccp_indicate(d, c, SCCP_CHANNELSTATE_GETDIGITS);
+	PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 
 	/* ok the number exist. allocate the asterisk channel */
 	if (!sccp_pbx_channel_allocate(c, NULL, NULL)) {
