@@ -9,8 +9,8 @@
  * \remarks     Purpose:        SCCP Hint
  *              When to use:    Does the business of hint status
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-19 15:09:41 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6025 $
  */
 
 /*!
@@ -34,14 +34,17 @@
 #include "sccp_utils.h"
 #include "sccp_indicate.h"											// only for SCCP_CHANNELSTATE_Idling
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6025 $");
 
-    /* ========================================================================================================================= Struct Definitions */
-    /*!
-     *\brief SCCP Hint Subscribing Device Structure
-     */
+/* ========================================================================================================================= Struct Definitions */
+/*!
+ *\brief SCCP Hint Subscribing Device Structure
+ */
 typedef struct sccp_hint_SubscribingDevice sccp_hint_SubscribingDevice_t;
 typedef struct sccp_hint_list sccp_hint_list_t;
+#ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+static char default_eid_str[32];
+#endif
 
 struct sccp_hint_SubscribingDevice {
 
@@ -95,7 +98,7 @@ struct sccp_hint_list {
 
 	int stateid;												/*!< subscription id in asterisk */
 #ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
-	struct pbx_event_sub *device_state_sub;									/*!< asterisk distributed device state subscription */
+	PBX_EVENT_SUBSCRIPTION *device_state_sub;									/*!< asterisk distributed device state subscription */
 #endif
 
 	SCCP_LIST_HEAD (, sccp_hint_SubscribingDevice_t) subscribers;						/*!< Hint Type Subscribers Linked List Entry */
@@ -119,19 +122,53 @@ static void sccp_hint_eventListener(const sccp_event_t * event);
 static inline boolean_t sccp_hint_isCIDavailabe(const sccp_device_t * device, const uint8_t positionOnDevice);
 
 #ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+#if ASTERISK_VERSION_GROUP >= 112
+static void sccp_hint_distributed_devstate_cb(void *data, struct stasis_subscription *sub, struct stasis_message *msg)
+#else
 static void sccp_hint_distributed_devstate_cb(const pbx_event_t * event, void *data)
+#endif
 {
-
-	sccp_hint_list_t *hint;
+	sccp_hint_list_t *hint = (sccp_hint_list_t *) data;
 	const char *cidName;
 	const char *cidNumber;
-
-	hint = (sccp_hint_list_t *) data;
-
+	//enum ast_device_state state;		/* maybe we should store the last state */
+	
+#if ASTERISK_VERSION_GROUP >= 112
+	struct ast_device_state_message *dev_state = stasis_message_data(msg);
+	if (ast_device_state_message_type() != stasis_message_type(msg)) {
+		return;
+	}
+	if (dev_state->eid) {
+		return;
+	}
+	//eid = dev_state->eid;
+	//state = dev_state->state;
+	//cidName = dev_state->
+	//cidNumber = dev_state->
+	cidName = "";
+	cidNumber = "";
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "Got new hint event %s, cidname: %s, cidnum: %s\n", hint->hint_dialplan, cidName ? cidName : "NULL", cidNumber ? cidNumber : "NULL");
+#else
+	const struct ast_eid *eid;
+	eid = ast_event_get_ie_raw(event, AST_EVENT_IE_EID);
+	//state = pbx_event_get_ie_uint(ast_event, AST_EVENT_IE_STATE);
+#if ASTERISK_VERSION_GROUP >= 108
 	cidName = pbx_event_get_ie_str(event, AST_EVENT_IE_CEL_CIDNAME);
 	cidNumber = pbx_event_get_ie_str(event, AST_EVENT_IE_CEL_CIDNUM);
-
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "Got new hint event %s, cidname: %s, cidnum: %s\n", hint->hint_dialplan, cidName ? cidName : "NULL", cidNumber ? cidNumber : "NULL");
+#else
+	cidName = "";
+	cidNumber = "";
+#endif
+	char eid_str[32] = "";
+	ast_eid_to_str(eid_str, sizeof(eid_str), (struct ast_eid *) eid);
+	if (!ast_eid_cmp(&ast_eid_default, eid)) {
+		// If the event originate from this server, skip updating (Was already done in sccp_hint_notifyPBX).
+		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "Skipping distribute devstate update from EID:'%s', MYEID:'%s' (i.e. myself)\n", eid_str, default_eid_str);
+		return;
+	}
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "Got new hint event %s, cidname: %s, cidnum: %s, originated from EID:'%s'\n", hint->hint_dialplan, cidName ? cidName : "NULL", cidNumber ? cidNumber : "NULL", eid_str);
+#endif
+	
 
 	if (cidName) {
 		sccp_copy_string(hint->callInfo.partyName, cidName, sizeof(hint->callInfo.partyName));
@@ -167,6 +204,9 @@ void sccp_hint_module_start(void)
 	SCCP_LIST_HEAD_INIT(&lineStates);
 	SCCP_LIST_HEAD_INIT(&sccp_hint_subscriptions);
 	sccp_event_subscribe(SCCP_EVENT_DEVICE_REGISTERED | SCCP_EVENT_DEVICE_UNREGISTERED | SCCP_EVENT_DEVICE_DETACHED | SCCP_EVENT_DEVICE_ATTACHED | SCCP_EVENT_LINESTATUS_CHANGED | SCCP_EVENT_FEATURE_CHANGED, sccp_hint_eventListener, TRUE);
+#ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+	ast_eid_to_str(default_eid_str, sizeof(default_eid_str), &ast_eid_default);
+#endif
 }
 
 /*!
@@ -338,7 +378,7 @@ static void sccp_hint_eventListener(const sccp_event_t * event)
 		case SCCP_EVENT_DEVICE_UNREGISTERED:
 			device = event->event.deviceRegistered.device;
 
-			if (device && device->id) {
+			if (device) {
 				char *deviceName = strdupa(device->id);
 
 				sccp_hint_deviceUnRegistered(deviceName);
@@ -487,10 +527,9 @@ static void sccp_hint_addSubscription4Device(const sccp_device_t * device, const
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_addSubscription4Device) create new hint for %s@%s\n", DEV_ID_LOG(device), hint_exten, hint_context);
 		hint = sccp_hint_create(hint_exten, hint_context);
 		if (!hint) {
-			pbx_log(LOG_ERROR, "%s: (sccp_hint_addSubscription4Device) hint create failed for %s@%s\n", DEV_ID_LOG(device), hint_exten, hint_context);
+			pbx_log(LOG_NOTICE, "%s: (sccp_hint_addSubscription4Device) hint create failed for %s@%s\n", DEV_ID_LOG(device), hint_exten, hint_context);
 			return;
 		}
-
 		SCCP_LIST_LOCK(&sccp_hint_subscriptions);
 		SCCP_LIST_INSERT_HEAD(&sccp_hint_subscriptions, hint, list);
 		SCCP_LIST_UNLOCK(&sccp_hint_subscriptions);
@@ -515,6 +554,7 @@ static void sccp_hint_addSubscription4Device(const sccp_device_t * device, const
 	SCCP_LIST_INSERT_HEAD(&hint->subscribers, subscriber, list);
 
 	sccp_dev_set_keyset(device, subscriber->instance, 0, KEYMODE_ONHOOK);
+
 	sccp_hint_notifySubscribers(hint);
 }
 
@@ -545,6 +585,15 @@ static sccp_hint_list_t *sccp_hint_create(char *hint_exten, char *hint_context)
 		return NULL;
 	}
 
+	/* remove the end after comma (i.e. cutting of the CustomPresence part from 'exten => 143,hint,SCCP/143,CustomPresence:143') */
+	int i=0;
+	while (hint_dialplan[i++]) {
+		if (hint_dialplan[i] == ',') {
+			hint_dialplan[i] = '\0';
+			break;
+		}
+	}
+
 	hint = sccp_malloc(sizeof(sccp_hint_list_t));
 	if (!hint) {
 		pbx_log(LOG_ERROR, "SCCP: (sccp_hint_create) Memory Allocation Error while creating hint list for hint: %s@%s\n", hint_exten, hint_context);
@@ -559,12 +608,24 @@ static sccp_hint_list_t *sccp_hint_create(char *hint_exten, char *hint_context)
 	sccp_copy_string(hint->context, hint_context, sizeof(hint->context));
 	sccp_copy_string(hint->hint_dialplan, hint_dialplan, sizeof(hint_dialplan));
 
+	/* subscripbe to the hint */
 	hint->stateid = pbx_extension_state_add(hint->context, hint->exten, sccp_hint_devstate_cb, hint);
 
 #ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+	/* subscripbe to the distributed hint event */
+#if ASTERISK_VERSION_GROUP >= 112
+#if CS_EXPERIMENTAL
+	struct stasis_topic *devstate_hint_dialplan = ast_device_state_topic(hint->hint_dialplan));
+	hint->device_state_sub = stasis_subscribe(devstate_hint_dialplan, sccp_hint_distributed_devstate_cb, hint);
+#else
+	hint->device_state_sub = stasis_subscribe(ast_device_state_topic_all(), sccp_hint_distributed_devstate_cb, hint);
+#endif
+#else
 	hint->device_state_sub = pbx_event_subscribe(AST_EVENT_DEVICE_STATE_CHANGE, sccp_hint_distributed_devstate_cb, "sccp_hint_distributed_devstate_cb", hint, AST_EVENT_IE_DEVICE, AST_EVENT_IE_PLTYPE_STR, hint->hint_dialplan, AST_EVENT_IE_END);
+#endif
 #endif
 
+	/* force hint update to get currentState */
 #if ASTERISK_VERSION_GROUP >= 111
 	struct ast_state_cb_info info;
 
@@ -632,7 +693,7 @@ static void sccp_hint_updateLineState(struct sccp_hint_lineState *lineState)
 		/* no line, or line without devices */
 		if (0 == SCCP_LIST_GETSIZE(&line->devices)) {
 			lineState->state = SCCP_CHANNELSTATE_CONGESTION;
-			lineState->callInfo.calltype = SKINNY_CALLTYPE_OUTBOUND;
+			lineState->callInfo.calltype = SKINNY_CALLTYPE_SENTINEL;
 
 			sccp_copy_string(lineState->callInfo.partyName, SKINNY_DISP_TEMP_FAIL, sizeof(lineState->callInfo.partyName));
 			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "SCCP: (sccp_hint_updateLineState) 0 devices register on linename: %s\n", line->name);
@@ -817,7 +878,7 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 						sccp_copy_string(lineState->callInfo.partyNumber, "cfwd", sizeof(lineState->callInfo.partyNumber));
 						sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: set speedial partyName: cfwd\n", line->name);
 						break;
-					case skinny_calltype_LOOKUPERROR:
+					case SKINNY_CALLTYPE_SENTINEL:
 						break;
 				}
 				break;
@@ -826,7 +887,7 @@ static void sccp_hint_updateLineStateForSingleChannel(struct sccp_hint_lineState
 			case SCCP_CHANNELSTATE_ZOMBIE:
 			case SCCP_CHANNELSTATE_PROGRESS:
 			case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
-			case sccp_channelstate_LOOKUPERROR:
+			case SCCP_CHANNELSTATE_SENTINEL:
 				/* unused states */
 				break;
 		}
@@ -924,7 +985,7 @@ static enum ast_device_state sccp_hint_hint2DeviceState(sccp_channelstate_t stat
 		case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
 			newDeviceState = AST_DEVICE_INUSE;
 			break;
-		case sccp_channelstate_LOOKUPERROR:
+		case SCCP_CHANNELSTATE_SENTINEL:
 			break;
 	}
 	return newDeviceState;
@@ -937,17 +998,17 @@ static enum ast_device_state sccp_hint_hint2DeviceState(sccp_channelstate_t stat
  */
 static void sccp_hint_notifyPBX(struct sccp_hint_lineState *lineState)
 {
-	char channelName[100];
 	sccp_hint_list_t *hint = NULL;
+	char lineName[StationMaxNameSize + 5];
+	sprintf(lineName, "SCCP/%s", lineState->line->name);
 
-	sprintf(channelName, "SCCP/%s", lineState->line->name);
 	enum ast_device_state newDeviceState = sccp_hint_hint2DeviceState(lineState->state);
-	enum ast_device_state oldDeviceState = AST_DEVICE_NOT_INUSE;
+	enum ast_device_state oldDeviceState = AST_DEVICE_UNAVAILABLE;
 
-#ifndef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+	/* Local Update */
+	SCCP_LIST_LOCK(&sccp_hint_subscriptions);
 	SCCP_LIST_TRAVERSE(&sccp_hint_subscriptions, hint, list) {
-		if (!strncasecmp(channelName, hint->hint_dialplan, sizeof(channelName))) {
-			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "SCCP: (sccp_hint_notifyPBX) %s <==> %s \n", channelName, hint->hint_dialplan);
+		if (sccp_strcaseequals(lineName, hint->hint_dialplan)) {
 			sccp_copy_string(hint->callInfo.partyName, lineState->callInfo.partyName, sizeof(hint->callInfo.partyName));
 			sccp_copy_string(hint->callInfo.partyNumber, lineState->callInfo.partyNumber, sizeof(hint->callInfo.partyNumber));
 
@@ -956,27 +1017,75 @@ static void sccp_hint_notifyPBX(struct sccp_hint_lineState *lineState)
 			break;
 		}
 	}
-#endif
+	SCCP_LIST_UNLOCK(&sccp_hint_subscriptions);
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) Notify asterisk to set state to sccp channelstate '%s' (%d) on line 'SCCP/%s'\n", sccp_channelstate2str(lineState->state), lineState->state, lineState->line->name);
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) => asterisk: '%s' (%d) => '%s' (%d) on line SCCP/%s\n", pbxsccp_devicestate2str(oldDeviceState), oldDeviceState, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
 
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
-
-	// if pbx devicestate does not change, no need to inform asterisk */
-	// if (hint && lineState->state == hint->currentState) {
-	if (hint && newDeviceState == oldDeviceState) {
-		sccp_hint_notifySubscribers(hint);								/* shortcut to inform sccp subscribers about changes e.g. cid update */
+	// if pbx devicestate does not change, no need to inform (local) asterisk */
+	if (newDeviceState == oldDeviceState) {
+		if (hint) {
+			sccp_hint_notifySubscribers(hint);								/* shortcut to inform sccp subscribers about changes e.g. cid update */
+		}
 	} else {
 #ifdef CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE
+		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX (distributed)) Notify asterisk to set state to sccp channelstate '%s' (%d) => asterisk: '%s' (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
+		/* Update Remote Servers Only (EID Will be checked upon Receipt of the CallBack / EID is added automatically)*/
+#if ASTERISK_VERSION_GROUP >= 112
+		/* needs work, to add the rest of the information, ast-13 has fixed templates for the events it seems. Maybe a celupdate wil work instead */
+		/* We need to do something like session_timeout_to_ami in channels/chan_sip */
+		
+		/* during module load we need to define a stasis local type definition*/
+		/*
+		STASIS_MESSAGE_TYPE_DEFN_LOCAL(session_timeout_type,
+			.to_ami = session_timeout_to_ami,
+		);
+		if (STASIS_MESSAGE_TYPE_INIT(session_timeout_type)) {
+			unload_module();
+			return AST_MODULE_LOAD_FAILURE;
+		}
+		*/
+		
+		/* module unload */
+		/*
+		STASIS_MESSAGE_TYPE_CLEANUP(session_timeout_type);
+		*/
+		
+		/* then here create a function for blob creation with the defined structure */
+		/*
+		struct ast_channel_blob *obj = stasis_message_data(msg);
+		const char *source = ast_json_string_get(ast_json_object_get(obj->blob, "source"));
+		ast_manager_event_blob_create(EVENT_FLAG_CALL, "SessionTimeout","%s","Source: %s\r\n",ast_str_buffer(channel_string), source);
+		*/
+		
+		/* here we can actually use the blob */
+		/*
+		RAII_VAR(struct ast_json *, blob, NULL, ast_json_unref);
+		blob = ast_json_pack("{s: s}", "source", source);
+		if (!blob) {
+			return;
+		}
+		ast_channel_publish_blob(chan, session_timeout_type(), blob);
+		*/
+		pbx_devstate_changed_literal(newDeviceState, lineName);
+#else  /* ASTERISK_VERSION_GROUP */
 		pbx_event_t *event;
-
-		event = pbx_event_new(AST_EVENT_DEVICE_STATE, AST_EVENT_IE_DEVICE, AST_EVENT_IE_PLTYPE_STR, channelName, AST_EVENT_IE_STATE, AST_EVENT_IE_PLTYPE_UINT, newDeviceState, AST_EVENT_IE_CEL_CIDNAME, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyName, AST_EVENT_IE_CEL_CIDNUM, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyNumber, AST_EVENT_IE_CEL_USERFIELD, AST_EVENT_IE_PLTYPE_UINT, lineState->callInfo.calltype, AST_EVENT_IE_END);
-		pbx_event_queue_and_cache(event);
-		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "SCCP: \n");
-		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX!distributed) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
-		pbx_devstate_changed_literal(newDeviceState, channelName);					/* come back via pbx callback and update subscribers */
-#else
-		pbx_devstate_changed_literal(newDeviceState, channelName);					/* come back via pbx callback and update subscribers */
+		event = pbx_event_new(AST_EVENT_DEVICE_STATE_CHANGE, 
+			AST_EVENT_IE_DEVICE, AST_EVENT_IE_PLTYPE_STR, lineName, 
+			AST_EVENT_IE_STATE, AST_EVENT_IE_PLTYPE_UINT, newDeviceState, 
+#if ASTERISK_VERSION_GROUP >= 108
+			AST_EVENT_IE_CEL_CIDNAME, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyName, 
+			AST_EVENT_IE_CEL_CIDNUM, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyNumber, 
+			AST_EVENT_IE_CEL_USERFIELD, AST_EVENT_IE_PLTYPE_UINT, lineState->callInfo.calltype, 
+			AST_EVENT_IE_CEL_EXTRA, AST_EVENT_IE_PLTYPE_STR, sccp_channelstate2str(lineState->state),
 #endif
-
+			/* AST_EVENT_IE_PRESENCE_STATE, AST_EVENT_IE_PLTYPE_UINT, ->privacy ?? */
+			AST_EVENT_IE_END);
+		pbx_event_queue_and_cache(event);
+#endif /* ASTERISK_VERSION_GROUP */
+#else  /* CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE */
+		pbx_devstate_changed_literal(newDeviceState, lineName);					/* come back via pbx callback and update subscribers */
+#endif /* CS_USE_ASTERISK_DISTRIBUTED_DEVSTATE */ 
+		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) Notified asterisk to set state to sccp channelstate '%s' (%d) => asterisk: '%s' (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
 	}
 }
 
@@ -995,6 +1104,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 #ifdef CS_DYNAMIC_SPEEDDIAL
 	sccp_speed_t k;
 	char displayMessage[80] = "";
+	int status;
 #endif
 
 	if (!hint) {
@@ -1002,7 +1112,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 		return;
 	}
 
-	if (!&GLOB(module_running) || SCCP_REF_RUNNING != sccp_refcount_isRunning()) {
+	if (!GLOB(module_running) || SCCP_REF_RUNNING != sccp_refcount_isRunning()) {
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_notifySubscribers) Skip processing hint while we are shutting down.\n", hint->exten);
 		return;
 	}
@@ -1021,18 +1131,17 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 
 				REQ(msg, FeatureStatDynamicMessage);
 				if (msg) {
-					msg->data.FeatureStatDynamicMessage.lel_instance = htolel(subscriber->instance);
-					msg->data.FeatureStatDynamicMessage.lel_type = htolel(SKINNY_BUTTONTYPE_BLFSPEEDDIAL);
+					
 
 					switch (hint->currentState) {
 						case SCCP_CHANNELSTATE_ONHOOK:
 							snprintf(displayMessage, sizeof(displayMessage), k.name, sizeof(displayMessage));
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_IDLE);
+							status = SKINNY_BLF_STATUS_IDLE;
 							break;
 
 						case SCCP_CHANNELSTATE_DOWN:
 							snprintf(displayMessage, sizeof(displayMessage), k.name, sizeof(displayMessage));
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_UNKNOWN);	/* default state */
+							status = SKINNY_BLF_STATUS_UNKNOWN;	/* default state */
 							break;
 
 						case SCCP_CHANNELSTATE_RINGING:
@@ -1047,17 +1156,17 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 							} else {
 								snprintf(displayMessage, sizeof(displayMessage), "%s", k.name);
 							}
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_ALERTING);	/* ringin */
+							status = SKINNY_BLF_STATUS_ALERTING;	/* ringin */
 							break;
 
 						case SCCP_CHANNELSTATE_DND:
 							snprintf(displayMessage, sizeof(displayMessage), k.name, sizeof(displayMessage));
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_DND);	/* dnd */
+							status = SKINNY_BLF_STATUS_DND;	/* dnd */
 							break;
 
 						case SCCP_CHANNELSTATE_CONGESTION:
 							snprintf(displayMessage, sizeof(displayMessage), k.name, sizeof(displayMessage));
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_UNKNOWN);	/* device/line not found */
+							status = SKINNY_BLF_STATUS_UNKNOWN;	/* device/line not found */
 							break;
 
 						default:
@@ -1072,12 +1181,33 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 							} else {
 								snprintf(displayMessage, sizeof(displayMessage), "%s", k.name);
 							}
-							msg->data.FeatureStatDynamicMessage.lel_status = htolel(SKINNY_BLF_STATUS_INUSE);	/* connected */
+							status = SKINNY_BLF_STATUS_INUSE;	/* connected */
 							break;
 					}
 					sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) set display name to: \"%s\"\n", DEV_ID_LOG(d), displayMessage);
 					sccp_copy_string(msg->data.FeatureStatDynamicMessage.DisplayName, displayMessage, sizeof(msg->data.FeatureStatDynamicMessage.DisplayName));
+					msg->data.FeatureStatDynamicMessage.DisplayName[strlen(displayMessage)-1] = '\0';
 					sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) notify device: %s@%d state: %d(%d)\n", DEV_ID_LOG(d), DEV_ID_LOG(d), subscriber->instance, hint->currentState, msg->data.FeatureStatDynamicMessage.lel_status);
+					
+					msg->data.FeatureStatDynamicMessage.lel_instance = htolel(subscriber->instance);
+					msg->data.FeatureStatDynamicMessage.lel_type = htolel(SKINNY_BUTTONTYPE_BLFSPEEDDIAL);
+					msg->data.FeatureStatDynamicMessage.lel_status = htolel(status);
+					
+					/*!
+					* hack to fix the white text without shadow issue -MC
+					*
+					* first send a label which is 1-character shorter than the correct one. 
+					* then send another message with a longer label (correct/final label) will force an update (in white over the back drop in black)
+					*/
+					sccp_dev_send(d, msg);
+					
+					
+					REQ(msg, FeatureStatDynamicMessage);
+					sccp_copy_string(msg->data.FeatureStatDynamicMessage.DisplayName, displayMessage, sizeof(msg->data.FeatureStatDynamicMessage.DisplayName));
+					msg->data.FeatureStatDynamicMessage.lel_instance = htolel(subscriber->instance);
+					msg->data.FeatureStatDynamicMessage.lel_type = htolel(SKINNY_BUTTONTYPE_BLFSPEEDDIAL);
+					msg->data.FeatureStatDynamicMessage.lel_status = htolel(status);
+					
 					sccp_dev_send(d, msg);
 				} else {
 					sccp_free(msg);
@@ -1095,7 +1225,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 				   With the old hint style we should only use SCCP_CHANNELSTATE_ONHOOK and SCCP_CHANNELSTATE_CALLREMOTEMULTILINE as callstate,
 				   otherwise we get a callplane on device -> set all states except onhook to SCCP_CHANNELSTATE_CALLREMOTEMULTILINE -MC
 				 */
-				uint32_t iconstate = SKINNY_CALLSTATE_CALLREMOTEMULTILINE;
+				skinny_callstate_t iconstate = SKINNY_CALLSTATE_CALLREMOTEMULTILINE;
 
 				switch (hint->currentState) {
 					case SCCP_CHANNELSTATE_DOWN:
@@ -1132,10 +1262,10 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 					case SCCP_CHANNELSTATE_CALLCONFERENCE:
 						iconstate = SKINNY_CALLSTATE_CALLREMOTEMULTILINE;
 						break;
-					case sccp_channelstate_LOOKUPERROR:
+					case SCCP_CHANNELSTATE_SENTINEL:
 						break;
 				}
-				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) setting icon to state %s (%d)\n", DEV_ID_LOG(d), sccp_channelstate2str(iconstate), iconstate);
+				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) setting icon to state %s (%d)\n", DEV_ID_LOG(d), skinny_callstate2str(iconstate), iconstate);
 
 				if (SCCP_CHANNELSTATE_RINGING == hint->previousState) {
 					/* we send a congestion to the phone, so call will not be marked as missed call */
@@ -1257,29 +1387,30 @@ sccp_channelstate_t sccp_hint_getLinestate(const char *linename, const char *dev
  * \called_from_asterisk
  */
 #include <asterisk/cli.h>
-int sccp_show_hint_lineStates(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_show_hint_lineStates(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
-	int local_total = 0;
+	int local_line_total = 0;
 
-#define CLI_AMI_TABLE_NAME Hint_LineStates
-#define CLI_AMI_TABLE_PER_ENTRY_NAME Hint_lineState
+#define CLI_AMI_TABLE_NAME HintLineStates
+#define CLI_AMI_TABLE_PER_ENTRY_NAME HintLineState
 #define CLI_AMI_TABLE_LIST_ITER_HEAD &lineStates
 #define CLI_AMI_TABLE_LIST_ITER_TYPE struct sccp_hint_lineState
 #define CLI_AMI_TABLE_LIST_ITER_VAR lineState
 #define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
-#define CLI_AMI_TABLE_FIELDS 														\
- 		CLI_AMI_TABLE_FIELD(LineName,		"-10.10s",	10,	lineState->line->name)					\
- 		CLI_AMI_TABLE_FIELD(State,		"-22.22s",	22,	sccp_channelstate2str(lineState->state))		\
- 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15s",	15,	lineState->callInfo.partyNumber)			\
- 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20s",	20,	lineState->callInfo.partyName)				\
- 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10s",	10,	(!SCCP_CHANNELSTATE_Idling(lineState->state) && lineState->callInfo.calltype) ? skinny_calltype2str(lineState->callInfo.calltype) : "INACTIVE")
+#define CLI_AMI_TABLE_FIELDS 															\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-10.10",	s,	10,	lineState->line->name)					\
+ 		CLI_AMI_TABLE_FIELD(State,		"-22.22",	s,	22,	sccp_channelstate2str(lineState->state))		\
+ 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15",	s,	15,	lineState->callInfo.partyNumber)			\
+ 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20",	s,	20,	lineState->callInfo.partyName)				\
+ 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10",	s,	10,	(!SCCP_CHANNELSTATE_Idling(lineState->state) && lineState->callInfo.calltype) ? skinny_calltype2str(lineState->callInfo.calltype) : "INACTIVE")
 
 #include "sccp_cli_table.h"
 
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
+		totals->tables = 1;
 	}
 	return RESULT_SUCCESS;
 }
@@ -1297,32 +1428,33 @@ int sccp_show_hint_lineStates(int fd, int *total, struct mansession *s, const st
  * \called_from_asterisk
  */
 //#include <asterisk/cli.h>
-int sccp_show_hint_subscriptions(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_show_hint_subscriptions(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
-	int local_total = 0;
+	int local_line_total = 0;
 
-#define CLI_AMI_TABLE_NAME Hint_Subscriptions
-#define CLI_AMI_TABLE_PER_ENTRY_NAME Hint_Subscription
+#define CLI_AMI_TABLE_NAME HintSubscriptions
+#define CLI_AMI_TABLE_PER_ENTRY_NAME HintSubscription
 #define CLI_AMI_TABLE_LIST_ITER_HEAD &sccp_hint_subscriptions
 #define CLI_AMI_TABLE_LIST_ITER_TYPE sccp_hint_list_t
 #define CLI_AMI_TABLE_LIST_ITER_VAR subscription
 #define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
-#define CLI_AMI_TABLE_FIELDS 														\
- 		CLI_AMI_TABLE_FIELD(Exten,		"-10.10s",	10,	subscription->exten)					\
- 		CLI_AMI_TABLE_FIELD(Content,		"-10.10s",	10,	subscription->context)					\
- 		CLI_AMI_TABLE_FIELD(Hint,		"-15.15s",	15,	subscription->hint_dialplan)				\
- 		CLI_AMI_TABLE_FIELD(State,		"-22.22s",	22,	sccp_channelstate2str(subscription->currentState))	\
- 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15s",	15,	subscription->callInfo.partyNumber)			\
- 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20s",	20,	subscription->callInfo.partyName)			\
- 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10s",	10,	(!SCCP_CHANNELSTATE_Idling(subscription->currentState) && subscription->callInfo.calltype) ? skinny_calltype2str(subscription->callInfo.calltype) : "INACTIVE")	\
- 		CLI_AMI_TABLE_FIELD(Subs,		"-4d",		4,	SCCP_LIST_GETSIZE(&subscription->subscribers))
+#define CLI_AMI_TABLE_FIELDS 															\
+ 		CLI_AMI_TABLE_FIELD(Exten,		"-10.10",	s,	10,	subscription->exten)					\
+ 		CLI_AMI_TABLE_FIELD(Context,		"-10.10",	s,	10,	subscription->context)					\
+ 		CLI_AMI_TABLE_FIELD(Hint,		"-15.15",	s,	15,	subscription->hint_dialplan)				\
+ 		CLI_AMI_TABLE_FIELD(State,		"-22.22",	s,	22,	sccp_channelstate2str(subscription->currentState))	\
+ 		CLI_AMI_TABLE_FIELD(CallInfoNumber,	"-15.15",	s,	15,	subscription->callInfo.partyNumber)			\
+ 		CLI_AMI_TABLE_FIELD(CallInfoName,	"-20.20",	s,	20,	subscription->callInfo.partyName)			\
+ 		CLI_AMI_TABLE_FIELD(Direction,		"-10.10",	s,	10,	(subscription->callInfo.calltype && subscription->callInfo.calltype != SKINNY_CALLTYPE_SENTINEL) ? skinny_calltype2str(subscription->callInfo.calltype) : "") \
+ 		CLI_AMI_TABLE_FIELD(Subs,		"-4",		d,		4,	SCCP_LIST_GETSIZE(&subscription->subscribers))
 
 #include "sccp_cli_table.h"
 
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
+		totals->tables = 1;
 	}
 	return RESULT_SUCCESS;
 }
