@@ -1,24 +1,24 @@
 /*!
- * \file	sccp_labels.h
+ * \file        sccp_labels.h
  * \brief       SCCP Labels Header
  * 
  * SCCP Button Number References and SCCP Display Number References 
  *
- * \author	Sergio Chersovani <mlists [at] c-net.it>
- * \note	Reworked, but based on chan_sccp code.
- *		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *		Modified by Jan Czmok and Julien Goodwin
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \author      Sergio Chersovani <mlists [at] c-net.it>
+ * \note        Reworked, but based on chan_sccp code.
+ *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *              Modified by Jan Czmok and Julien Goodwin
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-07-10 13:40:46 +0200 (Thu, 10 Jul 2014) $
- * $Revision: 5597 $  
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $  
  */
 
 #ifndef __SCCP_LABELS_H
 #define __SCCP_LABELS_H
 
-#define SKINNY_LBL_EMPTY                                             	0			/*< fake button */
+#define SKINNY_LBL_EMPTY                                             	0					/*< fake button */
 #define SKINNY_LBL_REDIAL                                            	1
 #define SKINNY_LBL_NEWCALL                                           	2
 #define SKINNY_LBL_HOLD                                              	3
@@ -75,9 +75,9 @@
 #define SKINNY_LBL_PRIVATE                                           	54
 #define SKINNY_LBL_NOT_ENOUGH_BANDWIDTH                              	55
 #define SKINNY_LBL_UNKNOWN_NUMBER                                    	56
-#define SKINNY_LBL_RMLSTC                                            	57
+#define SKINNY_LBL_RMLSTC                                            	57					/* Remove Last Conference Participant from the Conference (Moderator Only) */
 #define SKINNY_LBL_VOICEMAIL                                         	58
-#define SKINNY_LBL_IMMDIV                                            	59
+#define SKINNY_LBL_IMMDIV                                            	59					/* Immediate Divert to Voicemail */
 #define SKINNY_LBL_INTRCPT                                           	60
 #define SKINNY_LBL_SETWTCH                                           	61
 #define SKINNY_LBL_TRNSFVM                                           	62
@@ -118,106 +118,107 @@ static const struct skinny_label {
 	uint16_t label;
 	const char *const text;
 } skinny_labels[] = {
-/* INDENT-OFF */
-	{SKINNY_LBL_EMPTY, 						"Empty"},
-	{SKINNY_LBL_REDIAL, 						"Redial"},
-	{SKINNY_LBL_NEWCALL, 						"NewCall"},
-	{SKINNY_LBL_HOLD, 						"Hold"},
-	{SKINNY_LBL_TRANSFER, 						"Transfer"},
-	{SKINNY_LBL_CFWDALL, 						"CFwdALL"},
-	{SKINNY_LBL_CFWDBUSY, 						"CFwdBusy"},
-	{SKINNY_LBL_CFWDNOANSWER, 					"CFwdNoAnswer"},
-	{SKINNY_LBL_BACKSPACE, 						"<<"},
-	{SKINNY_LBL_ENDCALL, 						"EndCall"},
-	{SKINNY_LBL_RESUME, 						"Resume"},
-	{SKINNY_LBL_ANSWER, 						"Answer"},
-	{SKINNY_LBL_INFO, 						"Info"},
-	{SKINNY_LBL_CONFRN, 						"Confrn"},
-	{SKINNY_LBL_PARK, 						"Park"},
-	{SKINNY_LBL_JOIN, 						"Join"},
-	{SKINNY_LBL_MEETME, 						"MeetMe"},
-	{SKINNY_LBL_PICKUP, 						"PickUp"},
-	{SKINNY_LBL_GPICKUP, 						"GPickUp"},
-	{SKINNY_LBL_YOUR_CURRENT_OPTIONS, 				"Your current options"},
-	{SKINNY_LBL_OFF_HOOK, 						"Off Hook"},
-	{SKINNY_LBL_ON_HOOK, 						"On Hook"},
-	{SKINNY_LBL_RING_OUT, 						"Ring out"},
-	{SKINNY_LBL_FROM, 						"From "},
-	{SKINNY_LBL_CONNECTED, 						"Connected"},
-	{SKINNY_LBL_BUSY, 						"Busy"},
-	{SKINNY_LBL_LINE_IN_USE, 					"Line In Use"},
-	{SKINNY_LBL_CALL_WAITING, 					"Call Waiting"},
-	{SKINNY_LBL_CALL_TRANSFER, 					"Call Transfer"},
-	{SKINNY_LBL_CALL_PARK, 						"Call Park"},
-	{SKINNY_LBL_CALL_PROCEED, 					"Call Proceed"},
-	{SKINNY_LBL_IN_USE_REMOTE, 					"In Use Remote"},
-	{SKINNY_LBL_ENTER_NUMBER, 					"Enter number"},
-	{SKINNY_LBL_CALL_PARK_AT, 					"Call park At"},
-	{SKINNY_LBL_PRIMARY_ONLY, 					"Primary Only"},
-	{SKINNY_LBL_TEMP_FAIL, 						"Temp Fail"},
-	{SKINNY_LBL_YOU_HAVE_VOICEMAIL, 				"You Have a VoiceMail"},
-	{SKINNY_LBL_FORWARDED_TO, 					"Forwarded to"},
-	{SKINNY_LBL_CAN_NOT_COMPLETE_CONFERENCE, 			"Can Not Complete Conference"},
-	{SKINNY_LBL_NO_CONFERENCE_BRIDGE, 				"No Conference Bridge"},
-	{SKINNY_LBL_CAN_NOT_HOLD_PRIMARY_CONTROL, 			"Can Not Hold Primary Control"},
-	{SKINNY_LBL_INVALID_CONFERENCE_PARTICIPANT, 			"Invalid Conference Participant"},
-	{SKINNY_LBL_IN_CONFERENCE_ALREADY, 				"In Conference Already"},
-	{SKINNY_LBL_NO_PARTICIPANT_INFO, 				"No Participant Info"},
-	{SKINNY_LBL_EXCEED_MAXIMUM_PARTIES, 				"Exceed Maximum Parties"},
-	{SKINNY_LBL_KEY_IS_NOT_ACTIVE, 					"Key Is Not Active"},
-	{SKINNY_LBL_ERROR_NO_LICENSE, 					"Error No License"},
-	{SKINNY_LBL_ERROR_DBCONFIG, 					"Error DBConfig"},
-	{SKINNY_LBL_ERROR_DATABASE, 					"Error Database"},
-	{SKINNY_LBL_ERROR_PASS_LIMIT, 					"Error Pass Limit"},
-	{SKINNY_LBL_ERROR_UNKNOWN, 					"Error Unknown"},
-	{SKINNY_LBL_ERROR_MISMATCH, 					"Error Mismatch"},
-	{SKINNY_LBL_CONFERENCE, 					"Conference"},
-	{SKINNY_LBL_PARK_NUMBER, 					"Park Number"},
-	{SKINNY_LBL_PRIVATE, 						"Private"},
-	{SKINNY_LBL_NOT_ENOUGH_BANDWIDTH, 				"Not Enough Bandwidth"},
-	{SKINNY_LBL_UNKNOWN_NUMBER, 					"Unknown Number"},
-	{SKINNY_LBL_RMLSTC, 						"RmLstC"},
-	{SKINNY_LBL_VOICEMAIL, 						"Voicemail"},
-	{SKINNY_LBL_IMMDIV, 						"ImmDiv"},
-	{SKINNY_LBL_INTRCPT, 						"Intrcpt"},
-	{SKINNY_LBL_SETWTCH, 						"SetWtch"},
-	{SKINNY_LBL_TRNSFVM, 						"TrnsfVM"},
-	{SKINNY_LBL_DND, 						"DND"},
-	{SKINNY_LBL_DIVALL, 						"DivAll"},
-	{SKINNY_LBL_CALLBACK, 						"CallBack"},
-	{SKINNY_LBL_NETWORK_CONGESTION_REROUTING, 			"Network congestion,rerouting"},
-	{SKINNY_LBL_BARGE, 						"Barge"},
-	{SKINNY_LBL_FAILED_TO_SETUP_BARGE, 				"Failed to setup Barge"},
-	{SKINNY_LBL_ANOTHER_BARGE_EXISTS, 				"Another Barge exists"},
-	{SKINNY_LBL_INCOMPATIBLE_DEVICE_TYPE, 				"Incompatible device type"},
-	{SKINNY_LBL_NO_PARK_NUMBER_AVAILABLE, 				"No Park Number Available"},
-	{SKINNY_LBL_CALLPARK_REVERSION, 				"CallPark Reversion"},
-	{SKINNY_LBL_SERVICE_IS_NOT_ACTIVE, 				"Service is not Active"},
-	{SKINNY_LBL_HIGH_TRAFFIC_TRY_AGAIN_LATER, 			"High Traffic Try Again Later"},
-	{SKINNY_LBL_QRT, 						"QRT"},
-	{SKINNY_LBL_MCID, 						"MCID"},
-	{SKINNY_LBL_DIRTRFR, 						"DirTrfr"},
-	{SKINNY_LBL_SELECT, 						"Select"},
-	{SKINNY_LBL_CONFLIST, 						"ConfList"},
-	{SKINNY_LBL_IDIVERT, 						"iDivert"},
-	{SKINNY_LBL_CBARGE, 						"cBarge"},
-	{SKINNY_LBL_CAN_NOT_COMPLETE_TRANSFER, 				"Can Not Complete Transfer"},
-	{SKINNY_LBL_CAN_NOT_JOIN_CALLS, 				"Can Not Join Calls"},
-	{SKINNY_LBL_MCID_SUCCESSFUL, 					"Mcid Successful"},
-	{SKINNY_LBL_NUMBER_NOT_CONFIGURED, 				"Number Not Configured"},
-	{SKINNY_LBL_SECURITY_ERROR, 					"Security Error"},
-	{SKINNY_LBL_VIDEO_BANDWIDTH_UNAVAILABLE, 			"Video Bandwidth Unavailable"},
-	{SKINNY_LBL_VIDEO_MODE, 					"Video Mode"},
-	{SKINNY_LBL_MONITOR, 						"Record"},
-	{SKINNY_LBL_DIAL, 						"Dial"},
-/* INDENT-ON */
+	/* INDENT-OFF */
+	{
+	SKINNY_LBL_EMPTY, "Empty"}, {
+	SKINNY_LBL_REDIAL, "Redial"}, {
+	SKINNY_LBL_NEWCALL, "NewCall"}, {
+	SKINNY_LBL_HOLD, "Hold"}, {
+	SKINNY_LBL_TRANSFER, "Transfer"}, {
+	SKINNY_LBL_CFWDALL, "CFwdALL"}, {
+	SKINNY_LBL_CFWDBUSY, "CFwdBusy"}, {
+	SKINNY_LBL_CFWDNOANSWER, "CFwdNoAnswer"}, {
+	SKINNY_LBL_BACKSPACE, "<<"}, {
+	SKINNY_LBL_ENDCALL, "EndCall"}, {
+	SKINNY_LBL_RESUME, "Resume"}, {
+	SKINNY_LBL_ANSWER, "Answer"}, {
+	SKINNY_LBL_INFO, "Info"}, {
+	SKINNY_LBL_CONFRN, "Confrn"}, {
+	SKINNY_LBL_PARK, "Park"}, {
+	SKINNY_LBL_JOIN, "Join"}, {
+	SKINNY_LBL_MEETME, "MeetMe"}, {
+	SKINNY_LBL_PICKUP, "PickUp"}, {
+	SKINNY_LBL_GPICKUP, "GPickUp"}, {
+	SKINNY_LBL_YOUR_CURRENT_OPTIONS, "Your current options"}, {
+	SKINNY_LBL_OFF_HOOK, "Off Hook"}, {
+	SKINNY_LBL_ON_HOOK, "On Hook"}, {
+	SKINNY_LBL_RING_OUT, "Ring out"}, {
+	SKINNY_LBL_FROM, "From "}, {
+	SKINNY_LBL_CONNECTED, "Connected"}, {
+	SKINNY_LBL_BUSY, "Busy"}, {
+	SKINNY_LBL_LINE_IN_USE, "Line In Use"}, {
+	SKINNY_LBL_CALL_WAITING, "Call Waiting"}, {
+	SKINNY_LBL_CALL_TRANSFER, "Call Transfer"}, {
+	SKINNY_LBL_CALL_PARK, "Call Park"}, {
+	SKINNY_LBL_CALL_PROCEED, "Call Proceed"}, {
+	SKINNY_LBL_IN_USE_REMOTE, "In Use Remote"}, {
+	SKINNY_LBL_ENTER_NUMBER, "Enter number"}, {
+	SKINNY_LBL_CALL_PARK_AT, "Call park At"}, {
+	SKINNY_LBL_PRIMARY_ONLY, "Primary Only"}, {
+	SKINNY_LBL_TEMP_FAIL, "Temp Fail"}, {
+	SKINNY_LBL_YOU_HAVE_VOICEMAIL, "You Have a VoiceMail"}, {
+	SKINNY_LBL_FORWARDED_TO, "Forwarded to"}, {
+	SKINNY_LBL_CAN_NOT_COMPLETE_CONFERENCE, "Can Not Complete Conference"}, {
+	SKINNY_LBL_NO_CONFERENCE_BRIDGE, "No Conference Bridge"}, {
+	SKINNY_LBL_CAN_NOT_HOLD_PRIMARY_CONTROL, "Can Not Hold Primary Control"}, {
+	SKINNY_LBL_INVALID_CONFERENCE_PARTICIPANT, "Invalid Conference Participant"}, {
+	SKINNY_LBL_IN_CONFERENCE_ALREADY, "In Conference Already"}, {
+	SKINNY_LBL_NO_PARTICIPANT_INFO, "No Participant Info"}, {
+	SKINNY_LBL_EXCEED_MAXIMUM_PARTIES, "Exceed Maximum Parties"}, {
+	SKINNY_LBL_KEY_IS_NOT_ACTIVE, "Key Is Not Active"}, {
+	SKINNY_LBL_ERROR_NO_LICENSE, "Error No License"}, {
+	SKINNY_LBL_ERROR_DBCONFIG, "Error DBConfig"}, {
+	SKINNY_LBL_ERROR_DATABASE, "Error Database"}, {
+	SKINNY_LBL_ERROR_PASS_LIMIT, "Error Pass Limit"}, {
+	SKINNY_LBL_ERROR_UNKNOWN, "Error Unknown"}, {
+	SKINNY_LBL_ERROR_MISMATCH, "Error Mismatch"}, {
+	SKINNY_LBL_CONFERENCE, "Conference"}, {
+	SKINNY_LBL_PARK_NUMBER, "Park Number"}, {
+	SKINNY_LBL_PRIVATE, "Private"}, {
+	SKINNY_LBL_NOT_ENOUGH_BANDWIDTH, "Not Enough Bandwidth"}, {
+	SKINNY_LBL_UNKNOWN_NUMBER, "Unknown Number"}, {
+	SKINNY_LBL_RMLSTC, "RmLstC"}, {
+	SKINNY_LBL_VOICEMAIL, "Voicemail"}, {
+	SKINNY_LBL_IMMDIV, "ImmDiv"}, {
+	SKINNY_LBL_INTRCPT, "Intrcpt"}, {
+	SKINNY_LBL_SETWTCH, "SetWtch"}, {
+	SKINNY_LBL_TRNSFVM, "TrnsfVM"}, {
+	SKINNY_LBL_DND, "DND"}, {
+	SKINNY_LBL_DIVALL, "DivAll"}, {
+	SKINNY_LBL_CALLBACK, "CallBack"}, {
+	SKINNY_LBL_NETWORK_CONGESTION_REROUTING, "Network congestion,rerouting"}, {
+	SKINNY_LBL_BARGE, "Barge"}, {
+	SKINNY_LBL_FAILED_TO_SETUP_BARGE, "Failed to setup Barge"}, {
+	SKINNY_LBL_ANOTHER_BARGE_EXISTS, "Another Barge exists"}, {
+	SKINNY_LBL_INCOMPATIBLE_DEVICE_TYPE, "Incompatible device type"}, {
+	SKINNY_LBL_NO_PARK_NUMBER_AVAILABLE, "No Park Number Available"}, {
+	SKINNY_LBL_CALLPARK_REVERSION, "CallPark Reversion"}, {
+	SKINNY_LBL_SERVICE_IS_NOT_ACTIVE, "Service is not Active"}, {
+	SKINNY_LBL_HIGH_TRAFFIC_TRY_AGAIN_LATER, "High Traffic Try Again Later"}, {
+	SKINNY_LBL_QRT, "QRT"}, {
+	SKINNY_LBL_MCID, "MCID"}, {
+	SKINNY_LBL_DIRTRFR, "DirTrfr"}, {
+	SKINNY_LBL_SELECT, "Select"}, {
+	SKINNY_LBL_CONFLIST, "ConfList"}, {
+	SKINNY_LBL_IDIVERT, "iDivert"}, {
+	SKINNY_LBL_CBARGE, "cBarge"}, {
+	SKINNY_LBL_CAN_NOT_COMPLETE_TRANSFER, "Can Not Complete Transfer"}, {
+	SKINNY_LBL_CAN_NOT_JOIN_CALLS, "Can Not Join Calls"}, {
+	SKINNY_LBL_MCID_SUCCESSFUL, "Mcid Successful"}, {
+	SKINNY_LBL_NUMBER_NOT_CONFIGURED, "Number Not Configured"}, {
+	SKINNY_LBL_SECURITY_ERROR, "Security Error"}, {
+	SKINNY_LBL_VIDEO_BANDWIDTH_UNAVAILABLE, "Video Bandwidth Unavailable"}, {
+	SKINNY_LBL_VIDEO_MODE, "Video Mode"}, {
+	SKINNY_LBL_MONITOR, "Record"}, {
+	SKINNY_LBL_DIAL, "Dial"},
+	    /* INDENT-ON */
 };
 
 #define SKINNY_DISP_EMPTY                                		""
 
-#define SKINNY_DISP_ACCT						"\36\02"
-#define SKINNY_DISP_FLASH						"\36\03"
-#define SKINNY_DISP_LOGIN						"\36\04"
+#define SKINNY_DISP_ACCT						"\36\02"				/* Set Account Code / During Call Setup or Connected Call / Should be added to CDR */
+#define SKINNY_DISP_FLASH						"\36\03"				/* Hook Flash */
+#define SKINNY_DISP_LOGIN						"\36\04"				/* Provides personal identification number (PIN) access to restricted phone features */
 #define SKINNY_DISP_DEVICE_IN_HOME_LOCATION				"\36\05"
 #define SKINNY_DISP_DEVICE_IN_ROAMING_LOCATION				"\36\06"
 #define SKINNY_DISP_ENTER_AUTHORIZATION_CODE				"\36\07"
@@ -235,7 +236,7 @@ static const struct skinny_label {
 #define SKINNY_DISP_SELECT_A_SERVICE					"\36\21"
 #define SKINNY_DISP_LOCAL_SERVICES					"\36\22"
 #define SKINNY_DISP_ENTER_SEARCH_CRITERIA				"\36\23"
-#define SKINNY_DISP_NIGHT_SERVICE					"\36\24"
+#define SKINNY_DISP_NIGHT_SERVICE					"\36\24"				/* We should provide access to Night Server via a Feature Option */
 #define SKINNY_DISP_NIGHT_SERVICE_ACTIVE				"\36\25"
 #define SKINNY_DISP_NIGHT_SERVICE_DISABLED				"\36\26"
 #define SKINNY_DISP_LOGIN_SUCCESSFUL					"\36\27"
@@ -273,7 +274,7 @@ static const struct skinny_label {
 #define SKINNY_DISP_END_CALL						"\36\63"
 #define SKINNY_DISP_CONFERENCE_LIST					"\36\64"
 #define SKINNY_DISP_QUALITY_REPORTING_TOOL				"\36\65"
-#define SKINNY_DISP_HUNT_GROUP						"\36\66"
+#define SKINNY_DISP_HUNT_GROUP						"\36\66"				/* HLOG Button */
 #define SKINNY_DISP_USE_LINE_OR_JOIN_TO_COMPLETE			"\36\67"
 
 #define SKINNY_DISP_DO_NOT_DISTURB					"\36\70"
@@ -289,7 +290,7 @@ static const struct skinny_label {
 #define SKINNY_DISP_CANNOT_SEND_CALL_TO_MOBILE				"\36\101"
 #define SKINNY_DISP_RECORD						"\36\103"
 #define SKINNY_DISP_CANNOT_MOVE_CONVERSATION				"\36\104"
-#define SKINNY_DISP_CW_OFF						"\36\105"
+#define SKINNY_DISP_CW_OFF						"\36\105"				/* Call Waiting Off */
 #define SKINNY_DISP_COACHING						"\36\106"
 #define SKINNY_DISP_RECORDING						"\36\117"
 
@@ -368,7 +369,7 @@ static const struct skinny_label {
 #define SKINNY_DISP_DND                                   		"\200\77"
 
 #define SKINNY_DISP_DIVALL                                		"\200\100"
-#define SKINNY_DISP_CALLBACK                              		"\200\101"
+#define SKINNY_DISP_CALLBACK                              		"\200\101"				/*!< Call Completion */
 #define SKINNY_DISP_NETWORK_CONGESTION_REROUTING          		"\200\102"
 #define SKINNY_DISP_BARGE                                 		"\200\103"
 #define SKINNY_DISP_FAILED_TO_SETUP_BARGE                 		"\200\104"
@@ -379,7 +380,7 @@ static const struct skinny_label {
 #define SKINNY_DISP_CALLPARK_REVERSION                    		"\200\110"
 #define SKINNY_DISP_SERVICE_IS_NOT_ACTIVE                 		"\200\111"
 #define SKINNY_DISP_HIGH_TRAFFIC_TRY_AGAIN_LATER          		"\200\112"
-#define SKINNY_DISP_QRT                                   		"\200\113"
+#define SKINNY_DISP_QRT                                   		"\200\113"				/*<! Quality Request */
 #define SKINNY_DISP_MCID                                  		"\200\114"
 #define SKINNY_DISP_DIRTRFR                               		"\200\115"
 #define SKINNY_DISP_SELECT                                		"\200\116"
@@ -397,8 +398,8 @@ static const struct skinny_label {
 #define SKINNY_DISP_VIDMODE						"\200\130"
 #define SKINNY_DISP_MAX_CALL_DURATION_TIMEOUT				"\200\131"
 #define SKINNY_DISP_MAX_HOLD_DURATION_TIMEOUT				"\200\132"
-#define SKINNY_DISP_OPICKUP						"\200\133"
-#define SKINNY_DISP_HLOG                                        	"\200\134"				/*<! - FS */
+#define SKINNY_DISP_OPICKUP						"\200\133"				/*<! */
+#define SKINNY_DISP_HLOG                                        	"\200\134"				/*<! Huntgroup/queue Login/Logout */
 #define SKINNY_DISP_LOGGED_OUT_OF_HUNT_GROUP                    	"\200\135"				/*<! - FS */
 #define SKINNY_DISP_PARK_SLOT_UNAVAILABLE                       	"\200\136"				/*<! - FS */
 #define SKINNY_DISP_NO_CALL_AVAILABLE_FOR_PICKUP                	"\200\137"				/*<! - FS */
@@ -439,9 +440,9 @@ static const struct skinny_label {
 #define SKINNY_DISP_SERVICES_URL					"\200\177"
 
 // Need to be translated
-//#define SKINNY_DISP_MONITOR						"Record"
+//#define SKINNY_DISP_MONITOR                                           "Record"
 #define SKINNY_DISP_DIAL						"Dial"
-#define SKINNY_DISP_CALL_PROGRESS					"Call Progress" 
+#define SKINNY_DISP_CALL_PROGRESS					"Call Progress"
 #define SKINNY_DISP_SILENT						"Silent"
 #define SKINNY_DISP_ENTER_NUMBER_TO_FORWARD_TO				"Enter number to forward to"
 
@@ -462,3 +463,4 @@ static const struct skinny_label {
 #define SKINNY_GIVING_UP						"Giving Up"
 
 #endif
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
