@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-02-25 13:27:26 +0100 (Wed, 25 Feb 2015) $
+ * $Revision: 5958 $
  */
 
 #include <config.h>
@@ -22,7 +22,7 @@
 #include "sccp_config.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5958 $");
 static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptionId, int onoff);
 int __sccp_line_destroy(const void *ptr);
 int __sccp_lineDevice_destroy(const void *ptr);
@@ -77,7 +77,6 @@ void sccp_line_pre_reload(void)
 void sccp_line_post_reload(void)
 {
 	sccp_line_t *line;
-	sccp_linedevices_t *linedevice;
 
 	SCCP_RWLIST_TRAVERSE_SAFE_BEGIN(&GLOB(lines), line, list) {
 		if (!line->pendingDelete && !line->pendingUpdate) {
@@ -86,11 +85,30 @@ void sccp_line_post_reload(void)
 		AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
 
 		if (l) {
+			// existing lines
+			sccp_linedevices_t *linedevice;
 			SCCP_LIST_LOCK(&l->devices);
 			SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
 				linedevice->device->pendingUpdate = 1;
 			}
 			SCCP_LIST_UNLOCK(&l->devices);
+			
+			if (l->pendingUpdate) {
+				// new line
+				sccp_device_t *d = NULL;
+				sccp_buttonconfig_t *buttonconfig;
+				SCCP_RWLIST_RDLOCK(&GLOB(devices));
+				SCCP_LIST_TRAVERSE(&GLOB(devices), d, list) {
+					SCCP_LIST_LOCK(&d->buttonconfig);
+					SCCP_LIST_TRAVERSE(&d->buttonconfig, buttonconfig, list) {
+						if (buttonconfig->type == LINE && !sccp_strlen_zero(buttonconfig->button.line.name) && !sccp_strequals(line->name, buttonconfig->button.line.name) ) {
+							d->pendingUpdate = TRUE;
+						}
+					}
+					SCCP_LIST_UNLOCK(&d->buttonconfig);
+				}
+				SCCP_RWLIST_UNLOCK(&GLOB(devices));
+			}
 
 			if (l->pendingDelete) {
 				sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Deleting Line (post_reload)\n", l->name);
@@ -119,11 +137,11 @@ sccp_line_t *sccp_line_create(const char *name)
 	sccp_line_t *l = NULL;
 
 	/*      // do make sure line->name is unique before adding.
-	if ((l = sccp_line_find_byname(name, FALSE))) {
+	   if ((l = sccp_line_find_byname(name, FALSE))) {
 	   sccp_line_release(l);
 	   return NULL;
-	}
-	*/
+	   }
+	 */
 	l = (sccp_line_t *) sccp_refcount_object_alloc(sizeof(sccp_line_t), SCCP_REF_LINE, name, __sccp_line_destroy);
 
 	if (!l) {
@@ -314,9 +332,11 @@ int __sccp_line_destroy(const void *ptr)
 		SCCP_LIST_HEAD_DESTROY(&l->devices);
 	}
 	// cleanup mailboxes
-	//if (l->trnsfvm) {
-	//  sccp_free(l->trnsfvm);
-	//}
+	/*  
+	if (l->trnsfvm) {
+		sccp_free(l->trnsfvm);
+	}
+	*/
 	sccp_mailbox_t *mailbox = NULL;
 
 	SCCP_LIST_LOCK(&l->mailboxes);
@@ -337,14 +357,16 @@ int __sccp_line_destroy(const void *ptr)
 	if (SCCP_LIST_EMPTY(&l->mailboxes)) {
 		SCCP_LIST_HEAD_DESTROY(&l->mailboxes);
 	}
-	//#if CS_AST_HAS_NAMEDGROUP
-	//if (l->namedcallgroup) {
-	//  sccp_free(l->namedcallgroup);
-	//}
-	//if (l->namedpickupgroup) {
-	//  sccp_free(l->namedpickupgroup);
-	//}
-	//#endif
+	/*
+	#if CS_AST_HAS_NAMEDGROUP
+	if (l->namedcallgroup) {
+		sccp_free(l->namedcallgroup);
+	}
+	if (l->namedpickupgroup) {
+		sccp_free(l->namedpickupgroup);
+	}
+	#endif
+	*/
 	sccp_config_cleanup_dynamically_allocated_memory(l, SCCP_CONFIG_LINE_SEGMENT);
 
 	// cleanup channels
@@ -416,6 +438,30 @@ void sccp_line_delete_nolock(sccp_line_t * l)
 	sccp_line_clean(l, TRUE);
 }
 
+void sccp_line_copyCodecSetsFromLineToChannel(sccp_line_t *l, sccp_channel_t *c)
+{
+	sccp_linedevices_t *linedevice;
+	if (!l || !c) {
+		return;
+	}
+	/* combine all capabilities */
+	SCCP_LIST_LOCK(&l->devices);
+	SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+		if (linedevice == SCCP_LIST_FIRST(&l->devices)) {
+			memcpy(&c->capabilities.audio, &linedevice->device->capabilities.audio, sizeof(c->capabilities.audio));
+			memcpy(&c->capabilities.video, &linedevice->device->capabilities.video, sizeof(c->capabilities.video));
+			memcpy(&c->preferences.audio , &linedevice->device->preferences.audio , sizeof(c->preferences.audio));
+			memcpy(&c->preferences.video , &linedevice->device->preferences.video , sizeof(c->preferences.video));
+		} else {
+			sccp_utils_combineCodecSets((skinny_codec_t **)&c->capabilities.audio, linedevice->device->capabilities.audio);
+			sccp_utils_combineCodecSets((skinny_codec_t **)&c->capabilities.video, linedevice->device->capabilities.video);
+			sccp_utils_reduceCodecSet((skinny_codec_t **)&c->preferences.audio , linedevice->device->preferences.audio);
+			sccp_utils_reduceCodecSet((skinny_codec_t **)&c->preferences.video , linedevice->device->preferences.video);
+		}
+	}
+	SCCP_LIST_UNLOCK(&l->devices);
+}
+
 /*!
  * \brief Set a Call Forward on a specific Line
  * \param line SCCP Line
@@ -439,6 +485,12 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 	}
 	if ((linedevice = sccp_linedevice_find(device, line))) {
 		if (type == SCCP_CFWD_NONE) {
+			if (linedevice->cfwdAll.enabled) {
+				feature_type = SCCP_FEATURE_CFWDALL;
+			}
+			if (linedevice->cfwdBusy.enabled) {
+				feature_type = SCCP_FEATURE_CFWDBUSY;
+			}
 			linedevice->cfwdAll.enabled = 0;
 			linedevice->cfwdBusy.enabled = 0;
 			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward disabled on line %s\n", DEV_ID_LOG(device), line->name);
@@ -698,7 +750,7 @@ static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptio
 			} else {
 				/* un-register */
 
-				if (&l->devices && SCCP_LIST_GETSIZE(&l->devices) == 1) {			// only remove entry if it is the last one (shared line)
+				if (SCCP_LIST_GETSIZE(&l->devices) == 1) {			// only remove entry if it is the last one (shared line)
 					if (pbx_find_extension(NULL, NULL, &q, context, ext, 1, NULL, "", E_MATCH)) {
 						ast_context_remove_extension(context, ext, 1, NULL);
 						sccp_log((DEBUGCAT_LINE + DEBUGCAT_CONFIG)) (VERBOSE_PREFIX_1 "Unregistered RegContext: %s, Extension: %s\n", context, ext);
@@ -837,7 +889,7 @@ sccp_line_t *sccp_line_find_realtime_byname(const char *name)
 
 		sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_4 "SCCP: creating realtime line '%s'\n", name);
 
-		//  SCCP_RWLIST_WRLOCK(&GLOB(lines));
+		// SCCP_RWLIST_WRLOCK(&GLOB(lines));
 		if ((l = sccp_line_create(name))) {								/* already retained */
 			sccp_config_applyLineConfiguration(l, variable);
 			l->realtime = TRUE;
@@ -846,7 +898,7 @@ sccp_line_t *sccp_line_find_realtime_byname(const char *name)
 		} else {
 			pbx_log(LOG_ERROR, "SCCP: Unable to build realtime line '%s'\n", name);
 		}
-		//  SCCP_RWLIST_UNLOCK(&GLOB(lines));
+		// SCCP_RWLIST_UNLOCK(&GLOB(lines));
 		return l;
 	}
 
