@@ -5,8 +5,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2014-11-20 16:14:19 +0100 (Thu, 20 Nov 2014) $
- * $Revision: 5804 $  
+ * $Date: 2015-03-18 12:16:11 +0100 (Wed, 18 Mar 2015) $
+ * $Revision: 5979 $  
  */
 #include <config.h>
 #ifdef CS_SCCP_MANAGER
@@ -21,11 +21,11 @@
 #include "sccp_actions.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5804 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5979 $");
 
-    /*
-     * Descriptions
-     */
+/*
+ * Descriptions
+ */
 static char management_show_devices_desc[] = "Description: Lists SCCP devices in text format with details on current status. (DEPRECATED in favor of SCCPShowDevices)\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
 static char management_show_lines_desc[] = "Description: Lists SCCP lines in text format with details on current status. (DEPRECATED in favor of SCCPShowLines)\n" "\n" "LinelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
 static char management_restart_devices_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device to restart\n";
@@ -34,6 +34,7 @@ static char management_device_update_desc[] = "Description: restart a given devi
 static char management_device_set_dnd_desc[] = "Description: set dnd on device\n" "\n" "Variables:\n" "   Devicename: Name of device\n" "  DNDState: on (busy) / off / reject/ silent";
 static char management_line_fwd_update_desc[] = "Description: update forward status for line\n" "\n" "Variables:\n" "  Devicename: Name of device\n" "  Linename: Name of line\n" "  Forwardtype: type of cfwd (all | busy | noAnswer)\n" "  Disable: yes Disable call forward (optional)\n" "  Number: number to forward calls (optional)";
 static char management_fetch_config_metadata_desc[] = "Description: fetch configuration metadata\n" "\n" "Variables:\n" "  segment: Config Segment Name (if empty returns all segments).\n";
+
 #if ASTERISK_VERSION_GROUP >= 112
 static char management_startcall_desc[] = "Description: start a new call on a device/line\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  Linename: Name of the line\n" "  number: Number to call\n" "  ChannelId: Channel UniqueId to be set on the channel\n";
 #else
@@ -260,7 +261,7 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 		if (device->session) {
 			sccp_copy_string(clientAddress, sccp_socket_stringify(&device->session->sin), sizeof(clientAddress));
 		} else {
-			sprintf(clientAddress, "--");
+			sccp_copy_string(clientAddress, "--", sizeof(clientAddress));
 		}
 
 		strftime(regtime, sizeof(regtime), "%c", timeinfo);
@@ -661,20 +662,21 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 	}
 
 	AUTO_RELEASE sccp_channel_t *new_channel = NULL;
+
 #if ASTERISK_VERSION_GROUP >= 112
-        struct ast_assigned_ids ids = {
-        	.uniqueid = astman_get_header(m, "ChannelId"),
-                //.uniqueid2 = astman_get_header(m, "OtherChannelId")
+	struct ast_assigned_ids ids = {
+		.uniqueid = astman_get_header(m, "ChannelId"),
+		//.uniqueid2 = astman_get_header(m, "OtherChannelId")
 	};
-	if (	(ids.uniqueid && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid))
-		//|| (ids.uniqueid2 && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid2))
-		) {
+	if ((ids.uniqueid && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid))
+	    //|| (ids.uniqueid2 && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid2))
+	    ) {
 		astman_send_error_va(s, m, "Uniqueid length exceeds maximum of %d\n", AST_MAX_PUBLIC_UNIQUEID);
 		return 0;
 	}
-        new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, (ids.uniqueid) ? &ids : NULL);
+	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, (ids.uniqueid) ? &ids : NULL);
 #else
- 	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL);
+	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL);
 #endif
 	astman_send_ack(s, m, "Call Started");
 	return 0;
@@ -778,8 +780,7 @@ static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 	boolean_t errorMessage = TRUE;
 
 	if (atoi(channelId) == 0) {
-		sprintf(retValStr, "Channel Id has to be a number. You have provided: '%s'\r\n", channelId);
-		astman_send_error(s, m, retValStr);
+		astman_send_error(s, m, "Channel Id has to be a number\r\n");
 		return 0;
 	}
 
@@ -869,14 +870,13 @@ static void sccp_asterisk_parseStrToAstMessage(char *str, struct message *m)
  */
 static int sccp_asterisk_managerHookHelper(int category, const char *event, char *content)
 {
-	struct message m = { 0 };
 	PBX_CHANNEL_TYPE *pbxchannel = NULL;
 	PBX_CHANNEL_TYPE *pbxBridge = NULL;
-
 	char *str, *dupStr;
 
 	if (EVENT_FLAG_CALL == category) {
 		if (!strcasecmp("MonitorStart", event) || !strcasecmp("MonitorStop", event)) {
+			struct message m = { 0 };
 
 			str = dupStr = sccp_strdupa(content); /** need a dup, because converter to message structure will modify the str */
 
