@@ -5,8 +5,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-07 18:44:04 +0200 (Tue, 07 Apr 2015) $
+ * $Revision: 6002 $
  */
 
 #include <config.h>
@@ -17,7 +17,7 @@
 #include "sccp_line.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6002 $");
 #ifndef CS_AST_HAS_EVENT
 #define SCCP_MWI_CHECK_INTERVAL 30
 #endif
@@ -136,8 +136,7 @@ void sccp_mwi_event(void *userdata, struct stasis_subscription *sub, struct stas
 {
 	sccp_mailbox_subscriber_list_t *subscription = userdata;
 
-	pbx_log(LOG_NOTICE, "Got mwi-event\n");
-
+	sccp_log(DEBUGCAT_MWI)(VERBOSE_PREFIX_1 "Got mwi-event\n");
 	if (!subscription || !sub) {
 		return;
 	}
@@ -212,7 +211,7 @@ int sccp_mwi_checksubscription(const void *ptr)
 
 	char buffer[512];
 
-	sprintf(buffer, "%s@%s", subscription->mailbox, subscription->context);
+	snprintf(buffer, 512, "%s@%s", subscription->mailbox, subscription->context);
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_4 "SCCP: ckecking mailbox: %s\n", buffer);
 	pbx_app_inboxcount(buffer, &subscription->currentVoicemailStatistic.newmsgs, &subscription->currentVoicemailStatistic.oldmsgs);
 
@@ -369,7 +368,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		{												/* Fall back on checking the mailbox directly */
 			char buffer[512];
 
-			sprintf(buffer, "%s@%s", subscription->mailbox, subscription->context);
+			snprintf(buffer, 512, "%s@%s", subscription->mailbox, subscription->context);
 			pbx_app_inboxcount(buffer, &subscription->currentVoicemailStatistic.newmsgs, &subscription->currentVoicemailStatistic.oldmsgs);
 		}
 
@@ -388,7 +387,7 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_addMailboxSubscription) Adding STASIS Subscription for mailbox %s\n", subscription->mailbox);
 		char mailbox_context[512];
 
-		sprintf(mailbox_context, "%s@%s", subscription->mailbox, subscription->context);
+		snprintf(mailbox_context, 512, "%s@%s", subscription->mailbox, subscription->context);
 
 		struct stasis_topic *mailbox_specific_topic;
 
@@ -441,7 +440,7 @@ void sccp_mwi_checkLine(sccp_line_t * line)
 
 	SCCP_LIST_LOCK(&line->mailboxes);
 	SCCP_LIST_TRAVERSE(&line->mailboxes, mailbox, list) {
-		sprintf(buffer, "%s@%s", mailbox->mailbox, mailbox->context);
+		snprintf(buffer, 512, "%s@%s", mailbox->mailbox, mailbox->context);
 		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_checkLine) Line: %s, Mailbox: %s\n", line->name, buffer);
 		if (!sccp_strlen_zero(buffer)) {
 
@@ -616,7 +615,7 @@ void sccp_mwi_check(sccp_device_t * d)
 	if (newmsgs > 0) {
 		char buffer[StationMaxDisplayTextSize];
 
-		sprintf(buffer, "%s: (%d/%d)", SKINNY_DISP_YOU_HAVE_VOICEMAIL, newmsgs, oldmsgs);
+		snprintf(buffer, StationMaxDisplayTextSize, "%s: (%u/%u)", SKINNY_DISP_YOU_HAVE_VOICEMAIL, newmsgs, oldmsgs);
 		sccp_device_addMessageToStack(device, SCCP_MESSAGE_PRIORITY_VOICEMAIL, buffer);
 	} else {
 		sccp_device_clearMessageFromStack(device, SCCP_MESSAGE_PRIORITY_VOICEMAIL);
@@ -636,48 +635,49 @@ void sccp_mwi_check(sccp_device_t * d)
  * \called_from_asterisk
  */
 #include <asterisk/cli.h>
-int sccp_show_mwi_subscriptions(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_show_mwi_subscriptions(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
 	sccp_line_t *line = NULL;
 	sccp_mailboxLine_t *mailboxLine = NULL;
 	char linebuf[31] = "";
-	int local_total = 0;
+	int local_line_total = 0;
 
-#define CLI_AMI_TABLE_NAME MWI_Subscriptions
-#define CLI_AMI_TABLE_PER_ENTRY_NAME Mailbox_Subscriber
+#define CLI_AMI_TABLE_NAME MWISubscriptions
+#define CLI_AMI_TABLE_PER_ENTRY_NAME MailboxSubscriber
 #define CLI_AMI_TABLE_LIST_ITER_HEAD &sccp_mailbox_subscriptions
 #define CLI_AMI_TABLE_LIST_ITER_TYPE sccp_mailbox_subscriber_list_t
 #define CLI_AMI_TABLE_LIST_ITER_VAR subscription
 #define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
-#define CLI_AMI_TABLE_BEFORE_ITERATION 													\
- 		SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {						\
- 			line = mailboxLine->line;											\
- 			snprintf(linebuf,sizeof(linebuf),"%30s",line->name);								\
+#define CLI_AMI_TABLE_BEFORE_ITERATION 															\
+ 		SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {								\
+ 			line = mailboxLine->line;													\
+ 			snprintf(linebuf,sizeof(linebuf),"%s",line->name);										\
  		}
 
 #ifdef CS_AST_HAS_EVENT
-#define CLI_AMI_TABLE_FIELDS 														\
- 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10s",	10,	subscription->mailbox)				\
- 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30s",	30,	linebuf)					\
- 		CLI_AMI_TABLE_FIELD(Context,		"-15.15s",	15,	subscription->context)				\
- 		CLI_AMI_TABLE_FIELD(New,		"3.3d",		3,	subscription->currentVoicemailStatistic.newmsgs)\
- 		CLI_AMI_TABLE_FIELD(Old,		"3.3d",		3,	subscription->currentVoicemailStatistic.oldmsgs)\
- 		CLI_AMI_TABLE_FIELD(Sub,		"-3.3s",	3,	subscription->event_sub ? "YES" : "NO")
+#define CLI_AMI_TABLE_FIELDS 																\
+ 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10",	s,	10,	subscription->mailbox)						\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30",	s,	30,	linebuf)							\
+ 		CLI_AMI_TABLE_FIELD(Context,		"-15.15",	s,	15,	subscription->context)						\
+ 		CLI_AMI_TABLE_FIELD(New,		"3.3",		d,		3,	subscription->currentVoicemailStatistic.newmsgs)	\
+ 		CLI_AMI_TABLE_FIELD(Old,		"3.3",		d,		3,	subscription->currentVoicemailStatistic.oldmsgs)	\
+ 		CLI_AMI_TABLE_FIELD(Sub,		"-3.3",		s,	3,	subscription->event_sub ? "YES" : "NO")
 #include "sccp_cli_table.h"
 #else
-#define CLI_AMI_TABLE_FIELDS 														\
- 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10s",	10,	subscription->mailbox)				\
- 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30s",	30,	linebuf)					\
- 		CLI_AMI_TABLE_FIELD(Context,		"-15.15s",	15,	subscription->context)				\
- 		CLI_AMI_TABLE_FIELD(New,		"3.3d",		3,	subscription->currentVoicemailStatistic.newmsgs)\
- 		CLI_AMI_TABLE_FIELD(Old,		"3.3d",		3,	subscription->currentVoicemailStatistic.oldmsgs)
+#define CLI_AMI_TABLE_FIELDS 																\
+ 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10",	s,	10,	subscription->mailbox)						\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30",	s,	30,	linebuf)							\
+ 		CLI_AMI_TABLE_FIELD(Context,		"-15.15",	s,	15,	subscription->context)						\
+ 		CLI_AMI_TABLE_FIELD(New,		"3.3",		d,		3,	subscription->currentVoicemailStatistic.newmsgs)	\
+ 		CLI_AMI_TABLE_FIELD(Old,		"3.3",		d,		3,	subscription->currentVoicemailStatistic.oldmsgs)
 #include "sccp_cli_table.h"
 #endif
 
 	if (s) {
-		*total = local_total;
+		totals->lines = local_line_total;
+		totals->tables = 1;
 	}
 	return RESULT_SUCCESS;
 }
