@@ -5,16 +5,16 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-07 18:44:04 +0200 (Tue, 07 Apr 2015) $
+ * $Revision: 6002 $
  */
 
 #ifndef SCCP_MWI_H_
 #define SCCP_MWI_H_
 
+#include "sccp_cli.h"
 #ifdef CS_AST_HAS_EVENT
 #include "asterisk/event.h"
-
 #endif
 
 /*!
@@ -94,7 +94,7 @@ void sccp_mwi_event(void *userdata, struct stasis_subscription *sub, struct stas
 #else
 int sccp_mwi_checksubscription(const void *ptr);
 #endif
-int sccp_show_mwi_subscriptions(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[]);
+int sccp_show_mwi_subscriptions(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[]);
 
 #endif														/*SCCP_MWI_H_ */
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
