@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-12-13 18:49:44 +0100 (Sat, 13 Dec 2014) $
- * $Revision: 5812 $
+ * $Date: 2015-03-15 14:32:28 +0100 (Sun, 15 Mar 2015) $
+ * $Revision: 5969 $
  */
 #ifndef __PBX_IMPL_C
 #define __PBX_IMPL_C
@@ -26,11 +26,11 @@
 #include "sccp_indicate.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5812 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5969 $");
 
-    /*!
-     * \brief SCCP Structure to pass data to the pbx answer thread 
-     */
+/*!
+ * \brief SCCP Structure to pass data to the pbx answer thread
+ */
 struct sccp_answer_conveyor_struct {
 	uint32_t callid;
 	sccp_linedevices_t *linedevice;
@@ -98,9 +98,9 @@ FINAL:
  *
  * \callgraph
  * \callergraph
- * 
+ *
  * \called_from_asterisk
- * 
+ *
  * \note called with c retained
  */
 
@@ -172,7 +172,7 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 		   This would help users dial from call history lists on other phones, which do not have enbloc dialing,
 		   when using shared lines. */
 		if (NULL != cid_number && strlen(cid_number) > 0 && strlen(cid_number) < sizeof(suffixedNumber) - 2 && '0' == cid_number[0]) {
-			strncpy(suffixedNumber, cid_number, strlen(cid_number));
+			sccp_copy_string(suffixedNumber, cid_number, sizeof(suffixedNumber));
 			suffixedNumber[strlen(cid_number) + 0] = '#';
 			suffixedNumber[strlen(cid_number) + 1] = '\0';
 			sccp_channel_set_callingparty(c, cid_name, suffixedNumber);
@@ -232,12 +232,13 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 
 		if (active_channel) {
 			sccp_indicate(linedevice->device, c, SCCP_CHANNELSTATE_CALLWAITING);
-			
+
 			/* display the new call on prompt */
 			{
 				AUTO_RELEASE sccp_linedevices_t *activeChannelLinedevice = sccp_linedevice_find(linedevice->device, active_channel->line);
-				
+
 				char prompt[100];
+
 				snprintf(prompt, sizeof(prompt), "%s: %s: %s", active_channel->line->name, SKINNY_DISP_FROM, c->callInfo.callingPartyNumber);
 				sccp_dev_displayprompt(linedevice->device, activeChannelLinedevice->lineInstance, active_channel->callid, prompt, GLOB(digittimeout));
 			}
@@ -269,7 +270,7 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 	SCCP_LIST_UNLOCK(&l->devices);
 
 	if (isRinging) {
-		//  sccp_channel_setSkinnyCallstate(c, SKINNY_CALLSTATE_RINGIN);
+		// sccp_channel_setSkinnyCallstate(c, SKINNY_CALLSTATE_RINGIN);
 		sccp_channel_setChannelstate(c, SCCP_CHANNELSTATE_RINGING);
 		PBX(queue_control) (c->owner, AST_CONTROL_RINGING);
 	} else if (hasDNDParticipant) {
@@ -304,9 +305,9 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
  *
  * \callgraph
  * \callergraph
- * 
+ *
  * \called_from_asterisk via sccp_wrapper_asterisk.._hangup
- * 
+ *
  * \note sccp_channel should be retained in calling function
  */
 
@@ -331,7 +332,7 @@ int sccp_pbx_hangup(sccp_channel_t * channel)
 	AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(c);
 
 	if (d && c->state != SCCP_CHANNELSTATE_DOWN && SKINNY_DEVICE_RS_OK == d->registrationState) {
-		// if (GLOB(remotehangup_tone) && d && d->state == SCCP_DEVICESTATE_OFFHOOK && c == sccp_device_getActiveChannel_nolock(d))             /* Caused active channels never to be full released */
+		// if (GLOB(remotehangup_tone) && d && d->state == SCCP_DEVICESTATE_OFFHOOK && c == sccp_device_getActiveChannel_nolock(d))	/* Caused active channels never to be full released */
 		if (GLOB(remotehangup_tone) && d && d->state == SCCP_DEVICESTATE_OFFHOOK && c == d->active_channel) {
 			sccp_dev_starttone(d, GLOB(remotehangup_tone), 0, 0, 10);
 		}
@@ -365,17 +366,15 @@ int sccp_pbx_hangup(sccp_channel_t * channel)
 	/* remove call from transferee, transferer */
 	sccp_linedevices_t *linedevice = NULL;
 
-	if (&l->devices) {
-		SCCP_LIST_LOCK(&l->devices);
-		SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
-			AUTO_RELEASE sccp_device_t *tmpDevice = sccp_device_retain(linedevice->device);
+	SCCP_LIST_LOCK(&l->devices);
+	SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+		AUTO_RELEASE sccp_device_t *tmpDevice = sccp_device_retain(linedevice->device);
 
-			if (tmpDevice) {
-				sccp_channel_transfer_release(tmpDevice, c);
-			}
+		if (tmpDevice) {
+			sccp_channel_transfer_release(tmpDevice, c);
 		}
-		SCCP_LIST_UNLOCK(&l->devices);
 	}
+	SCCP_LIST_UNLOCK(&l->devices);
 	/* done - remove call from transferee, transferer */
 
 	sccp_line_removeChannel(l, c);
@@ -383,16 +382,14 @@ int sccp_pbx_hangup(sccp_channel_t * channel)
 	if (!d) {
 		/* channel is not answered, just ringin over all devices */
 		/* find the first the device on which it is registered and hangup that one (__sccp_indicate_remote_device will do the rest) */
-		if (&l->devices) {
-			SCCP_LIST_LOCK(&l->devices);
-			SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
-				if (linedevice->device && SKINNY_DEVICE_RS_OK == linedevice->device->registrationState) {
-					d = sccp_device_retain(linedevice->device);
-					break;
-				}
+		SCCP_LIST_LOCK(&l->devices);
+		SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+			if (linedevice->device && SKINNY_DEVICE_RS_OK == linedevice->device->registrationState) {
+				d = sccp_device_retain(linedevice->device);
+				break;
 			}
-			SCCP_LIST_UNLOCK(&l->devices);
 		}
+		SCCP_LIST_UNLOCK(&l->devices);
 	}
 	if (d) {
 		d->monitorFeature.status &= ~SCCP_FEATURE_MONITOR_STATE_ACTIVE;
@@ -529,6 +526,18 @@ int sccp_pbx_answer(sccp_channel_t * channel)
 		AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(c);
 
 		if (d) {
+			if (d->earlyrtp == SCCP_EARLYRTP_IMMEDIATE) {
+				/* 
+				* Redial button isnt't working properly in immediate mode, because the
+				* last dialed number was being remembered too early. This fix
+				* remembers the last dialed number in the same cases, where the dialed number
+				* is being sent - after receiving of RINGOUT -Pavel Troller
+				*/
+				sccp_device_setLastNumberDialed(d, c->dialedNumber); 
+				if (PBX(set_dialed_number)){
+					PBX(set_dialed_number) (c, c->dialedNumber);
+				}
+			}
 			sccp_indicate(d, c, SCCP_CHANNELSTATE_PROCEED);
 			sccp_indicate(d, c, SCCP_CHANNELSTATE_CONNECTED);
 
@@ -555,7 +564,7 @@ int sccp_pbx_answer(sccp_channel_t * channel)
  *
  * \callgraph
  * \callergraph
- * 
+ *
  * \lock
  *  - usecnt_lock
  */
@@ -563,6 +572,7 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 {
 	PBX_CHANNEL_TYPE *tmp;
 	AUTO_RELEASE sccp_channel_t *c = sccp_channel_retain(channel);
+	AUTO_RELEASE sccp_device_t *d = NULL;
 
 	if (!c) {
 		return -1;
@@ -583,19 +593,23 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 
 	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP: try to allocate channel on line: %s\n", l->name);
 	/* Don't hold a sccp pvt lock while we allocate a channel */
-	AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(c);
-
-	if (d) {
-		sccp_linedevices_t *linedevice;
-
-		SCCP_LIST_LOCK(&l->devices);
-		SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
-			if (linedevice->device == d) {
-				break;
-			}
-		}
-		SCCP_LIST_UNLOCK(&l->devices);
 
+	sccp_linedevices_t *linedevice = NULL;
+        if ((d = sccp_channel_getDevice_retained(c))) {
+                SCCP_LIST_LOCK(&l->devices);
+                SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
+                        if (linedevice->device == d) {
+                                break;
+                        }
+                }
+                SCCP_LIST_UNLOCK(&l->devices);
+        } else if (SCCP_LIST_GETSIZE(&l->devices) == 1) {
+                SCCP_LIST_LOCK(&l->devices);
+                linedevice = SCCP_LIST_FIRST(&l->devices);
+                d = sccp_device_retain(linedevice->device);
+                SCCP_LIST_UNLOCK(&l->devices);
+        }
+        if (linedevice) {		/* single line channel */
 		switch (c->calltype) {
 			case SKINNY_CALLTYPE_INBOUND:
 				/* append subscriptionId to cid */
@@ -626,11 +640,14 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 					sprintf(c->callInfo.callingPartyName, "%s%s", l->cid_name, (l->defaultSubscriptionId.name) ? l->defaultSubscriptionId.name : "");
 				}
 				break;
-			case skinny_calltype_LOOKUPERROR:
+			case SKINNY_CALLTYPE_SENTINEL:
 				break;
 		}
-	} else {
-
+		memcpy(&c->capabilities.audio, &linedevice->device->capabilities.audio, sizeof(c->capabilities.audio));
+		memcpy(&c->capabilities.video, &linedevice->device->capabilities.video, sizeof(c->capabilities.video));
+		memcpy(&c->preferences.audio , &linedevice->device->preferences.audio , sizeof(c->preferences.audio));
+		memcpy(&c->preferences.video , &linedevice->device->preferences.video , sizeof(c->preferences.video));
+	} else {			/* shared line */
 		switch (c->calltype) {
 			case SKINNY_CALLTYPE_INBOUND:
 				sprintf(c->callInfo.calledPartyNumber, "%s%s", l->cid_num, (l->defaultSubscriptionId.number) ? l->defaultSubscriptionId.number : "");
@@ -641,27 +658,33 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 				sprintf(c->callInfo.callingPartyNumber, "%s%s", l->cid_num, (l->defaultSubscriptionId.number) ? l->defaultSubscriptionId.number : "");
 				sprintf(c->callInfo.callingPartyName, "%s%s", l->cid_name, (l->defaultSubscriptionId.name) ? l->defaultSubscriptionId.name : "");
 				break;
-			case skinny_calltype_LOOKUPERROR:
+			case SKINNY_CALLTYPE_SENTINEL:
 				break;
 		}
+		/* \todo we should be doing this when a device is attached to a line, and store the caps/prefs inside the sccp_line_t */
+		/* \todo it would be nice if we could set audio preferences by line instead of only per device, especially in case of shared line */
+		sccp_line_copyCodecSetsFromLineToChannel(l, c);
 	}
 
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:     cid_num: \"%s\"\n", c->callInfo.callingPartyNumber);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:    cid_name: \"%s\"\n", c->callInfo.callingPartyName);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP: accountcode: \"%s\"\n", l->accountcode);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:       exten: \"%s\"\n", c->dialedNumber);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:     context: \"%s\"\n", l->context);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:    amaflags: \"%d\"\n", l->amaflags);
-	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:   chan/call: \"%s-%08x\"\n", l->name, c->callid);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:              cid_num: \"%s\"\n", c->callInfo.callingPartyNumber);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:             cid_name: \"%s\"\n", c->callInfo.callingPartyName);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:          accountcode: \"%s\"\n", l->accountcode);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:                exten: \"%s\"\n", c->dialedNumber);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:              context: \"%s\"\n", l->context);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:             amaflags: \"%d\"\n", l->amaflags);
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:            chan/call: \"%s-%08x\"\n", l->name, c->callid);
+	char s1[512], s2[512];
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:combined capabilities: \"%s\"\n", sccp_multiple_codecs2str(s1, sizeof(s1) - 1, channel->capabilities.audio, SKINNY_MAX_CAPABILITIES));
+	sccp_log((DEBUGCAT_PBX + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP:  reduced preferences: \"%s\"\n", sccp_multiple_codecs2str(s2, sizeof(s2) - 1, channel->preferences.audio, SKINNY_MAX_CAPABILITIES));
 
 	/* This should definitely fix CDR */
 	//tmp = pbx_channel_alloc(1, AST_STATE_DOWN, c->callInfo.callingPartyNumber, c->callInfo.callingPartyName, l->accountcode, c->dialedNumber, l->context, l->amaflags, "SCCP/%s-%08x", l->name, c->callid);
 	PBX(alloc_pbxChannel) (c, ids, parentChannel, &tmp);
-	if (!tmp) {
+	if (!tmp || !c->owner) {
 		pbx_log(LOG_ERROR, "%s: Unable to allocate asterisk channel on line %s\n", l->id, l->name);
 		return 0;
 	}
-	sccp_channel_updateChannelCapability(c);
+       	sccp_channel_updateChannelCapability(c);
 	PBX(set_nativeAudioFormats) (c, c->preferences.audio, 1);
 
 	/* can be replaced with c->designator */
@@ -685,6 +708,9 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
 	if (PBX(set_callerid_number)) {
 		PBX(set_callerid_number) (c, c->callInfo.callingPartyNumber);
 	}
+	if (PBX(set_callerid_ani)) {
+		PBX(set_callerid_ani) (c, c->callInfo.callingPartyNumber);
+	}
 	if (PBX(set_callerid_name)) {
 		PBX(set_callerid_name) (c, c->callInfo.callingPartyName);
 	}
@@ -757,7 +783,7 @@ uint8_t sccp_pbx_channel_allocate(sccp_channel_t * channel, const void *ids, con
  * \brief Schedule Asterisk Dial
  * \param data Data as constant
  * \return Success as int
- * 
+ *
  * \called_from_asterisk
  */
 int sccp_pbx_sched_dial(const void *data)
@@ -792,9 +818,9 @@ sccp_extension_status_t sccp_pbx_helper(sccp_channel_t * c)
 		}
 	}
 
-	if ((c->ss_action != SCCP_SS_GETCBARGEROOM) && (c->ss_action != SCCP_SS_GETMEETMEROOM)
+	if ((c->softswitch_action != SCCP_SOFTSWITCH_GETCBARGEROOM) && (c->softswitch_action != SCCP_SOFTSWITCH_GETMEETMEROOM)
 #ifdef CS_SCCP_CONFERENCE
-	    && (c->ss_action != SCCP_SS_GETCONFERENCEROOM)
+	    && (c->softswitch_action != SCCP_SOFTSWITCH_GETCONFERENCEROOM)
 #endif
 	    ) {
 
@@ -850,7 +876,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 				if (PBX(send_digits)) {
 					PBX(send_digits) (channel, c->dialedNumber);
 				}
-				sccp_channel_set_calledparty(c, "", c->dialedNumber);
+				sccp_channel_set_calledparty(c, NULL, c->dialedNumber);
 			}
 			goto EXIT_FUNC;
 		}
@@ -883,7 +909,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 		}
 
 		/* we don't need to check for a device type but just if the device has an id, otherwise back home  -FS */
-		if (!d->id || sccp_strlen_zero(d->id)) {
+		if (sccp_strlen_zero(d->id)) {
 			pbx_log(LOG_ERROR, "SCCP: (sccp_pbx_softswitch) No <device> identifier available. Returning from dial thread. Exiting\n");
 			goto EXIT_FUNC;
 		}
@@ -925,8 +951,8 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 		}
 
 		/* This will choose what to do */
-		switch (c->ss_action) {
-			case SCCP_SS_GETFORWARDEXTEN:
+		switch (c->softswitch_action) {
+			case SCCP_SOFTSWITCH_GETFORWARDEXTEN:
 				{
 					sccp_callforward_t type = (sccp_callforward_t) c->ss_data;
 
@@ -938,7 +964,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					goto EXIT_FUNC;								// leave simple switch without dial
 				}
 #ifdef CS_SCCP_PICKUP
-			case SCCP_SS_GETPICKUPEXTEN:
+			case SCCP_SOFTSWITCH_GETPICKUPEXTEN:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Pickup Extension\n", d->id);
 				// like we're dialing but we're not :)
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_DIALING);
@@ -948,10 +974,11 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CALL_PROCEED, GLOB(digittimeout));
 
 				if (!sccp_strlen_zero(shortenedNumber)) {
-					sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (sccp_pbx_softswitch) Asterisk request to pickup exten '%s'\n", shortenedNumber);
-					//if (!sccp_feat_directed_pickup(c, shortenedNumber)) {
-					//	sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
-					//}
+ 					sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: (sccp_pbx_softswitch) Asterisk request to pickup exten '%s'\n", shortenedNumber);
+					if (sccp_feat_directed_pickup(c, shortenedNumber)) {
+						sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);
+					}
+					
 					PBX(set_callstate) (c, AST_STATE_UP);
 				} else {
 					// without a number we can also close the call. Isn't it true ?
@@ -960,7 +987,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 				goto EXIT_FUNC;									// leave simpleswitch without dial
 #endif														// CS_SCCP_PICKUP
 #ifdef CS_SCCP_CONFERENCE
-			case SCCP_SS_GETCONFERENCEROOM:
+			case SCCP_SOFTSWITCH_GETCONFERENCEROOM:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Conference request\n", d->id);
 				if (c->owner && !pbx_check_hangup(c->owner)) {
 					sccp_indicate(d, c, SCCP_CHANNELSTATE_DIALING);
@@ -982,7 +1009,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 				}
 				goto EXIT_FUNC;
 #endif														// CS_SCCP_CONFERENCE
-			case SCCP_SS_GETMEETMEROOM:
+			case SCCP_SOFTSWITCH_GETMEETMEROOM:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Meetme request\n", d->id);
 				if (!sccp_strlen_zero(shortenedNumber) && !sccp_strlen_zero(c->line->meetmenum)) {
 					sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Meetme request for room '%s' on extension '%s'\n", d->id, shortenedNumber, c->line->meetmenum);
@@ -1002,7 +1029,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					goto EXIT_FUNC;
 				}
 				break;
-			case SCCP_SS_GETBARGEEXTEN:
+			case SCCP_SOFTSWITCH_GETBARGEEXTEN:
 				// like we're dialing but we're not :)
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Barge Extension\n", d->id);
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_DIALING);
@@ -1021,7 +1048,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					sccp_channel_endcall(c);
 				}
 				goto EXIT_FUNC;									// leave simpleswitch without dial
-			case SCCP_SS_GETCBARGEROOM:
+			case SCCP_SOFTSWITCH_GETCBARGEROOM:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Get Conference Barge Extension\n", d->id);
 				// like we're dialing but we're not :)
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_DIALING);
@@ -1039,12 +1066,15 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					sccp_channel_endcall(c);
 				}
 				goto EXIT_FUNC;									// leave simpleswitch without dial
-			case SCCP_SS_DIAL:
+			case SCCP_SOFTSWITCH_DIAL:
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Dial Extension %s\n", d->id, shortenedNumber);
 
 				sccp_copy_string(c->callInfo.calledPartyNumber, shortenedNumber, sizeof(c->callInfo.calledPartyNumber));
 				sccp_indicate(d, c, SCCP_CHANNELSTATE_DIALING);
 				break;
+			case SCCP_SOFTSWITCH_SENTINEL:
+				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: (sccp_pbx_softswitch) Unknown Softswitch Request\n", d->id);
+				goto EXIT_FUNC;
 		}
 
 		/* set private variable */
@@ -1086,7 +1116,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 
 		sccp_channel_set_calledparty(c, "", shortenedNumber);
 
-#if 0														/* Remarked out by Pavel Troller for the earlyrtp immediate implementation. Checking if there will be negative fall out. 
+#if 0														/* Remarked out by Pavel Troller for the earlyrtp immediate implementation. Checking if there will be negative fall out.
 														   It might have to check the device->earlyrtp state, to do the correct thing (Let's see). */
 
 		/* proceed call state is needed to display the called number. The phone will not display callinfo in offhook state */
@@ -1117,7 +1147,7 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 					case AST_PBX_FAILED:
 						pbx_log(LOG_ERROR, "%s: (sccp_pbx_softswitch) channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
 						/* \todo change indicate to something more suitable */
-						sccp_indicate(d, c, SCCP_CHANNELSTATE_INVALIDNUMBER);		/* will auto hangup after SCCP_HANGUP_TIMEOUT */
+						sccp_indicate(d, c, SCCP_CHANNELSTATE_CONGESTION);		/* will auto hangup after SCCP_HANGUP_TIMEOUT */
 						break;
 					case AST_PBX_CALL_LIMIT:
 						pbx_log(LOG_WARNING, "%s: (sccp_pbx_softswitch) call limit reached for channel %s-%08x failed to start new thread to dial %s\n", DEV_ID_LOG(d), l->name, c->callid, shortenedNumber);
@@ -1127,13 +1157,23 @@ void *sccp_pbx_softswitch(sccp_channel_t * channel)
 						sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) pbx started\n", DEV_ID_LOG(d));
 #ifdef CS_MANAGER_EVENTS
 						if (GLOB(callevents)) {
-							manager_event(EVENT_FLAG_SYSTEM, "ChannelUpdate", "Channel: %s\r\nUniqueid: %s\r\nChanneltype: %s\r\nSCCPdevice: %s\r\nSCCPline: %s\r\nSCCPcallid: %08x\r\nSCCPCallDesignator: %s\r\n", (pbx_channel) ? pbx_channel_name(pbx_channel) : "(null)", (pbx_channel) ? pbx_channel_uniqueid(pbx_channel) : "(null)", "SCCP", (d) ? DEV_ID_LOG(d) : "(null)", (l && l->name) ? l->name : "(null)", (c && c->callid) ? c->callid : 0,
+							manager_event(EVENT_FLAG_SYSTEM, "ChannelUpdate", "Channel: %s\r\nUniqueid: %s\r\nChanneltype: %s\r\nSCCPdevice: %s\r\nSCCPline: %s\r\nSCCPcallid: %08x\r\nSCCPCallDesignator: %s\r\n", (pbx_channel) ? pbx_channel_name(pbx_channel) : "(null)", (pbx_channel) ? pbx_channel_uniqueid(pbx_channel) : "(null)", "SCCP", (d) ? DEV_ID_LOG(d) : "(null)", (l) ? l->name : "(null)", (c && c->callid) ? c->callid : 0,
 								      c ? c->designator : "(null)");
 						}
 #endif														// CS_MANAGER_EVENTS
 						break;
 				}
-				sccp_device_setLastNumberDialed(d, shortenedNumber);
+				if (d->earlyrtp != SCCP_EARLYRTP_IMMEDIATE){
+					/* 
+					 * too early to set last dialed number for immediate mode -Pavel Troller
+					 */
+					sccp_device_setLastNumberDialed(d, shortenedNumber);
+					if (PBX(set_dialed_number)){
+						PBX(set_dialed_number) (c, shortenedNumber);
+					}
+				}
+				
+
 			} else {
 				sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_1 "%s: (sccp_pbx_softswitch) pbx_check_hangup(chan): %d on line %s\n", DEV_ID_LOG(d), (pbx_channel && pbx_check_hangup(pbx_channel)), l->name);
 			}
@@ -1168,7 +1208,7 @@ EXIT_FUNC:
  *
  * \test Dialplan Transfer Needs to be tested
  * \todo pbx_transfer needs to be implemented correctly
- * 
+ *
  * \called_from_asterisk
  */
 int sccp_pbx_transfer(PBX_CHANNEL_TYPE * ast, const char *dest)
