@@ -6,8 +6,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-11 19:05:51 +0200 (Sat, 11 Apr 2015) $
+ * $Revision: 6011 $
  */
 
 #include <config.h>
@@ -20,11 +20,7 @@
 #include "sccp_socket.h"
 #include <asterisk/unaligned.h>
 
-#define ENUMMACRO_FILE "sccp_protocol_enums.hh"
-#define ENUMMACRO_GENERATE ENUMMACRO_GENFUNCTION
-#include "sccp_enum_macro.h"
-#undef  ENUMMACRO_GENERATE
-#undef  ENUMMACRO_FILE
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6011 $");
 
 /* CallInfo Message */
 
@@ -43,28 +39,28 @@ static void sccp_device_sendCallinfoV3(const sccp_device_t * device, const sccp_
 
 	REQ(msg, CallInfoMessage);
 
-	if (channel->callInfo.callingPartyName) {
+	if (!sccp_strlen_zero(channel->callInfo.callingPartyName)) {
 		sccp_copy_string(msg->data.CallInfoMessage.callingPartyName, channel->callInfo.callingPartyName, sizeof(msg->data.CallInfoMessage.callingPartyName));
 	}
-	if (channel->callInfo.callingPartyNumber) {
+	if (!sccp_strlen_zero(channel->callInfo.callingPartyNumber)) {
 		sccp_copy_string(msg->data.CallInfoMessage.callingParty, channel->callInfo.callingPartyNumber, sizeof(msg->data.CallInfoMessage.callingParty));
 	}
-	if (channel->callInfo.calledPartyName) {
+	if (!sccp_strlen_zero(channel->callInfo.calledPartyName)) {
 		sccp_copy_string(msg->data.CallInfoMessage.calledPartyName, channel->callInfo.calledPartyName, sizeof(msg->data.CallInfoMessage.calledPartyName));
 	}
-	if (channel->callInfo.calledPartyNumber) {
+	if (!sccp_strlen_zero(channel->callInfo.calledPartyNumber)) {
 		sccp_copy_string(msg->data.CallInfoMessage.calledParty, channel->callInfo.calledPartyNumber, sizeof(msg->data.CallInfoMessage.calledParty));
 	}
-	if (channel->callInfo.originalCalledPartyName) {
+	if (!sccp_strlen_zero(channel->callInfo.originalCalledPartyName)) {
 		sccp_copy_string(msg->data.CallInfoMessage.originalCalledPartyName, channel->callInfo.originalCalledPartyName, sizeof(msg->data.CallInfoMessage.originalCalledPartyName));
 	}
-	if (channel->callInfo.originalCalledPartyNumber) {
+	if (!sccp_strlen_zero(channel->callInfo.originalCalledPartyNumber)) {
 		sccp_copy_string(msg->data.CallInfoMessage.originalCalledParty, channel->callInfo.originalCalledPartyNumber, sizeof(msg->data.CallInfoMessage.originalCalledParty));
 	}
-	if (channel->callInfo.lastRedirectingPartyName) {
+	if (!sccp_strlen_zero(channel->callInfo.lastRedirectingPartyName)) {
 		sccp_copy_string(msg->data.CallInfoMessage.lastRedirectingPartyName, channel->callInfo.lastRedirectingPartyName, sizeof(msg->data.CallInfoMessage.lastRedirectingPartyName));
 	}
-	if (channel->callInfo.lastRedirectingPartyNumber) {
+	if (!sccp_strlen_zero(channel->callInfo.lastRedirectingPartyNumber)) {
 		sccp_copy_string(msg->data.CallInfoMessage.lastRedirectingParty, channel->callInfo.lastRedirectingPartyNumber, sizeof(msg->data.CallInfoMessage.lastRedirectingParty));
 	}
 	if (channel->callInfo.originalCdpnRedirectReason) {
@@ -73,16 +69,16 @@ static void sccp_device_sendCallinfoV3(const sccp_device_t * device, const sccp_
 	if (channel->callInfo.lastRedirectingReason) {
 		msg->data.CallInfoMessage.lastRedirectingReason = htolel(channel->callInfo.lastRedirectingReason);
 	}
-	if (channel->callInfo.cgpnVoiceMailbox) {
+	if (!sccp_strlen_zero(channel->callInfo.cgpnVoiceMailbox)) {
 		sccp_copy_string(msg->data.CallInfoMessage.cgpnVoiceMailbox, channel->callInfo.cgpnVoiceMailbox, sizeof(msg->data.CallInfoMessage.cgpnVoiceMailbox));
 	}
-	if (channel->callInfo.cdpnVoiceMailbox) {
+	if (!sccp_strlen_zero(channel->callInfo.cdpnVoiceMailbox)) {
 		sccp_copy_string(msg->data.CallInfoMessage.cdpnVoiceMailbox, channel->callInfo.cdpnVoiceMailbox, sizeof(msg->data.CallInfoMessage.cdpnVoiceMailbox));
 	}
-	if (channel->callInfo.originalCdpnVoiceMailbox) {
+	if (!sccp_strlen_zero(channel->callInfo.originalCdpnVoiceMailbox)) {
 		sccp_copy_string(msg->data.CallInfoMessage.originalCdpnVoiceMailbox, channel->callInfo.originalCdpnVoiceMailbox, sizeof(msg->data.CallInfoMessage.originalCdpnVoiceMailbox));
 	}
-	if (channel->callInfo.lastRedirectingVoiceMailbox) {
+	if (!sccp_strlen_zero(channel->callInfo.lastRedirectingVoiceMailbox)) {
 		sccp_copy_string(msg->data.CallInfoMessage.lastRedirectingVoiceMailbox, channel->callInfo.lastRedirectingVoiceMailbox, sizeof(msg->data.CallInfoMessage.lastRedirectingVoiceMailbox));
 	}
 
@@ -154,14 +150,14 @@ static void sccp_device_sendCallinfoV7(const sccp_device_t * device, const sccp_
 	msg->data.CallInfoDynamicMessage.lel_lastRedirectingReason = htolel(channel->callInfo.lastRedirectingReason);
 
 	if (dummy_len) {
-		//  int bufferSize = dummy_len + ARRAY_LEN(data);
+		// int bufferSize = dummy_len + ARRAY_LEN(data);
 		int bufferSize = dummy_len + dataSize;
 		char buffer[bufferSize];
 
 		memset(&buffer[0], 0, bufferSize);
 		int pos = 0;
 
-		//  for (i = 0; i < ARRAY_LEN(data); i++) {
+		// for (i = 0; i < ARRAY_LEN(data); i++) {
 		for (i = 0; i < dataSize; i++) {
 			sccp_log(DEBUGCAT_HIGH) (VERBOSE_PREFIX_3 "SCCP: cid field %d, value: '%s'\n", i, (data[i]) ? data[i] : "");
 			if (data[i]) {
@@ -240,14 +236,14 @@ static void sccp_protocol_sendCallinfoV16(const sccp_device_t * device, const sc
 	msg->data.CallInfoDynamicMessage.lel_lastRedirectingReason = htolel(channel->callInfo.lastRedirectingReason);
 
 	if (dummy_len) {
-		//  int bufferSize = dummy_len + ARRAY_LEN(data);
+		// int bufferSize = dummy_len + ARRAY_LEN(data);
 		int bufferSize = dummy_len + dataSize;
 		char buffer[bufferSize];
 
 		memset(&buffer[0], 0, bufferSize);
 		int pos = 0;
 
-		//  for (i = 0; i < ARRAY_LEN(data); i++) {
+		// for (i = 0; i < ARRAY_LEN(data); i++) {
 		for (i = 0; i < dataSize; i++) {
 			sccp_log(DEBUGCAT_HIGH) (VERBOSE_PREFIX_3 "SCCP: cid field %d, value: '%s'\n", i, (data[i]) ? data[i] : "");
 			if (data[i]) {
@@ -502,16 +498,19 @@ static void sccp_protocol_sendRegisterAckV3(const sccp_device_t * device, uint8_
 	REQ(msg, RegisterAckMessage);
 
 	/* just for documentation */
-#if 0
-	msg->data.RegisterAckMessage.unknown1 = 0x00;
-	msg->data.RegisterAckMessage.unknown2 = 0x00;
-	msg->data.RegisterAckMessage.unknown3 = 0x00;
-#endif
-	msg->data.RegisterAckMessage.protocolVer = device->protocol->version;
+	msg->data.RegisterAckMessage.protocolVer2 = 0x00;							// 0x00;
+	msg->data.RegisterAckMessage.phoneFeatures1 = 0x00;							// 0x00;
+	msg->data.RegisterAckMessage.phoneFeatures2 = 0x00;							// 0x00;
+
+	msg->data.RegisterAckMessage.maxProtocolVer = device->protocol->version;
 	msg->data.RegisterAckMessage.lel_keepAliveInterval = htolel(keepAliveInterval);
 	msg->data.RegisterAckMessage.lel_secondaryKeepAliveInterval = htolel(secondaryKeepAlive);
 
-	memcpy(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	if (!sccp_strlen_zero(dateformat)) {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	} else {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, "M/D/Y", sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	}
 	sccp_dev_send(device, msg);
 }
 
@@ -524,16 +523,20 @@ static void sccp_protocol_sendRegisterAckV4(const sccp_device_t * device, uint8_
 
 	REQ(msg, RegisterAckMessage);
 
-	msg->data.RegisterAckMessage.unknown1 = 0x20;								// 0x00;
-	msg->data.RegisterAckMessage.unknown2 = 0x00;								// 0x00;
-	msg->data.RegisterAckMessage.unknown3 = 0xFE;								// 0xFE;
+	msg->data.RegisterAckMessage.protocolVer2 = 0x20;							// 0x20;
+	msg->data.RegisterAckMessage.phoneFeatures1 = 0xF1;							// 0xF1;
+	msg->data.RegisterAckMessage.phoneFeatures2 = 0xFE;							// 0xFF;
 
-	msg->data.RegisterAckMessage.protocolVer = device->protocol->version;
+	msg->data.RegisterAckMessage.maxProtocolVer = device->protocol->version;
 
 	msg->data.RegisterAckMessage.lel_keepAliveInterval = htolel(keepAliveInterval);
 	msg->data.RegisterAckMessage.lel_secondaryKeepAliveInterval = htolel(secondaryKeepAlive);
 
-	memcpy(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	if (!sccp_strlen_zero(dateformat)) {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	} else {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, "M/D/Y", sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	}
 	sccp_dev_send(device, msg);
 }
 
@@ -546,15 +549,19 @@ static void sccp_protocol_sendRegisterAckV11(const sccp_device_t * device, uint8
 
 	REQ(msg, RegisterAckMessage);
 
-	msg->data.RegisterAckMessage.unknown1 = 0x20;								// 0x00;
-	msg->data.RegisterAckMessage.unknown2 = 0xF1;								// 0xF1;
-	msg->data.RegisterAckMessage.unknown3 = 0xFF;								// 0xFF;
+	msg->data.RegisterAckMessage.protocolVer2 = 0x20;							// 0x20;
+	msg->data.RegisterAckMessage.phoneFeatures1 = 0xF1;							// 0xF1;
+	msg->data.RegisterAckMessage.phoneFeatures2 = 0xFF;							// 0xFF;
 
-	msg->data.RegisterAckMessage.protocolVer = device->protocol->version;
+	msg->data.RegisterAckMessage.maxProtocolVer = device->protocol->version;
 	msg->data.RegisterAckMessage.lel_keepAliveInterval = htolel(keepAliveInterval);
 	msg->data.RegisterAckMessage.lel_secondaryKeepAliveInterval = htolel(secondaryKeepAlive);
 
-	memcpy(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	if (!sccp_strlen_zero(dateformat)) {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, dateformat, sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	} else {
+		sccp_copy_string(msg->data.RegisterAckMessage.dateTemplate, "M/D/Y", sizeof(msg->data.RegisterAckMessage.dateTemplate));
+	}
 	sccp_dev_send(device, msg);
 }
 
@@ -747,7 +754,7 @@ static void sccp_protocol_sendOpenMultiMediaChannelV17(const sccp_device_t * dev
 }
 
 /*!
- * \brief Send Start MultiMedia Transmission (V3)
+ * \brief Send Start Media Transmission (V3)
  */
 static void sccp_protocol_sendStartMediaTransmissionV3(const sccp_device_t * device, const sccp_channel_t * channel)
 {
@@ -782,7 +789,7 @@ static void sccp_protocol_sendStartMediaTransmissionV3(const sccp_device_t * dev
 }
 
 /*!
- * \brief Send Start MultiMedia Transmission (V17)
+ * \brief Send Start Media Transmission (V17)
  */
 static void sccp_protocol_sendStartMediaTransmissionV17(const sccp_device_t * device, const sccp_channel_t * channel)
 {
@@ -819,7 +826,7 @@ static void sccp_protocol_sendStartMediaTransmissionV17(const sccp_device_t * de
 }
 
 /*!
- * \brief Send Start MultiMedia Transmission (v22)
+ * \brief Send Start Media Transmission (v22)
  */
 static void sccp_protocol_sendStartMediaTransmissionv22(const sccp_device_t * device, const sccp_channel_t * channel)
 {
@@ -1007,11 +1014,6 @@ static void sccp_protocol_sendUserToDeviceDataVersion1Message(const sccp_device_
 				memcpy(&msg[segment]->data.UserToDeviceDataVersion1Message.data, xmlData + xmlDataStart, msg_len);
 				xmlDataStart += msg_len;
 			}
-#ifndef CS_EXPERIMENTAL
-			//sccp_dump_msg(msg[segment]);
-#else
-			sccp_dump_msg(msg[segment]);
-#endif
 			sccp_dev_send(device, msg[segment]);
 			usleep(10);
 			sccp_log(DEBUGCAT_HIGH) (VERBOSE_PREFIX_1 "%s: (sccp_protocol_sendUserToDeviceDataVersion1Message) Message sent to device  (hdr_len: %d, msglen: %d/%d, padding: %d, msg-size: %d).\n", DEV_ID_LOG(device), hdr_len, msg_len, (int) strlen(xmlData), padding, hdr_len + msg_len + padding);
@@ -1233,7 +1235,7 @@ static void sccp_protocol_parseStartMultiMediaTransmissionAckV17(const sccp_msg_
 		ss->ss_family = AF_INET;
 		struct sockaddr_in *sin = (struct sockaddr_in *) ss;
 
-		memcpy(&sin->sin_addr, &msg->data.StartMultiMediaTransmissionAck.v17.bel_ipAddr, sizeof(sin->sin_addr));
+		memcpy(&sin->sin_addr, &msg->data.StartMultiMediaTransmissionAck.v17.bel_ipAddr, sizeof(struct in_addr));
 		sin->sin_port = htons(htolel(msg->data.StartMultiMediaTransmissionAck.v17.lel_portNumber));
 	} else {												// read ipv6 address
 		/* what to do with IPv4-mapped IPv6 addresses */
