@@ -8,8 +8,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-14 15:53:13 +0100 (Fri, 14 Nov 2014) $
- * $Revision: 5799 $
+ * $Date: 2015-04-11 19:05:51 +0200 (Sat, 11 Apr 2015) $
+ * $Revision: 6011 $
  */
 
 #include <config.h>
@@ -20,12 +20,12 @@
 #include "sccp_rtp.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5799 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6011 $");
 
-    /*!
-     * \brief create a new rtp server for audio data
-     * \param c SCCP Channel
-     */
+/*!
+ * \brief create a new rtp server for audio data
+ * \param c SCCP Channel
+ */
 int sccp_rtp_createAudioServer(const sccp_channel_t * c)
 {
 	boolean_t rtpResult = FALSE;
@@ -142,8 +142,8 @@ void sccp_rtp_set_peer(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockaddr
 	//memcpy(&c->rtp.audio.phone_remote, new_peer, sizeof(c->rtp.audio.phone_remote));
 	memcpy(&rtp->phone_remote, new_peer, sizeof(rtp->phone_remote));
 
-	//  inet_ntop(rtp->phone_remote.ss_family, sccp_socket_getAddr(&rtp->phone_remote), addressString, sizeof(addressString));
-	//pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s:%d\n", c->currentDeviceId, addressString, sccp_socket_getPort(new_peer));
+	// inet_ntop(rtp->phone_remote.ss_family, sccp_socket_getAddr(&rtp->phone_remote), addressString, sizeof(addressString));
+	// pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s:%d\n", c->currentDeviceId, addressString, sccp_socket_getPort(new_peer));
 	pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s\n", c->currentDeviceId, sccp_socket_stringify(&rtp->phone_remote));
 
 	if (rtp->readState & SCCP_RTP_STATUS_ACTIVE) {
@@ -152,8 +152,8 @@ void sccp_rtp_set_peer(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockaddr
 		sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_peer) Stop media transmission on channel %d\n", c->currentDeviceId, c->callid);
 
 		/*! \todo we should check if this is a video or autio rtp */
-		//  sccp_channel_stopmediatransmission(c);
-		//sccp_channel_startMediaTransmission(c);
+		// sccp_channel_stopmediatransmission(c);
+		// sccp_channel_startMediaTransmission(c);
 		sccp_channel_updateMediaTransmission(c);
 	}
 
@@ -180,17 +180,17 @@ void sccp_rtp_set_phone(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockadd
 		/* check if we have new infos */
 		/*! \todo if we enable this, we get an audio issue when resume on the same device, so we need to force asterisk to update -MC */
 		/*
-		   if (socket_equals(new_peer, &c->rtp.audio.phone)) {
-		   sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_phone) remote information are equal to the current one, ignore change\n", c->currentDeviceId);
-		   //return;
-		   } 
-		 */
+		if (socket_equals(new_peer, &c->rtp.audio.phone)) {
+			sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_phone) remote information are equal to the current one, ignore change\n", c->currentDeviceId);
+			//return;
+		} 
+		*/
 
 		memcpy(&rtp->phone, new_peer, sizeof(rtp->phone));
 
 		//update pbx
 		if (PBX(rtp_setPhoneAddress)) {
-			PBX(rtp_setPhoneAddress) (rtp, new_peer, device->nat);
+			PBX(rtp_setPhoneAddress) (rtp, new_peer, device->nat >= SCCP_NAT_ON ? 1 : 0);
 		}
 
 		char buf1[NI_MAXHOST + NI_MAXSERV];
@@ -199,7 +199,7 @@ void sccp_rtp_set_phone(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockadd
 		char buf2[NI_MAXHOST + NI_MAXSERV];
 
 		sccp_copy_string(buf2, sccp_socket_stringify(&rtp->phone), sizeof(buf2));
-		sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "%s: Tell PBX   to send RTP/UDP media from %s to %s (NAT: %s)\n", DEV_ID_LOG(device), buf1, buf2, device->nat ? "yes" : "no");
+		sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "%s: Tell PBX   to send RTP/UDP media from %s to %s (NAT: %s)\n", DEV_ID_LOG(device), buf1, buf2, sccp_nat2str(device->nat));
 	}
 }
 
@@ -220,7 +220,7 @@ sccp_rtp_info_t sccp_rtp_getAudioPeerInfo(const sccp_channel_t * c, struct sccp_
 
 	result = SCCP_RTP_INFO_AVAILABLE;
 	// \todo add apply_ha(d->ha, &sin) check here instead
-	if (device->directrtp && !device->nat && !c->conference) {
+	if (device->directrtp && device->nat == SCCP_NAT_OFF && !c->conference) {
 		result |= SCCP_RTP_INFO_ALLOW_DIRECTRTP;
 	}
 	return result;
@@ -242,7 +242,7 @@ sccp_rtp_info_t sccp_rtp_getVideoPeerInfo(const sccp_channel_t * c, struct sccp_
 	*rtp = &(((sccp_channel_t *) c)->rtp.video);
 
 	result = SCCP_RTP_INFO_AVAILABLE;
-	if (device->directrtp && !device->nat && !c->conference) {
+	if (device->directrtp && device->nat == SCCP_NAT_OFF && !c->conference) {
 		result |= SCCP_RTP_INFO_ALLOW_DIRECTRTP;
 	}
 	return result;
@@ -320,7 +320,7 @@ boolean_t sccp_rtp_getUs(const struct sccp_rtp *rtp, struct sockaddr_storage *us
 		PBX(rtp_getUs) (rtp->rtp, us);
 		return TRUE;
 	} else {
-		//  us = &rtp->phone_remote;
+		// us = &rtp->phone_remote;
 		return FALSE;
 	}
 }
