@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-27 14:34:55 +0100 (Thu, 27 Nov 2014) $
- * $Revision: 5809 $
+ * $Date: 2015-03-14 16:28:13 +0100 (Sat, 14 Mar 2015) $
+ * $Revision: 5964 $
  */
 
 #include <config.h>
@@ -25,12 +25,12 @@
 #include "sccp_actions.h"
 #include "sccp_rtp.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5809 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5964 $");
 
-    /* =========================================================================================== Global */
-    /*!
-     * \brief Global list of softkeys
-     */
+/* =========================================================================================== Global */
+/*!
+ * \brief Global list of softkeys
+ */
 struct softKeySetConfigList softKeySetConfig;									/*!< List of SoftKeySets */
 
 /*!
@@ -56,7 +56,7 @@ static void sccp_sk_dial(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_device
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Dial Pressed\n", DEV_ID_LOG(d));
 	if (c && !PBX(getChannelPbx) (c)) {									// Prevent dialling if in an inappropriate state.
 		/* Only handle this in DIALING state. AFAIK GETDIGITS is used only for call forward and related input functions. (-DD) */
-		if (c->ss_action == SCCP_SS_GETFORWARDEXTEN) {
+		if (c->softswitch_action == SCCP_SOFTSWITCH_GETFORWARDEXTEN) {
 			sccp_pbx_softswitch(c);
 
 		} else if (c->state == SCCP_CHANNELSTATE_DIGITSFOLL) {
@@ -299,25 +299,46 @@ static void sccp_sk_dnd(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_device_
 
 	if (!d->dndFeature.enabled) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Feature disabled\n", DEV_ID_LOG(d));
-		sccp_dev_displayprompt(d, lineInstance, c->callid, SKINNY_DISP_DND " " SKINNY_DISP_SERVICE_IS_NOT_ACTIVE, SCCP_DISPLAYSTATUS_TIMEOUT);
+		sccp_dev_displayprompt(d, lineInstance, c ? c->callid : 0, SKINNY_DISP_DND " " SKINNY_DISP_SERVICE_IS_NOT_ACTIVE, SCCP_DISPLAYSTATUS_TIMEOUT);
 		return;
 	}
 
-	if (!strcasecmp(d->dndFeature.configOptions, "reject")) {
-		/* config is set to: dnd=reject */
-		if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
-			d->dndFeature.status = SCCP_DNDMODE_REJECT;
+	do {
+		if (l) {
+			if (l->dndmode == SCCP_DNDMODE_REJECT) {
+				if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
+					d->dndFeature.status = SCCP_DNDMODE_REJECT;
+				} else {
+					d->dndFeature.status = SCCP_DNDMODE_OFF;
+				}
+				break;
+			} else if (l->dndmode == SCCP_DNDMODE_SILENT) {	
+				/* config is set to: dnd=silent */
+				if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
+					d->dndFeature.status = SCCP_DNDMODE_SILENT;
+				} else {
+					d->dndFeature.status = SCCP_DNDMODE_OFF;
+				}
+				break;
+			}
 		} else {
-			d->dndFeature.status = SCCP_DNDMODE_OFF;
+			if (d->dndmode == SCCP_DNDMODE_REJECT) {
+				if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
+					d->dndFeature.status = SCCP_DNDMODE_REJECT;
+				} else {
+					d->dndFeature.status = SCCP_DNDMODE_OFF;
+				}
+				break;
+			} else if (d->dndmode == SCCP_DNDMODE_SILENT) {	
+				/* config is set to: dnd=silent */
+				if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
+					d->dndFeature.status = SCCP_DNDMODE_SILENT;
+				} else {
+					d->dndFeature.status = SCCP_DNDMODE_OFF;
+				}
+				break;
+			}
 		}
-	} else if (!strcasecmp(d->dndFeature.configOptions, "silent")) {
-		/* config is set to: dnd=silent */
-		if (d->dndFeature.status == SCCP_DNDMODE_OFF) {
-			d->dndFeature.status = SCCP_DNDMODE_SILENT;
-		} else {
-			d->dndFeature.status = SCCP_DNDMODE_OFF;
-		}
-	} else {
 		/* for all other config us the toggle mode */
 		switch (d->dndFeature.status) {
 			case SCCP_DNDMODE_OFF:
@@ -333,7 +354,8 @@ static void sccp_sk_dnd(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_device_
 				d->dndFeature.status = SCCP_DNDMODE_OFF;
 				break;
 		}
-	}
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Pressed (New Status: %s, Feature enabled: %s)\n", DEV_ID_LOG(d), sccp_dndmode2str(d->dndFeature.status), d->dndFeature.enabled ? "YES" : "NO");
+	} while (0);
 
 	sccp_feat_changed(d, NULL, SCCP_FEATURE_DND);								/* notify the modules the the DND-feature changed state */
 	sccp_dev_check_displayprompt(d);									/*! \todo we should use the feature changed event to check displayprompt */
@@ -870,7 +892,6 @@ static void sccp_sk_gpickup(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_dev
 #endif
 }
 
-#if CS_EXPERIMENTAL
 /*!
  * \brief Execute URI(s) 
  * \n Usage: \ref sk_uriaction
@@ -932,7 +953,6 @@ static void sccp_sk_uriaction(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_d
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: Send '%s' to Phone\n", DEV_ID_LOG(d), pbx_str_buffer(xmlStr));
 	d->protocol->sendUserToDeviceDataVersionMessage(d, APPID_URIHOOK, lineInstance, c ? c->callid : 0, transactionID, pbx_str_buffer(xmlStr), 0);
 }
-#endif
 
 /*!
  * \brief Softkey Function Callback by SKINNY LABEL
@@ -979,12 +999,10 @@ gcc_inline static const sccp_softkeyMap_cb_t *sccp_getSoftkeyMap_by_SoftkeyEvent
 
 	const sccp_softkeyMap_cb_t *mySoftkeyCbMap = softkeyCbMap;
 
-#ifdef CS_EXPERIMENTAL
 	if (d->softkeyset && d->softkeyset->softkeyCbMap) {
 		mySoftkeyCbMap = d->softkeyset->softkeyCbMap;
 	}
 	sccp_log(DEBUGCAT_SOFTKEY) (VERBOSE_PREFIX_3 "%s: (sccp_getSoftkeyMap_by_SoftkeyEvent) default: %p, softkeyset: %p, softkeyCbMap: %p\n", d->id, softkeyCbMap, d->softkeyset, d->softkeyset ? d->softkeyset->softkeyCbMap : NULL);
-#endif
 
 	for (i = 0; i < ARRAY_LEN(softkeyCbMap); i++) {
 		if (mySoftkeyCbMap[i].event == event) {
@@ -1002,9 +1020,7 @@ gcc_inline static const sccp_softkeyMap_cb_t *sccp_getSoftkeyMap_by_SoftkeyEvent
  */
 void sccp_softkey_pre_reload(void)
 {
-#ifdef CS_EXPERIMENTAL
 	sccp_softkey_clear();
-#endif
 }
 
 /*!
@@ -1012,7 +1028,6 @@ void sccp_softkey_pre_reload(void)
  */
 void sccp_softkey_post_reload(void)
 {
-#ifdef CS_EXPERIMENTAL
 	/* only required because softkeys are parsed after devices */
 	/* incase softkeysets have changed but device was not reloaded, then d->softkeyset needs to be fixed up */
 	sccp_softKeySetConfiguration_t *softkeyset;
@@ -1022,14 +1037,17 @@ void sccp_softkey_post_reload(void)
 	SCCP_LIST_TRAVERSE(&softKeySetConfig, softkeyset, list) {
 		SCCP_RWLIST_WRLOCK(&GLOB(devices));
 		SCCP_RWLIST_TRAVERSE(&GLOB(devices), d, list) {
-			if (d->softkeyDefinition && !strcasecmp(d->softkeyDefinition, softkeyset->name)) {
+			if (sccp_strcaseequals(d->softkeyDefinition, softkeyset->name)) {
+				sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "Re-attaching softkeyset: %s to device d: %s\n", softkeyset->name, d->id);
 				d->softkeyset = softkeyset;
+				d->softKeyConfiguration.modes = softkeyset->modes;
+				d->softKeyConfiguration.size = softkeyset->numberOfSoftKeySets;
+//				sccp_handle_soft_key_set_req(d->s, d, NULL);
 			}
 		}
-		SCCP_RWLIST_WRLOCK(&GLOB(devices));
+		SCCP_RWLIST_UNLOCK(&GLOB(devices));
 	}
 	SCCP_LIST_UNLOCK(&softKeySetConfig);
-#endif
 }
 
 /*!
@@ -1044,10 +1062,11 @@ void sccp_softkey_clear(void)
 	while ((k = SCCP_LIST_REMOVE_HEAD(&softKeySetConfig, list))) {
 		for (i = 0; i < (sizeof(SoftKeyModes) / sizeof(softkey_modes)); i++) {
 			if (k->modes[i].ptr) {
+				//sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "Freeing KeyMode Ptr: %p for KeyMode %i\n", k->modes[i].ptr, i);
 				sccp_free(k->modes[i].ptr);
+				k->modes[i].count = 0;
 			}
 		}
-#ifdef CS_EXPERIMENTAL
 		if (k->softkeyCbMap) {
 			for (i = 0; i < ARRAY_LEN(softkeyCbMap); i++) {
 				if (!sccp_strlen_zero(k->softkeyCbMap[i].uriactionstr)) {
@@ -1056,7 +1075,7 @@ void sccp_softkey_clear(void)
 			}
 			sccp_free(k->softkeyCbMap);
 		}
-#endif
+		//sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "Softkeyset: %s removed\n", k->name);
 		sccp_free(k);
 	}
 	SCCP_LIST_UNLOCK(&softKeySetConfig);
@@ -1078,7 +1097,6 @@ sccp_softkeyMap_cb_t __attribute__ ((malloc)) * sccp_softkeyMap_copyStaticallyMa
 	return newSoftKeyMap;
 }
 
-#if CS_EXPERIMENTAL
 /*!
  * \brief Replace a specific softkey callback entry by sccp_sk_uriaction and fill it's uriactionstr
  * \note strdup, needs to be freed
@@ -1097,7 +1115,6 @@ boolean_t sccp_softkeyMap_replaceCallBackByUriAction(sccp_softkeyMap_cb_t * soft
 	}
 	return FALSE;
 }
-#endif
 
 /*!
  * \brief Execute Softkey Callback by SofkeyEvent
@@ -1138,7 +1155,7 @@ void sccp_softkey_setSoftkeyState(sccp_device_t * device, uint8_t softKeySet, ui
 
 	/* find softkey */
 	for (i = 0; i < device->softKeyConfiguration.modes[softKeySet].count; i++) {
-		if (device->softKeyConfiguration.modes[softKeySet].ptr[i] == softKey) {
+		if (device->softKeyConfiguration.modes[softKeySet].ptr && device->softKeyConfiguration.modes[softKeySet].ptr[i] == softKey) {
 			sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: found softkey '%s' at %d\n", DEV_ID_LOG(device), label2str(device->softKeyConfiguration.modes[softKeySet].ptr[i]), i);
 
 			if (enable) {
