@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2015-04-19 20:49:40 +0200 (Sun, 19 Apr 2015) $
+ * $Revision: 6026 $
  */
 
 #include <config.h>
@@ -17,14 +17,16 @@
 #include "sccp_device.h"
 #include "sccp_utils.h"
 #include "sccp_socket.h"
+#if HAVE_ICONV_H
+#include <iconv.h>
+#endif
+SCCP_FILE_VERSION(__FILE__, "$Revision: 6026 $");
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
-
-    /*!
-     * \brief Print out a messagebuffer
-     * \param messagebuffer Pointer to Message Buffer as char
-     * \param len Lenght as Int
-     */
+/*!
+ * \brief Print out a messagebuffer
+ * \param messagebuffer Pointer to Message Buffer as char
+ * \param len Lenght as Int
+ */
 void sccp_dump_packet(unsigned char *messagebuffer, int len)
 {
 	static const int numcolumns = 16;									// number output columns
@@ -43,9 +45,9 @@ void sccp_dump_packet(unsigned char *messagebuffer, int len)
 	char *chrptr;
 
 	do {
-		//  memset(hexout, 0, sizeof(hexout));
+		// memset(hexout, 0, sizeof(hexout));
 		memset(hexout, 0, (numcolumns * 3) + (numcolumns / 8) + 1);
-		//  memset(chrout, 0, sizeof(chrout));
+		// memset(chrout, 0, sizeof(chrout));
 		memset(chrout, 0, numcolumns + 1);
 		hexptr = hexout;
 		chrptr = chrout;
@@ -81,6 +83,10 @@ int sccp_addons_taps(sccp_device_t * d)
 	sccp_addon_t *cur = NULL;
 	int taps = 0;
 
+	if (SCCP_LIST_GETSIZE(&d->addons) > 0 && (d->skinny_type == SKINNY_DEVICETYPE_CISCO7941 || d->skinny_type == SKINNY_DEVICETYPE_CISCO7941GE)) {
+		pbx_log(LOG_WARNING, "%s: %s devices do no support AddOns/Expansion Modules of any kind. Skipping !\n", DEV_ID_LOG(d), skinny_devicetype2str(d->skinny_type));
+	}
+
 	if (!strcasecmp(d->config_type, "7914")) {
 		pbx_log(LOG_WARNING, "%s: config_type '%s' forces addon compatibily mode. Possibly faulty config file.\n", DEV_ID_LOG(d), d->config_type);
 		return 28;											// in compatibility mode it returns 28 taps for a double 7914 addon
@@ -96,6 +102,10 @@ int sccp_addons_taps(sccp_device_t * d)
 		if (cur->type == SKINNY_DEVICETYPE_CISCO_ADDON_7915_24BUTTON || cur->type == SKINNY_DEVICETYPE_CISCO_ADDON_7916_24BUTTON) {
 			taps += 24;
 		}
+		/* should maybe check if connected via SPCS */
+		if (cur->type == SKINNY_DEVICETYPE_CISCO_ADDON_SPA500S || cur->type == SKINNY_DEVICETYPE_CISCO_ADDON_SPA500DS || cur->type == SKINNY_DEVICETYPE_CISCO_ADDON_SPA932DS) {
+			taps += 32;
+		}
 		sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Found (%d) taps on device addon (%d)\n", (d->id ? d->id : "SCCP"), taps, cur->type);
 	}
 	SCCP_LIST_UNLOCK(&d->addons);
@@ -170,7 +180,7 @@ void sccp_dev_dbclean(void)
 
 			SCCP_RWLIST_RDLOCK(&GLOB(devices));
 			SCCP_RWLIST_TRAVERSE(&GLOB(devices), d, list) {
-				if (d->id && !strcasecmp(d->id, key)) {
+				if (!strcasecmp(d->id, key)) {
 					break;
 				}
 			}
@@ -191,12 +201,20 @@ void sccp_dev_dbclean(void)
 
 gcc_inline const char *msgtype2str(sccp_mid_t type)
 {														/* sccp_protocol.h */
-	return sccp_messagetypes[type].text;
+	if (type >= SPCP_MESSAGE_OFFSET) {
+		return spcp_messagetypes[type - SPCP_MESSAGE_OFFSET].text;
+	} else {
+		return sccp_messagetypes[type].text;
+	}
 }
 
 gcc_inline size_t msgtype2size(sccp_mid_t type)
 {														/* sccp_protocol.h */
-	return sccp_messagetypes[type].size + SCCP_PACKET_HEADER;
+	if (type >= SPCP_MESSAGE_OFFSET) {
+		return spcp_messagetypes[type - SPCP_MESSAGE_OFFSET].size + SCCP_PACKET_HEADER;
+	} else {
+		return sccp_messagetypes[type].size + SCCP_PACKET_HEADER;
+	}
 }
 
 gcc_inline const char *pbxsccp_devicestate2str(uint32_t value)
@@ -293,6 +311,7 @@ char *sccp_multiple_codecs2str(char *buf, size_t size, const skinny_codec_t * co
 /*!
  * \brief Remove Codec from Skinny Codec Preferences
  */
+/*
 static void skinny_codec_pref_remove(skinny_codec_t * skinny_codec_prefs, skinny_codec_t skinny_codec)
 {
 	int x = 0;
@@ -318,6 +337,22 @@ static void skinny_codec_pref_remove(skinny_codec_t * skinny_codec_prefs, skinny
 		}
 	}
 }
+*/
+static void skinny_codec_pref_remove(skinny_codec_t * skinny_codec_prefs, skinny_codec_t skinny_codec)
+{
+	int x = 0;
+	boolean_t found = FALSE;
+
+	for (x = 0; x < SKINNY_MAX_CAPABILITIES && skinny_codec_prefs[x] != SKINNY_CODEC_NONE; x++) {
+		if (skinny_codec_prefs[x] == skinny_codec) {
+			found = TRUE;
+			break;
+		}
+		if (found) {
+			memmove(skinny_codec_prefs + x, skinny_codec_prefs + (x + 1), (SKINNY_MAX_CAPABILITIES - (x + 1)) * sizeof(skinny_codec_t));  // move left
+		}
+	}
+}
 
 /*!
  * \brief Append Codec to Skinny Codec Preferences
@@ -330,12 +365,8 @@ static int skinny_codec_pref_append(skinny_codec_t * skinny_codec_pref, skinny_c
 	skinny_codec_pref_remove(skinny_codec_pref, skinny_codec);
 
 	for (x = 0; x < SKINNY_MAX_CAPABILITIES; x++) {
-		// insert behaviour: skip if already there (cheaper)
-		//  if (skinny_codec_pref[x] == skinny_codec) {
-		//    return x;
-		//  }
 		if (SKINNY_CODEC_NONE == skinny_codec_pref[x]) {
-			sccp_log((DEBUGCAT_CODEC)) (VERBOSE_PREFIX_1 "inserting codec '%s (%d)' at pos %d\n", codec2name(skinny_codec), skinny_codec, x);
+			//sccp_log((DEBUGCAT_CODEC)) (VERBOSE_PREFIX_1 "inserting codec '%s (%d)' at pos %d\n", codec2name(skinny_codec), skinny_codec, x);
 			skinny_codec_pref[x] = skinny_codec;
 			return x;
 		}
@@ -406,6 +437,55 @@ boolean_t sccp_utils_isCodecCompatible(skinny_codec_t codec, const skinny_codec_
 	return FALSE;
 }
 
+/*!
+ * \brief get smallest common denominator codecset
+ */
+void sccp_utils_reduceCodecSet(skinny_codec_t **baseCodecs, const skinny_codec_t reduceByCodecs[])
+{
+	uint8_t x = 0, y;
+	skinny_codec_t *base = *((skinny_codec_t (*)[SKINNY_MAX_CAPABILITIES])baseCodecs);
+	boolean_t found = FALSE;
+
+	while (x < SKINNY_MAX_CAPABILITIES && base[x] != SKINNY_CODEC_NONE) {
+		found = FALSE;
+		for (y = 0; y < SKINNY_MAX_CAPABILITIES && reduceByCodecs[y] != SKINNY_CODEC_NONE; y++) {
+			if (base[x] == reduceByCodecs[y]) {
+				found = TRUE;
+				break;
+			}
+		}
+		if (!found) {
+			memmove(base + x, base + (x + 1), (SKINNY_MAX_CAPABILITIES - (x + 1)) * sizeof(skinny_codec_t));  // move left
+			continue;
+		}
+		x++;
+	}
+	
+	baseCodecs = &base;
+}
+
+void sccp_utils_combineCodecSets(skinny_codec_t **baseCodecs, const skinny_codec_t addCodecs[])
+{
+	uint8_t x, y;
+	skinny_codec_t *base = *((skinny_codec_t (*)[SKINNY_MAX_CAPABILITIES])baseCodecs);
+	boolean_t found = FALSE;
+	
+	for (y = 0; y < SKINNY_MAX_CAPABILITIES && addCodecs[y] != SKINNY_CODEC_NONE; y++) {
+		found = FALSE;
+		for (x = 0; x < SKINNY_MAX_CAPABILITIES && base[x] != SKINNY_CODEC_NONE; x++) {
+			if (addCodecs[y] == base[x]) {
+				found = TRUE;
+				break;
+			}
+		}
+		if (!found && (x + 1 < SKINNY_MAX_CAPABILITIES)) {
+			memmove(base + (x + 1), base + x, (SKINNY_MAX_CAPABILITIES - (x + 1)) * sizeof(skinny_codec_t));  // move right
+			base[x] = addCodecs[y];
+		}
+	}
+	baseCodecs = &base;
+}
+
 #ifndef HAVE_PBX_STRINGS_H
 
 /*!
@@ -540,7 +620,6 @@ sccp_device_t *sccp_device_find_byipaddress(struct sockaddr_storage * sas)
 
 	SCCP_RWLIST_RDLOCK(&GLOB(devices));
 	SCCP_RWLIST_TRAVERSE(&GLOB(devices), d, list) {
-		//  if (d->session && IN6_ARE_ADDR_EQUAL(&d->session->sin,  sas) == 0 ) {
 		if (d->session && sccp_socket_cmp_addr(&d->session->sin, sas) == 0) {
 			d = sccp_device_retain(d);
 			break;
@@ -977,7 +1056,7 @@ sccp_msg_t *sccp_utils_buildLineStatDynamicMessage(uint32_t lineInstance, uint32
 	if (dummy_len) {
 		char buffer[dummy_len + padding];
 
-		//  memset(&buffer[0], 0, sizeof(buffer));
+		// memset(&buffer[0], 0, sizeof(buffer));
 		memset(&buffer[0], 0, dummy_len + padding);
 
 		if (dirNum_len) {
@@ -989,7 +1068,7 @@ sccp_msg_t *sccp_utils_buildLineStatDynamicMessage(uint32_t lineInstance, uint32
 		if (lineDisplayName_len) {
 			memcpy(&buffer[dirNum_len + FQDN_len + 2], lineDisplayName, lineDisplayName_len);
 		}
-		//  memcpy(&msg->data.LineStatDynamicMessage.dummy, &buffer[0], sizeof(buffer));
+		// memcpy(&msg->data.LineStatDynamicMessage.dummy, &buffer[0], sizeof(buffer));
 		sccp_log(DEBUGCAT_CORE) (VERBOSE_PREFIX_3 "LineStatDynamicMessage.dummy: %s\n", buffer);
 		memcpy(&msg->data.LineStatDynamicMessage.dummy, &buffer[0], dummy_len + padding);
 	}
@@ -1008,9 +1087,11 @@ sccp_msg_t *sccp_utils_buildLineStatDynamicMessage(uint32_t lineInstance, uint32
  */
 int socket_equals(struct sockaddr_storage *s0, struct sockaddr_storage *s1)
 {
-	//if (s0->sin_addr.s_addr != s1->sin_addr.s_addr || s0->sin_port != s1->sin_port || s0->sin_family != s1->sin_family) {
-	//  return 0;
-	//}
+	/*
+	if (s0->sin_addr.s_addr != s1->sin_addr.s_addr || s0->sin_port != s1->sin_port || s0->sin_family != s1->sin_family) {
+		return 0;
+	}
+	*/
 
 	//if (IN6_ARE_ADDR_EQUAL(s0, s1) && s0->ss_family == s1->ss_family ){
 	if (sccp_socket_cmp_addr(s0, s1) && s0->ss_family == s1->ss_family) {
@@ -1196,6 +1277,8 @@ skinny_codec_t sccp_utils_findBestCodec(const skinny_codec_t ourPreferences[], i
 	return 0;
 }
 
+
+
 /*!
  * \brief Free a list of Host Access Rules
  * \param ha The head of the list of HAs to free
@@ -1343,9 +1426,9 @@ int sccp_apply_ha_default(const struct sccp_ha *ha, const struct sockaddr_storag
 				continue;
 			}
 		}
-		//  char *straddr = ast_strdupa(sccp_socket_stringify_addr(&current_ha->netaddr));
-		//  char *strmask = ast_strdupa(sccp_socket_stringify_addr(&current_ha->netmask));
-		//  sccp_log(DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s:%s/%s\n", AST_SENSE_DENY == current_ha->sense ? "deny" : "permit", straddr, strmask);
+		// char *straddr = ast_strdupa(sccp_socket_stringify_addr(&current_ha->netaddr));
+		// char *strmask = ast_strdupa(sccp_socket_stringify_addr(&current_ha->netmask));
+		// sccp_log(DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s:%s/%s\n", AST_SENSE_DENY == current_ha->sense ? "deny" : "permit", straddr, strmask);
 
 		/* For each rule, if this address and the netmask = the net address
 		   apply the current rule */
@@ -1641,6 +1724,140 @@ void sccp_print_ha(struct ast_str *buf, int buflen, struct sccp_ha *path)
 	}
 }
 
+#if CS_TEST_FRAMEWORK
+AST_TEST_DEFINE(chan_sccp_ha_tests)
+{
+	struct sccp_ha *ha = NULL;
+	struct sockaddr_storage sas10, sas1015, sas172;
+	int error;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "chan_sccp_ha_tests";
+		info->category = "/channels/chan_sccp/ha/";
+		info->summary = "chan-sccp-b ha test";
+		info->description = "chan-sccp-b ha tests";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_test_status_update(test, "Executing chan-sccp-b ha path tests...\n");
+
+	ast_test_status_update(test, "Setting up sockaddr_storage...\n");
+	sccp_sockaddr_storage_parse(&sas10, "10.0.0.1", PARSE_PORT_FORBID);
+	if (!sccp_socket_is_IPv4(&sas10)) {
+		ast_test_status_update(test, "creating sas10 failed'\n");
+		goto RETURN_FAILURE;
+	}
+	sccp_sockaddr_storage_parse(&sas1015, "10.15.15.1", PARSE_PORT_FORBID);
+	if (!sccp_socket_is_IPv4(&sas1015)) {
+		ast_test_status_update(test, "creating sas1015 failed'\n");
+		goto RETURN_FAILURE;
+	}
+	sccp_sockaddr_storage_parse(&sas172, "172.16.0.1", PARSE_PORT_FORBID);
+	if (!sccp_socket_is_IPv4(&sas172)) {
+		ast_test_status_update(test, "creating sas172 failed'\n");
+		goto RETURN_FAILURE;
+	}
+
+	// test 1
+	ast_test_status_update(test, "test 1: ha deny all\n");
+	ha = sccp_append_ha("deny", "0.0.0.0/0.0.0.0", ha, &error);
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas10) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.0.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas1015) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.15.15.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas172) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '172.16.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+
+	// test 2
+	ast_test_status_update(test, "test 2: added permit 10.15.15.0/255.255.255.0\n");
+	ha = sccp_append_ha("permit", "10.15.15.0/255.255.255.0", ha, &error);
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas10) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.0.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas1015) == AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.15.15.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas172) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '172.16.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+
+	// test 3
+	ast_test_status_update(test, "test 3: added second permit 10.15.15.0/255.255.255.0\n");
+	ha = sccp_append_ha("permit", "10.15.15.0/255.255.255.0", ha, &error);
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas10) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.0.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas1015) == AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.15.15.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas172) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '172.16.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	sccp_free_ha(ha);
+	ha = NULL;
+
+	// test 4
+	ast_test_status_update(test, "test 4: deny all + permit 10.15.15.0/255.255.255.0\n");
+	ha = sccp_append_ha("deny", "0.0.0.0/0.0.0.0", ha, &error);
+	ha = sccp_append_ha("permit", "10.0.0.0/255.0.0.0", ha, &error);
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas10) == AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.0.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas1015) == AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.15.15.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas172) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '172.16.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+
+	// test 5
+	ast_test_status_update(test, "test 5: add deny_all at the end\n");
+	ha = sccp_append_ha("deny", "0.0.0.0/0.0.0.0", ha, &error);
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas10) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.0.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas1015) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '10.15.15.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+	if (sccp_apply_ha(ha, (struct sockaddr_storage *) &sas172) != AST_SENSE_DENY) {
+		ast_test_status_update(test, "sccp_apply_ha(ha, '172.16.0.1) failed'\n");
+		goto RETURN_FAILURE;
+	}
+
+	sccp_free_ha(ha);
+	ha = NULL;
+	return AST_TEST_PASS;
+
+RETURN_FAILURE:
+	if (ha) {
+		sccp_free_ha(ha);
+		ha = NULL;
+	}
+	return AST_TEST_FAIL;
+}
+
+#endif
+
 /*!
  * \brief Yields string representation from channel (for debug).
  * \param c SCCP channel
@@ -1648,7 +1865,7 @@ void sccp_print_ha(struct ast_str *buf, int buflen, struct sccp_ha *path)
  */
 const char *sccp_channel_toString(sccp_channel_t * c)
 {
-	if (c && c->designator) {
+	if (c) {
 		return (const char *) c->designator;
 	} else {
 		return "";
@@ -1822,13 +2039,6 @@ char *sccp_dec2binstr(char *buf, size_t size, int value)
 
 gcc_inline void sccp_copy_string(char *dst, const char *src, size_t size)
 {
-#ifdef CS_EXPERIMENTAL
-	/*alternative implementation */
-	//if (size > 1) {
-	//  *((char *)mempcpy(dst, src, size-1)) = '\0';
-	//} else {
-	//  *dst = '\0';
-	//}
 	if (size != 0) {
 		while (--size != 0) {
 			if ((*dst++ = *src++) == '\0') {
@@ -1836,14 +2046,7 @@ gcc_inline void sccp_copy_string(char *dst, const char *src, size_t size)
 			}
 		}
 	}
-	// always end string with \0
 	*dst = '\0';
-
-#else
-	if (size != 0) {
-		ast_copy_string(dst, src, size);
-	}
-#endif
 }
 
 /* 
@@ -1875,4 +2078,42 @@ char *sccp_trimwhitespace(char *str)
 	return str;
 }
 
+#if HAVE_ICONV_H
+gcc_inline boolean_t sccp_utils_convUtf8toLatin1(const char *utf8str, char *buf, size_t len) {
+	iconv_t cd;
+	size_t incount, outcount = len;
+	incount = strlen(utf8str);
+	if (incount) {
+		cd = iconv_open("ISO8859-1", "UTF-8");
+		if (cd == (iconv_t) -1) {
+			pbx_log(LOG_ERROR, "conversion from 'UTF-8' to 'ISO8859-1' not available.\n");
+			return FALSE;
+		}
+		if (iconv(cd, (void *) &utf8str, &incount, &buf, &outcount) == (size_t) -1) {
+			if (errno == E2BIG)
+				pbx_log(LOG_WARNING, "SCCP: Iconv: output buffer too small.\n");
+			else if (errno == EILSEQ)
+				pbx_log(LOG_WARNING,  "SCCP: Iconv: illegal character.\n");
+			else if (errno == EINVAL)
+				pbx_log(LOG_WARNING,  "SCCP: Iconv: incomplete character sequence.\n");
+			else
+				pbx_log(LOG_WARNING,  "SCCP: Iconv: error %d: %s.\n", errno, strerror(errno));
+		}
+		iconv_close(cd);
+	}
+	return TRUE;
+}
+#endif
+
+#if CS_TEST_FRAMEWORK
+void sccp_utils_register_tests(void)
+{
+        AST_TEST_REGISTER(chan_sccp_ha_tests);
+}
+
+void sccp_utils_unregister_tests(void)
+{
+        AST_TEST_UNREGISTER(chan_sccp_ha_tests);
+}
+#endif
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
